#!/usr/bin/env python
import sys
if not "/usr/bp/lib" in sys.path:
   sys.path.append("/usr/bp/lib")

class Cond:
  def __init__(self):
    self.days = 0
    self.start = 0
    self.end = 0
    self.setid = 0
    self.overview = False
    self.archives = False
    self.media = False
    self.label = ''
    self.serial= ''
    self.barcode = ''


import getpass
import subprocess
import uarch_textToHTML
import time
from datetime import datetime
from optparse import OptionParser


epochNow = int(time.time())
parser = OptionParser()
parser.add_option("-O", "--overview",   dest="overview", action='store_true',
		  help="generate an overview report on archive sets within the given time range")
parser.add_option("-d", "--days",   dest="days", default=0, action='store',
		  help="show all archive sets in the last given days, this option will be ignored if start/end dates are set")
parser.add_option("-s", "--start",   dest="start", action='store',
		  help="set the start date in mm-dd-yyyy format to show all archive sets")
parser.add_option("-e", "--end",   dest="end", action='store',
		  help="set the end date in mm-dd-yyyy format to show all archive sets")
parser.add_option("-A", "--archives",   dest="archives", action='store_true',
		  help="generate an detailed report of all archives in the given archive set")
parser.add_option("-i", "--id",   dest="setid", action='store',
		  help="the archive set id. It can be gained by running Archive Set Overview report")
parser.add_option("-M", "--media",   dest="media", action='store_true',
		  help="generate an report of what archive sets are on the given media.  Query condition can be media label, serial number and barcode (for tapes). Accepts partial pattern")
parser.add_option("-l", "--label",   dest="label", action='store',
		  help="the media label, you can input part of the serial to get more results")
parser.add_option("-n", "--serial",   dest="serial", action='store',
		  help="the media serial number, you can input part of the label to get more results")
parser.add_option("-b", "--barcode",   dest="barcode", action='store',
		  help="the tape barcode, you can input part of the barcode to get more results")
(opts, args) = parser.parse_args()

if hasattr(datetime, 'strptime'):
	#python 2.6+
	strptime = datetime.strptime
else:
	#python 2.4 equivalent
	strptime = lambda date_string, format: datetime(*(time.strptime(date_string, format)[0:6]))


def main():

	selected_action = 0
	if opts.overview:
		selected_action += 1
	if opts.archives:
		selected_action += 1
	if opts.media:
		selected_action += 1
	
	if selected_action > 1:
		sys.exit("Please select one report type at a time, please type -h for help")
	if selected_action < 1:
		sys.exit("Please select a report type, please type -h for help")
		
	if opts.overview:
		print("You have selected to do an Archive Set Overview report")
		archive_set_overview()
	elif opts.archives:
		print("You have selected to do an Archive Details report")
		archive_details()
	elif opts.media:
		print("You have selected to do a Media & ArchiveSet report")
		media_archive_sets()
		
	return


######################
def archive_set_overview():

	range_given,days_given,legend = False,False,[]
	overall_set_header=['archive set id', 'description', 'status','retention days', 'start time', \
				'target media', 'target slots', 'media label', 'elapsed time',\
				'compressed', 'encrypted', 'total size (MB)', 'total filecount']

	if ((not opts.start) or (not opts.end)) and (not opts.days):
		sys.exit("Please assign a date range, or assign a last day. You can find help with -h (--help) option")
	elif not ((not opts.start) or (not opts.end)):
		try:
			timeArray = time.strptime(opts.start, "%m-%d-%Y")
			Cond.start = int(time.mktime(timeArray))
			timeArray = time.strptime(opts.end, "%m-%d-%Y")
			Cond.end = int(time.mktime(timeArray))

			#Cond.start = strptime(opts.start, "%m-%d-%Y")
		except ValueError:
			sys.exit("Please assign a proper date range in mm-dd-yyyy format")
		else:
			if Cond.end < Cond.start:
				print ("Warning: The end date is smaller than the start date, please check the input")
			range_given = True
			legend.append("Start date is: " + opts.start)
			legend.append("End date is: " + opts.end)
	elif not (not opts.days):
		if opts.days.isdigit() :
			days_given = True
			legend.append("Date range: last " + opts.days + " days")
			Cond.days = opts.days
		else: 
			sys.exit("Please assign a proper last day value")
		
	condstr = ""
	if range_given:
		#print("date range is given, ignoring last days")
		condstr = " and creation_timestamp  between " + str(Cond.start) +  \
			" and " + str(Cond.end)
	else:
		#print("No proper date range given, using last days")
		condstr = " and now()::date - to_timestamp(creation_timestamp)::date  < " + Cond.days
		
        querystr = "psql bpdb -Upostgres -c 'select a.archive_set_id, p.description, a.status, p.retention_days,\
                         to_timestamp(creation_timestamp) as start_time, p.target, p.target_slots, a.media_label, a.elapsed_secs,\
                         p.compress, p.encrypt \
			 from bp.archive_sets a, bp.archive_profiles p \
                         where a.profile_id = p.profile_id"
        querystr += condstr
        orderstr = " order by start_time DESC;'"
        querystr += orderstr
	#print querystr
        process = subprocess.Popen(querystr, stdout=subprocess.PIPE, stderr=None, shell=True)
        i=0
	table1=[]
	table2=[]
	
        for line in process.stdout:
                if ("-----" in line) or ("archive_set" in line) or ("rows)" in line) or ("row)" in line) or (len(line) < 3):
                        continue
                else:
			rows=[]
                        for cols in line.split("|"):
				if cols.isspace():
					rows.append("none")
				else:
					rows.append(cols)
			table1.append(rows)
                        i+=1

	if i < 1:
		sys.exit("No archive set found from the input range")

        querystr = "psql bpdb -Upostgres -c 'select archive_set_id, to_timestamp(creation_timestamp) as start_time,\
			sum(file_mib_size) as total_size, sum(file_count) as file_count from bp.archive_sets \
			left join bp.archives using (archive_set_id) where 1=1"
        querystr += condstr
        querystr += " group by archive_set_id, start_time"
        querystr += orderstr

        process = subprocess.Popen(querystr, stdout=subprocess.PIPE, stderr=None, shell=True)
        for line in process.stdout:
                if ("-----" in line) or ("total_size" in line) or ("rows)" in line) or (len(line) < 3):
                        continue
                else:
			rows=[]
                        for cols in line.split("|"):
				if cols.isspace():
					rows.append("  0  ")
				else:
					rows.append(cols)
			table2.append(rows)
					

	table_data=[]
	for i in range(0, len(table1)):
		rows=[]
		line1 = table1[i]
		for j in range(0, len(line1) ):
			if j == 2:
				if "archive" in line1[j]:
					rows.append(line1[j][8:])
				else:
					rows.append(line1[j][:])
			elif j == 8:
				seconds = int(line1[j])
				m, s = divmod(seconds, 60)
				h, m = divmod(m, 60)
				rows.append("%d:%02d:%02d" % (h, m, s))
			else:
				rows.append(line1[j])
		line2 = table2[i]
		rows.append(line2[2])
		rows.append(line2[3])
		table_data.append(rows)

	HTMLFILE = '/var/www/html/recoveryconsole/reports.dir/archive_overview.html'
	#legend.append("Report is generated by: " + getpass.getuser())
	legend.append("Report generated time: " + datetime.now().strftime("%Y-%m-%d %I:%M:%S"))
	f = open(HTMLFILE, 'w')


	f.write('<center><font size="6">Archive Set Overview</font></center>\n')
	f.write('<HR WIDTH="100%" ALIGN="LEFT">');
	f.write('<HR WIDTH="100%" ALIGN="LEFT">\n');
	f.write('<p>')

	htmlcode = uarch_textToHTML.list(legend, ordered=False)
	f.write(htmlcode)
	f.write('<p>')


	htmlcode = uarch_textToHTML.table(table_data,header_row = overall_set_header)
	#print htmlcode
	f.write(htmlcode)
	f.write('<p>')

	print("Report is successfully generated, please visit this link to view: " +
		"https://" + get_sys_ip() + "/recoveryconsole/reports.dir/archive_overview.html") 

######################
def archive_details():

	legend=[]
	archive_details_header=['archive id', 'backup id', 'backup type', 'client name', \
				'backup time', 'original size (MB)', 'actual size (MB)', 'file count',\
				'compressed', 'encrypted', 'successful', 'error msg']

	if (not opts.setid): 
		sys.exit("Please assign an archive set id, you can find help with -h (--help) option")
	elif not opts.setid.isdigit():
		sys.exit("Please assign a proper archive set id")
	else:
		legend.append("Archive set id is: " + opts.setid)
		
	condstr = " and archive_set_id = " +  opts.setid
		
        querystr = "psql bpdb -Upostgres -c 'select archive_id, orig_backup_no, backtype, client_name, p.key1, p.key2, to_timestamp(orig_time), \
			orig_mib_size, file_mib_size, file_count, compressed, encrypted, archive_success, error_string \
			from bp.archives join bp.archive_instances ai using (archive_instance_id) join bp.application_instances p \
			 using (instance_id) where 1=1"

        querystr += condstr
        orderstr = " order by archive_id;'"
        querystr += orderstr
	#print querystr
        process = subprocess.Popen(querystr, stdout=subprocess.PIPE, stderr=None, shell=True)

	i, j, table_data = 0, 0, []
	
        for line in process.stdout:
                if ("-----" in line) or ("archive_id" in line) or ("rows)" in line) or ("row)" in line) or (len(line) < 3):
                        continue
                else:
			rows=[]
			rawdata=[]
			
                        for cols in line.split("|"):
				if cols.isspace():
					rawdata.append("none")
				else:
					rawdata.append(cols)

                        for j in range (0, len(rawdata)):
				backtype = int(rawdata[2])
				if j == 2:
					if backtype == 1: rows.append("Master")
					elif backtype == 2: rows.append("Differential")
					elif backtype == 3: rows.append("Incremental")
					elif backtype == 4: rows.append("Selective")
					elif backtype == 5: rows.append("Exchange full")
					elif backtype == 6: rows.append("Exchange diff")
					elif backtype == 7: rows.append("Exchange incr")
					elif backtype == 9: rows.append("MSSQL full")
					elif backtype == 10: rows.append("MSSQL diff")
					elif backtype == 11: rows.append("MSSQL incr")
					elif backtype == 12: rows.append("VMESX full")
					elif backtype == 13: rows.append("VMESX diff")
					elif backtype == 14: rows.append("VMESX incr")
					elif backtype == 15: rows.append("VMESX config")
					elif backtype == 1031: rows.append("HyperV full")
					elif backtype == 1032: rows.append("HyperV diff")
					elif backtype == 1033: rows.append("HyperV incr")
					elif backtype == 1034: rows.append("Oracle full")
					elif backtype == 1035: rows.append("Oracle incr")
					elif backtype == 1036: rows.append("Sharepoint full")
					elif backtype == 1037: rows.append("Sharepoint diff")
					elif backtype == 1038: rows.append("system metadata")
					else : rows.append("unknown")
				elif j == 3:
					if (backtype == 5) or (backtype == 6) or (backtype == 7) or \
					   (backtype == 1034) or (backtype == 1035) or \
					   (backtype == 1036) or (backtype == 1037) or \
					   (backtype == 1031) or (backtype == 1032) or (backtype == 1033):
						rows.append(rawdata[4])
					elif (backtype == 9) or (backtype == 10) or (backtype == 11) or \
					     (backtype == 12) or (backtype == 13) or (backtype == 14):
						rows.append(rawdata[5])
					else:
						rows.append(rawdata[j])
			
				elif (j == 4) or (j == 5):
					continue
				else:
					rows.append(rawdata[j])
			table_data.append(rows)
			i += 1
	if i < 1:
		sys.exit("The given archive set id has no data")

	HTMLFILE = '/var/www/html/recoveryconsole/reports.dir/archive_details.html'
	#legend.append("Report is generated by: " + getpass.getuser())
	legend.append("Report generated time: " + datetime.now().strftime("%Y-%m-%d %I:%M:%S"))
	f = open(HTMLFILE, 'w')


	f.write('<center><font size="6">Archive Details</font></center>\n')
	f.write('<HR WIDTH="100%" ALIGN="LEFT">');
	f.write('<HR WIDTH="100%" ALIGN="LEFT">\n');
	f.write('<p>')

	htmlcode = uarch_textToHTML.list(legend, ordered=False)
	f.write(htmlcode)
	f.write('<p>')


	htmlcode = uarch_textToHTML.table(table_data,header_row = archive_details_header)
	#print htmlcode
	f.write(htmlcode)
	f.write('<p>')

	print("Report is successfully generated, please visit this link to view: " +
		"https://" + get_sys_ip() + "/recoveryconsole/reports.dir/archive_details.html") 

######################
def media_archive_sets():

	media_set_header=['media name', 'media label', 'serial number',\
				'bar code', 'expiration date', 'reusable', 'archive sets']

	if (not opts.label) and (not opts.serial) and (not opts.barcode):
		sys.exit("Please provide some information about the media. You can find help with -h (--help) option")

	condstr, legend  = "", []
	lab, sn, bc = str(opts.label), str(opts.serial), str(opts.barcode)
	
	if opts.label:
		condstr += " and media_label like '%" + lab + "%'"
		legend.append("Input media label: " + lab)
			
	#if not sn.isspace():
	if opts.serial:
		condstr += " and serial_no like '%" + sn + "%'"
		legend.append("Input media serial: " + sn)

	#if not bc.isspace():
	if opts.barcode:
		condstr += " and barcode like '%" + bc + "%'"
		legend.append("Input tape barcode: " + bc)
	
	#print (condstr)
		
        querystr = '''psql bpdb -Upostgres -c "select p.target, a.media_label, d.serial_no, t.barcode, \
			to_timestamp(a.creation_timestamp + (p.retention_days + 1) * 86400)::date as expdate,\
			(case when ((a.creation_timestamp + (p.retention_days + 1) * 86400) < \
			FLOOR(EXTRACT(EPOCH FROM NOW()))) then 'yes' else 'no' end) as reusable, \
			a.archive_set_id from bp.archive_sets a join bp.archive_profiles p using (profile_id) \
			join bp.archive_disks d using (archive_set_id) left join bp.archive_tapes t \
			using (serial_no) where 1=1'''
        querystr += condstr
        orderstr = ' ;"'
        querystr += orderstr
	#print (querystr)
        process = subprocess.Popen(querystr, stdout=subprocess.PIPE, stderr=None, shell=True)
	
	i, table_data = 0, []
        for line in process.stdout:
                if ("-----" in line) or ("media_label" in line) or ("rows)" in line) or ("row)" in line) or (len(line) < 3):
                        continue
                else:
			rows=[]
                        for cols in line.split("|"):
				if cols.isspace():
					rows.append("none")
				else:
					rows.append(cols)
			table_data.append(rows)
			i += 1

	if i < 1:
		sys.exit("No data found from the input")
				
	HTMLFILE = '/var/www/html/recoveryconsole/reports.dir/media_archive_sets.html'
	#legend.append("Report is generated by: " + getpass.getuser())
	legend.append("Report generated time: " + datetime.now().strftime("%Y-%m-%d %I:%M:%S"))
	f = open(HTMLFILE, 'w')


	f.write('<center><font size="6">Media & ArchiveSet Report</font></center>\n')
	f.write('<HR WIDTH="100%" ALIGN="LEFT">');
	f.write('<HR WIDTH="100%" ALIGN="LEFT">\n');
	f.write('<p>')

	htmlcode = uarch_textToHTML.list(legend, ordered=False)
	f.write(htmlcode)
	f.write('<p>')


	htmlcode = uarch_textToHTML.table(table_data,header_row = media_set_header)
	#print htmlcode
	f.write(htmlcode)
	f.write('<p>')

	print("Report is successfully generated, please visit this link to view: " +
		"https://" + get_sys_ip() + "/recoveryconsole/reports.dir/media_archive_sets.html") 

######################
def get_sys_ip():
	command = "hostname -i"
        process = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=None, shell=True)
	
	addr = ''
        for line in process.stdout:
                if ("127.0.0" in line) or (len(line) < 3):
                        continue
                else:
			addr = line.rstrip()
	return addr	

if __name__ == '__main__':
    main()
