#!/bin/sh
## 5536 BT	check_space.sh created.
## 5711 BT	getProfileName() has been modified to handle numbers given that are < 1 or > #_of_profiles
## 5710 BT	getReport() has been modified to prompt whether you would like to check another profile.
## 5709 BT	changed wording for report
## 5699 BT	Verified that ARCHIVELOG is being set.  Also moved trap to after lockfile check.
## 5770 BT	check_space.sh now supports disk date range and tape profiles.
## 5807 BT	Fixed getReport().  If getProfile returns 1 then it will exit the check_space.sh script.
## 5805 BT	Wording change of more to additional in getReport().
## 5878 BT	Changed wording.
## 5978 BT      MountArchiveDisk() will not make the /archives path as necessary so that it may function properly.
## 5983 BT	getReport() will now verify that the tape drive exists.  If it does not then it will notify to stdout and continue to menu.
## 5990 BT      Fixed recent change to CheckBkupSize().  If isCheckingProfile=="y" then DiskSize is the size of the disk; otherwise, it is the
##              free space of the disk.
## 6007 BT 	Added wording and catch to prevent execution on a SFF.
## 6305 BT	Added wording for if archive media is MDA and the profile won't fit and there are more disks total than disks in units on the archive controller.
## 6437 BT	Added severely needed call to create a new user_data.rpt for sizing calculations.

. /etc/default/bp.ini > /dev/null 2>&1
. ${BPDIR}/bin/archive/constants.incl
. ${BPDIR}/bin/archive/common.incl
. ${BPDIR}/bin/archive/backup.incl
TwTarget="Unknown"
MAILTO=`${BPBIN}/bputil -g "Unattended Backup Information" "MailTo" "NULL" ${BPDIR}/bpinit/master.ini`
REPORT="${BPRPT}/archive_space_check.rpt"
ARCHIVEMOUNTED="False"
# 5807
QuietFlag="n" # Always interactive
# 5990
isCheckingProfile="y"

# 5710
stty sane
stty erase ^H


getProfileName(){
	#echo getProfileName
	clear
	TMPFILE="/tmp/tempfile.$$"
	cat /dev/null > ${TMPFILE}
	COUNT=1
	${ECHO} "Select a profile to check the archive size for:"
	for PROFILE in `ls ${BPDIR}/profiles.dir/*.spr 2> /dev/null`
	do
		${BPBIN}/bputil -g "Archive" "BackupMethod" "NULL" ${PROFILE} > /dev/null 2>&1
		if [[ ${?} == 0 ]]; then
			${ECHO} "\t${COUNT}. \c"	
			${ECHO} ${PROFILE} | tee -a ${TMPFILE}
			((COUNT=${COUNT}+1))
		fi
		PAGE=`expr $COUNT % 23`
		if [ "$PAGE" = "0" ]; then
			read -n 1 -p "---Press space to continue---"
			echo ""
		fi
	done
	((COUNT=${COUNT}-1))
	if [[ ${COUNT} == 0 ]]; then
		${ECHO} "There are no valid archive profiles."
		read
		# 5711
		rm -f "${TMPFILE}" 2> /dev/null
		return 1
	fi
	${ECHO} "Enter selection or 'q' to quit:  \c"
	read ANS
	echo ${ANS} | grep -q ^[0-9][0-9]*$
	if [[ ${?} == 0 ]]; then
		# 5711
		if [[ ${ANS} -gt ${COUNT} || ${ANS} -lt 1 ]]; then
			getProfileName
			# 5807
			return $?
		else
			ARCHIVEPROFILE=`head -n ${ANS} ${TMPFILE} | tail -n 1`
		fi
	else
		if [[ "${ANS}" == "q" || "${ANS}" == "Q" ]]; then
			return 1
		else
			getProfileName
			# 5807
			return $?
		fi
	fi
	return 0
}

filterClients2()
{
	crossReference
	TMP="/tmp/tempfile"
	mv -f ${ALIST} ${TMP}
	cat /dev/null > ${ALIST}
	ThisClient=`$BPBIN/bputil -g Clients Client1 xxx $ARCHIVEPROFILE`
	ClientCtr=1
	FCLIENTS=""
	while [ ! "$ThisClient" = "xxx" ]
	do
		FCLIENTS="$ThisClient $FCLIENTS"
		ClientCtr=`expr $ClientCtr + 1`
		ThisClient=`$BPBIN/bputil -g Clients Client$ClientCtr xxx $ARCHIVEPROFILE`
	done
	for CLIENT in ${FCLIENTS}
	do
		grep "|${CLIENT}|" ${TMP} >> ${ALIST}
	done
	rm -f ${TMP}
	return 0
}


getReport()
{
	# 5710
	exitret=0
	while true
	do
		# 5807
		ARCHIVEPROFILE=NULL
		getProfileName
		[ $? -eq 1 ] && break
		# 5807
		if [ "$ARCHIVEPROFILE" = "NULL" ]; then
			continue
		fi
		# 5770
		BkupType=`${BPDIR}/bin/bputil -g "Archive" "BackupType" "NULL" ${ARCHIVEPROFILE}`
		BkupMethod=`${BPDIR}/bin/bputil -g "Archive" "BackupMethod" "NULL" ${ARCHIVEPROFILE}`
		#ArchTarget=`${BPDIR}/bin/bputil -g "Archive" "TargetDevice" "NULL" ${ARCHIVEPROFILE}`
		# 5770
		if [ "$BkupType" = "Disk" ]
		then
			TwArchivingSupported && $ECHO "Checking disk profile..."
			sleep 1
			OldArchTarget=$ArchTarget
			GetDPUTypeAndArchTarget
			[ $? -eq 2 ] && break  # User wants to exit?
			if [ "$ArchTarget" != "$OldArchTarget" ] || [ "$ARCHIVEMOUNTED" = "False" ]; then
				# Need to mount the archive drive
				$ECHO "Attempting to mount the archive disk..."
				[ "$ARCHIVEMOUNTED" = "True" ] && RemoveArchiveDisk
				ARCHIVEMOUNTED="False"
				MountArchiveDisk $DPUType $ArchTarget NOSETPROFILE SILENTMODE
				[ $? -eq 0 ] && ARCHIVEMOUNTED="True"
			fi
		elif [ "$BkupType" = "Tape" ]
		then
			$ECHO "Checking tape profile..."
			sleep 1
		else
			continue
		fi

		# 5770
		if [ "$BkupMethod" = "Date" ]; then
			cat /dev/null > ${ALIST}
			START=$(date -d "`/usr/bp/bin/bputil -g "Archive" "ReadableStartDate" "NULL" ${ARCHIVEPROFILE}`" +%s)
			END=$(date -d "`/usr/bp/bin/bputil -g "Archive" "ReadableEndDate" "NULL" ${ARCHIVEPROFILE}`" +%s)
			filterClients2
		fi
		TMPFILE="/tmp/tempfile.$$"
		cat /dev/null > ${TMPFILE}
		$ECHO "`date`" > ${TMPFILE}
		$ECHO "\nSpace requirement to archive using:  ${ARCHIVEPROFILE}" >> ${TMPFILE}
		QuietFlag="y" TotalBkupSize >> ${TMPFILE}
		TotalCombinedData=`grep "Total Combined Data" /tmp/tempfile.$$ | awk ' { print $4 } '`
		DeviceCapacity=0
		if [ "$BkupType" = "Disk" ]
		then
			DeviceCapacity=`grep "Device Capacity" /tmp/tempfile.$$ | awk ' { print $3 } '`
			((SpaceNeeded=$TotalCombinedData-$DeviceCapacity))
			if [[ ${SpaceNeeded} -gt 0 ]]; then
				# 5709, 5805
				$ECHO "\nNeed $SpaceNeeded MB additional space to archive this profile to the current archive unit." >>${TMPFILE}
				# 6305
				if [ "$TwMDA" == "y" ] && TwArchivingSupported
				then
					NumPortsWithoutUnits=`tw_cli info $TwController | grep ^p[0-9] | grep -v NOT-PRESENT | awk ' { print $1"_"$3 } ' | grep -v _u | wc -l`
					if [[ "${NumPortsWithoutUnits}" -gt "0" ]]; then
						$ECHO "\nIf a new disk has been inserted for the purpose of adding it to the archive media\nthen run \"Setup Media\" from the Archiving section of the panel.  Then re-run the \"Profile Checker\"." >> ${TMPFILE}
					fi
				fi
			else
				((SpaceNeeded=0-${SpaceNeeded}))
				# 5709
				$ECHO "\n$SpaceNeeded MB of additional data may be archived to this archive unit." >> ${TMPFILE}
			fi
		else
			cat /dev/null > ${TMPFILE}
			$ECHO "`date`" > ${TMPFILE}
			$ECHO "\nSpace requirement to archive using:  ${ARCHIVEPROFILE}" >> ${TMPFILE}
			BackupDevice=`${BPDIR}/bin/bputil -g "Archive" "BackupDevice" "NULL" ${ARCHIVEPROFILE}`
			# 5983
			FOUNDDEVICE=`psql bpdb -U postgres --tuples-only -c "select * from bp.devices where dev_name = '${BackupDevice}'" 2> /dev/null | grep -v ^$`
			if [[ "${FOUNDDEVICE}" == "" ]]; then
				$ECHO "Tape device ${BackupDevice} is not found in the database."
				$ECHO "Press <Enter> to return to the menu."
				read
				continue
			fi
			DeviceCapacity=`psql bpdb -U postgres --tuples-only -c "select capacity from bp.devices where dev_name = '${BackupDevice}'" | grep -v ^$ | awk ' { print $1 } '`
			((DeviceCapacity=${DeviceCapacity}/1024))
			NUMTAPES=$((${TotalCombinedData}/${DeviceCapacity}))
			if [[ ${TotalCombinedData}%${DeviceCapacity} -gt 0 ]]; then
				((NUMTAPES=${NUMTAPES}+1))
			fi
			$ECHO "" >> ${TMPFILE}
			$ECHO "Total Data: ${TotalCombinedData} MB" >> ${TMPFILE}
			$ECHO "Device Capacity: ${DeviceCapacity} MB" >> ${TMPFILE}
			$ECHO "Expected Number of Tapes: ${NUMTAPES}" >> ${TMPFILE}
		fi
		$ECHO | tee -a ${TMPFILE}
		# 5878
		$ECHO "Note:  The Profile Checker checks whether or not a single profile" >> ${TMPFILE}
		$ECHO "       will fit on the archive media.  It is not a cumulative total" >> ${TMPFILE}
		$ECHO "       for multiple profiles." >> ${TMPFILE}
		# 5710
		rm -f ${REPORT}
		cat ${TMPFILE} > ${REPORT}
		rm -f ${TMPFILE}
		# 5710
		clear
		cat ${REPORT}
		$ECHO "\n\nPress <Enter> to return... \c"	
		read
		if [[ -s ${BPRPT}/archive_space_check.rpt && ${MAILTO} != "NULL" ]]; then
			mail -s "`hostname`: Check Archive Size - ${ARCHIVEPROFILE}" ${MAILTO} < ${REPORT}
		fi

		# 5710
		$ECHO "\nCheck another profile? [y/n] (Default: n) \c"
		read ANS
		ANS=`echo ${ANS} | tr [A-Z] [a-z]`
		if [[ "${ANS}" != "y" ]]; then
			break
		fi
		AllClientBkups=""
		TotalCombinedData=0
		DeviceCapacity=0
		SpaceNeeded=0
		clear
	done
	if [ "$ARCHIVEMOUNTED" = "True" ]; then
		$ECHO "Cleaning up /archives mount..."
		RemoveArchiveDisk
		[[ "$ArchTarget" =~ "3ware" ]] && unspec $TwTarget
	fi
	return 0
}

OperationInProgress="Profile Checker"
GetDPUType

ExitIfArchivingNotSupported
trap 'cleanUp $?' EXIT TERM INT
CreateLockOrExit

getReport

exit $?
