#!/usr/bin/php
<?php

$BPDIR = trim(shell_exec('eval `grep BPDIR /etc/default/bp.ini`; echo $BPDIR'));
$autoDR = "$BPDIR/profiles.dir/DR/.autoDR";
$preRestore = "PreRestore";
$error = "Error";


function write_to_status_file($section, $field, $value, $file) {
	global $BPDIR;
	$cmd = "$BPDIR/bin/bputil -p \"$section\" \"$field\" \"$value\" $file";
	shell_exec($cmd);
}

function exit_func($ret, $syncUser, $ip) {
	if ($ret) {
		global $BPDIR;
		global $count;
		global $status_file;
		$msg = "Raising alert and trap";
		$count++;
		write_to_status_file("PreRestore", "Message$count", $msg, $status_file);
		$msg1 = "$syncUser failed. For details, please see Disaster Recovery Log. Last failed on " . strftime('%a %b %d %I:%M:%S %p %Y');
		bp_send_notification(103, $syncUser, $msg1);
	}
	bp_destroy_cookie();
	exit;
}

function computeNextRun($timeArr, $frequency) {
	$year = $timeArr['tm_year'];
	$mday = $timeArr['tm_mday'];
	$month = $timeArr['tm_mon'] + 1;
	$days = $timeArr['tm_yday'];

	$totalDays = 364;
	if ($year%4 == 0) {
		$totalDays = 365; /* first day starts at 0 */
	} 
	$daysInMonth = 31;
	switch($month) {
		case 1:
		case 3:
		case 5:
		case 7:
		case 8:
		case 10:
		case 12:
			$daysInMonth = 31;
			break;
		case 4:
		case 6:
		case 9:
		case 11:
			$daysInMonth = 30;
			break;
		case 2:
			if ($totalDays = 365) {
				$daysInMonth = 29;
			} else {
				$daysInMonth = 28;
			}
			break;
	}
	$frequency = trim($frequency);
	if ($frequency === "daily") {
		if ($mday == $daysInMonth) {
			/* last day of month, change month, year */
			if ($month == 11) {
				$month = 0;
				$days = 0;
				$year++;
			} else {
				$days++;
				$month++;
			}
			$mday = 1;
		} else {
			$mday++;
			$days++;
		}
	} else if ($frequency === "weekly") {
		/* add 7 days to lastTime */
		if ($mday == $daysInMonth) {
			if ($month == 12) {
				$month = 1 ;
				$days = 6;
				$year++;
			} else {
				$days = $days + 7;
				$month++;
			}
			$mday = 7;
		} else {
			/* Not last day of month */
			$mday = $mday + 7;
			$days = $days + 7;
			if ($mday > $daysInMonth) {
				/* next run is moved to next month */
				$month++;
				if ($month > 12) {
					/* Move to next year and reset days*/
					$year++;
					$days = $mday - $daysInMonth - 1;
					$mday = $mday - $daysInMonth;
					$month = 1;
				} else {
					$mday = $mday - $daysInMonth;
				}
			} 
		}
	}
	$time = mktime($timeArr['tm_hour'], $timeArr['tm_min'], $timeArr['tm_sec'], $month, $mday, $year+1900);
	$nextTime = strftime('%Y%m%d%H%M%S', $time);
	return $nextTime;
}

function updateNextTime($syncUser, $ip) {
	/* Set lastTime and nextTime in .autoDR */
	$now = strftime('%Y%m%d%H%M%S');
	global $BPDIR;
	global $autoDR;
	global $count;
	global $status_file;

	/* nextTime in .autoDR ~= $now. Set it to last time */
	$lastTime = trim(shell_exec("$BPDIR/bin/bputil -g \"$syncUser\" \"nextTime\" \"\" $autoDR"));
	if ($lastTime === "") {
		/* do not fail here - lastTime will reflect incorrect value when profile loads.
		 * Still OK, as long as we run auto DR */
	}

	/* Find when the next run is. Hmm..from the cron? or autoDR? */
	$frequency = shell_exec("$BPDIR/bin/bputil -g \"$syncUser\" \"frequency\" \"\" $autoDR");
	$timeArr = strptime($now, '%Y%m%d%H%M%S');
	if ($timeArr == NULL) {
		$msg = "Cannot update next time. Invalid date/time format found";
		write_to_status_file("PreRestore", "Error$count", $msg, $status_file);
		exit_func(1, $syncUser, $ip);
	}
	$frequency = trim($frequency);
	if (strcmp($frequency, "daily") != 0 &&
		strcmp($frequency, "weekly") != 0) {
		$msg = "Invalid frequency set for automatic profile\n";
		write_to_status_file("PreRestore", "Error$count", $msg, $status_file);
		exit_func(1, $syncUser, $ip);
	}
	$nextTime = computeNextRun($timeArr, $frequency);
	$cmd = "$BPDIR/bin/bputil -p \"$syncUser\" \"lastTime\" $now $autoDR";
	shell_exec($cmd);
	$cmd = "$BPDIR/bin/bputil -p \"$syncUser\" \"nextTime\" $nextTime $autoDR";
	shell_exec($cmd);
}

/* 
 *
 * Main program starts here
 * usage autoDR.php <syncUser> <targetIP>
 *
 */

/* This is a secret option used while saving the profile 
 * if start_time < current_time */
if ($argc == 4 && $argv[1] === "getNextRun") {
	$timeArr = strptime($argv[2], '%Y%m%d%H%M%S');
	if ($timeArr == NULL) {
		echo "false";
		exit;
	}
	$nextTime = computeNextRun($timeArr, $argv[3]);
	echo $nextTime;
	exit;
}

if (isset($argv[1]) &&  isset($argv[2])) {
	$syncUser = $argv[1];
	$ip       = $argv[2];
} else {
	echo "Invalid arguments. Sync user and target IP address are required.\n";
	exit_func(1, "notFound", "notFound");
}

$res = bp_bypass_cookie(3);
if ($res === false) {
	echo "Failed to bypass cookie\n";
	var_dump(bp_error());
	exit_func(1, $syncUser, $ip);
}

$dpuName = bp_get_dpu_name($syncUser);
if ($dpuName === false) {
	echo "Appliance name not found for ", $syncUser;
	exit_func(1,$syncUser, $ip);
}

$status_file = "$BPDIR/reports.dir/$dpuName" . "_RestoreStatus.ini"; 
$count = 1;

updateNextTime($syncUser, $ip);
write_to_status_file($preRestore, "syncUser", $syncUser, $status_file);
write_to_status_file($preRestore, "target", $ip, $status_file);
write_to_status_file($preRestore, "startTime", strftime('%c'), $status_file);


$res = bp_dr_in_progress($syncUser, $ip);
if ($res === false) {
	$msg =    strftime('%c') . "Error occured while checking for another restore process";
	write_to_status_file($preRestore, $error.$count, $msg, $status_file);
	$count++;
	write_to_status_file($preRestore, $error.$count, bp_error(), $status_file);
	exit_func(1, $syncUser, $ip);
}

if ($res === "yes") {
	$msg =    strftime('%c') . " Restore in progress for $syncUser or  $ip ";
	write_to_status_file($preRestore, $error.$count, $msg, $status_file);
	exit_func(1, $syncUser, $ip);
}

/* do not unlink status file. If everything goes fine, restore_state API will do that.*/

/* Resolve/Close previously open alerts */
bp_send_notification(105, $syncUser);

$profile_info = bp_load_auto_dr_profile($syncUser, $ip);
if ($profile_info === false) {
	$msg =   strftime('%c') . " Failed to load profile:";
	write_to_status_file($preRestore, $error.$count, $msg, $status_file);
	$count++;
	write_to_status_file($preRestore, $error.$count, bp_error(), $status_file);
	exit_func(1, $syncUser, $ip);
} 
if (!$profile_info['profile_present']) {
	$msg =    strftime('%c') . " Profile for  $syncUser , $ip not found ";
	write_to_status_file($preRestore, $error.$count, $msg, $status_file);
	exit_func(1, $syncUser, $ip);
}

$profile = $profile_info['profile_info'];

/* Check if user and ip from the profile match arguments.
 * If they don't, we will be running some other profile. Exit */
if ($syncUser === $profile['sync_user'] && 
	$ip === $profile['target_ip']) {
	$syncUser = $profile['sync_user'];
	$ip = $profile['target_ip'];
	$dpuName = $profile['dpu_name'];
} else {
	$msg =    strftime('%c') . " Profile found for " . $profile['sync_user'] . " and ".  $profile['target_ip'] . " (No matching sync user or IP address) ";
	write_to_status_file($preRestore, $error.$count, $msg, $status_file);
	exit_func(1, $syncUser, $ip);
}

$res = bp_save_storage_preference($profile['replace_storage'], $syncUser, $dpuName);
if ($res === false) {
	$msg =    strftime('%c') . " Failed to save storage preference: ";
	write_to_status_file($preRestore, $error.$count, $msg, $status_file);
	$count++;
	write_to_status_file($preRestore, $error.$count, bp_error(), $status_file);
	exit_func(1, $syncUser, $ip);
}

$res = bp_save_auto_dr_preference($profile['auto_dr'], $syncUser, $dpuName);
if ($res === false) {
	$msg =    strftime('%c') . " Failed to save auto DR preference: ";
	write_to_status_file($preRestore, $error.$count, $msg, $status_file);
	$count++;
	write_to_status_file($preRestore, $error.$count, bp_error(), $status_file);
	exit_func(1, $syncUser, $ip);
}

$res = bp_save_encryption_preference($profile['check_encryption'], $syncUser, $dpuName);
if ($res === false) {
	$msg =    strftime('%c') . " Failed to save encryption preference: ";
	write_to_status_file($preRestore, $error.$count, $msg, $status_file);
	$count++;
	write_to_status_file($preRestore, $error.$count, bp_error(), $status_file);
	exit_func(1, $syncUser, $ip);
}

/* Just get a list of clients, this will extract dpu_clients, fix permissions etc */
$res = bp_get_dpu_clients($syncUser, $dpuName);
if ($res === false) {
	$msg =    strftime('%c') . " Failed to read client information from vaulted appliance state ";
	write_to_status_file($preRestore, $error.$count, $msg, $status_file);
	exit_func(1, $syncUser, $ip);
}

/* Get client list */
$clientDevs = $profile['clients_devices'];
foreach ($clientDevs as $clientDev) {
	$clients[] = $clientDev['client_name'];
}

/* Check if saved-client is available in the new list */
foreach ($clients as $client) {
	if (!in_array($client, $res)) {
		$msg =  "Client '" . $client . "' is not available for restore from " . $syncUser . " ";
		write_to_status_file($preRestore, $error.$count, $msg, $status_file);
		$count++;
		$msg =  "Please recreate automatic Disaster Recovery profile to remove old clients. ";
		write_to_status_file($preRestore, $error.$count, $msg, $status_file);
		exit_func(1, $syncUser, $ip);
	}
}

/* Check target version */
$res = bp_dpu_restore_vault_check_version($syncUser, $dpuName, $ip);
if ($res === false) {
	$msg =    strftime('%c') . " Failed to check target version: ";
	write_to_status_file($preRestore, $error.$count, $msg, $status_file);
	$count++;
	write_to_status_file($preRestore, $error.$count, bp_error(), $status_file);
	exit_func(1, $syncUser, $ip);
}

/* Call rewrite_client_metadata if storage pref = true, 
 * i.e target storage is selected. */
if ($profile['replace_storage']) {
	$res = bp_rewrite_client_metadata($profile['clients_devices'], $syncUser, $dpuName);
	if ($res === false) {
		$msg =    strftime('%c') . " Failed to rewrite client metadata: ";
		write_to_status_file($preRestore, $error.$count, $msg, $status_file);
		$count++;
		write_to_status_file($preRestore, $error.$count, bp_error(), $status_file);
		exit_func(1, $syncUser, $ip);
	}
	/* Check target size before restoring state */
	$res = bp_vault_validate_devices($syncUser, $dpuName, $ip, $clients);
	if ($res === false) {
		$msg =    strftime('%c') . " Failed to validate devices: ";
		write_to_status_file($preRestore, $error.$count, $msg, $status_file);
		$count++;
		write_to_status_file($preRestore, $error.$count, bp_error(), $status_file);
		exit_func(1, $syncUser, $ip);
	}
}

$msg =  " Restoring appliance state to ". $ip;
/* Restore state. Encryption is checked and will be running when this 
 * returns true*/
$res = bp_dpu_restore_vault_state($syncUser, $dpuName, $ip);
if ($res === false) {
	$msg =    strftime('%c') . " Failed to restore state: ";
	write_to_status_file($preRestore, $error.$count, $msg, $status_file);
	$count++;
	write_to_status_file($preRestore, $error.$count, bp_error(), $status_file);
	exit_func(1, $syncUser, $ip);
}

$msg =    strftime('%c') . " State restore complete";
write_to_status_file($preRestore, "Message$count", $msg, $status_file);
$count++;

$msg =    strftime('%c') . " Queueing jobs for client restore";
write_to_status_file($preRestore, "Message$count", $msg, $status_file);
$count++;

$res = bp_dpu_restore_clients($syncUser, $dpuName, $ip, $clients);
if ($res === false) {
	$msg =    strftime('%c') . " Failed to queue jobs to restore clients:";
	write_to_status_file($preRestore, $error.$count, $msg, $status_file);
	$count++;
	write_to_status_file($preRestore, $error.$count, bp_error(), $status_file);
	exit_func(1, $syncUser, $ip);
}
$msg =    strftime('%c') . " Successfully queued jobs to restore client backups";
write_to_status_file($preRestore, "Message$count", $msg, $status_file);
exit_func(0, $syncUser, $ip);

