#!/usr/bin/php

<?php

// This program can be run standalone -- it generates the report to the
// file $BPDIR/reports.dir/failure_report.html and sends the email.  It is
// normally run by a cron job every hour to find and report any failures
// that appeared in the schedule_history table over the last hour (based on
// the time the status was last updated, not the scheduled event time).

$true_value_array = array(0 => "true", 1 => "True", 2 => "TRue", 3=> "TRUe", 4=> "TRUE", 5=>"1", 
					6 => "On", 7=> "ON", 7=> "on", 9=> "Yes", 10=> "YEs", 11=> "YES", 
					12=> "yes", 13=> "y", 14=> "Y");

$debug=false; // set this to true to get debug printout
$dbconn_string = "port=5432 dbname=bpdb user=postgres";
$bpdir = trim(shell_exec('eval `grep BPDIR /etc/default/bp.ini`; echo $BPDIR'));

// The mail_report function
require("$bpdir/bin/mail_report.php");

// These are app-specific column headings for the object names
$objcol = array(
	"Exchange" => "Storage Group or Database Name",
	"SQL Server" => "SQL Server Instance/Database Name",
	"VMware" => "ESX Server/Virtual Machine",
	"Hyper-V" => "Virtual Machine",
	"Archive" => "Archiving Job",
	"file-level" => "Client Name",
	"Oracle" => "Oracle Instance",
	"SharePoint" => "Sharepoint Farm",
	"UCS Service Profile" => "UCS Service Profile",
	"Volume" => "Volume",
	"Xen" => "Xen Virtual Machine");

$dbconn = pg_connect($dbconn_string);
if($dbconn === FALSE) {
	print "ERROR: Failed to connect to the database.\n";
	die(1);
}

// Get the schedules that should be in the report
$query = "SELECT schedule_id, s.name AS sched_name, s.description, ";
$query .= "al.name AS app_name, al.type AS app_type ";
$query .= "FROM bp.schedules AS s ";
$query .= "JOIN bp.application_lookup AS al USING(app_id) ";
$query .= "WHERE enabled=true AND failure_report=true ";
$query .= "ORDER BY s.name";
if ($debug) print $query."\n";
$res = pg_query($dbconn, $query);
if ($res === false) {
	print "ERROR: failed to access schedules table:\n";
	print pg_last_error($dbconn)."\n";
	die(1);
}

// Don't generate the report unless there is at least one schedule with
// reporting enabled.
if (pg_num_rows($res) == 0) {
	print "There are no schedules with the Failure Report enabled.\n";
	exit(1);
}

// Get the System name and version for the general info section.
$query = "SELECT name, version FROM bp.systems WHERE is_local_system=true";
$vres = pg_query($dbconn, $query);
if ($vres === false) {
	print "ERROR: failed to retrieve version from systems table:\n";
	print pg_last_error($dbconn)."\n";
	die(1);
}
$vers = pg_fetch_result($vres, 0, "version");
$dpu = pg_fetch_result($vres, 0, "name");
pg_free_result($vres);

$tmpfile = tempnam($bpdir."/reports.dir", "failrpt");
if (!$fp = fopen($tmpfile, "w")) {
	echo "An error occurred opening $tmpfile for writing.\n";
	die(1);
}
if ($debug) print "temporary file name is: ".$tmpfile."\n";

$nschedules = 0;
$nfailed = 0;

$now = time();
// one hour ago;
$begin = $now - 3600;
// strip the minutes/seconds to get the hour (if the report is running some
// minutes or seconds after the hour, we want to start reporting at the top
// of the hour).  e.g., it's now 9:12=>since 8:00
$begin = round($begin/3600) * 3600 ;
if ($debug) {
	// set begin to oldest status_time to get all
	$begin = 1291240806;
	printf("=== now=%d (%s), begin=%d (%s)\n",
	$now, date("r", $now), $begin, date("r", $begin));
}

bp_bypass_cookie(3, 'failure_report');
$useHtml = bp_get_ini_value('Unattended Backup Information',
		'NotificationsUseHtml');

$useHtml = in_array($useHtml, $true_value_array);

if($useHtml == true){
	/*------------------------------------------------------
	 * Define the CSS styles, which are strings inserted into tags below.
	 * The php variables help clean up the style stuff for the tags below
	-------------------------------------------------------- */

	$idFooter = ' style="font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif;"'; 
	$classCell = ' style="font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif;' .
			' border:1px solid #bbbbbb; font-size:.8em;padding:.2em 0" ';
	$idMain = ' style="font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif;' .
			' margin:0 auto 0;" ';
	$tagHR = ' style="margin:3px 10px; border-color:#cccccc;border-style:solid;border-width:1px 0 0; clear:both; height:0;" ';
	$classBody = ' style="font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif;' . 
			'font-size:1em;letter-spacing:1px;background-color:#bbbbbb; margin:.3em;" align="left"';
	$classColhead = ' style=" font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif;' .
			' letter-spacing: 1px; background-color: #bbbbbb; margin: .2em; font-size: 1em;" ';
	$classSubtabletitle = ' style="padding:.3em 1.5em;font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial,' . 
			' sans-serif;letter-spacing:1px;background-color:#bbbbbb;" ';
	$tagH3 = ' style="font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif; font-size: 1.2em;" ';
	$classFooterp = ' style="text-align:left;font:.9em;font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif;" ';
	$classTitlecell = ' style="font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif;' .
			' letter-spacing:1px; background-color:#999999; 720px; padding:1.5em 3em;text-align: center" ';
	$tagTable = ' style="border-collapse:collapse;" ';
	$superTableWidth = 'width="720px" style="width:720px"';
	$bgColorGray = 'bgcolor="#bbbbbb" ';
	$colSpan2 = 'colspan="2"';
	$emptyCell = 'bgcolor=\"#ffffff\" style=\"background-color:#fff;padding:.25em;font-size:1px;\"';
} else {
	/*-----------------------------------------------------
	 * Remove CSS styles and replace values with simple HTML mode
	-------------------------------------------------------*/
	$idFooter = '';
	$classCell = '';
	$idMain = '';
	$tagHR = ' ';
	$classBody = ' ';
	$classColhead = ' ';
	$classSubtabletitle = ' ';
	$tagH3 = ' ';
	$classFooterp = '';
	$classTitlecell = ' ';
	$tagTable = '';
	$superTableWidth = ' width="100%" ';
	$bgColorGray = ' ';
	$colSpan2 = 'colspan="2"';
	$emptyCell = '';
}



// Loop through the schedules and get event details from schedule_history
while ($sched = pg_fetch_assoc($res)) {

	// query to get failed events for the schedule
	$query  = "SELECT sh.type, status_string, job_no, backup_no, ";
	$query .= "scheduled_start_time, key1, key2, key3, key4, node_name ";
	$query .= "FROM bp.schedule_history AS sh ";
	$query .= "JOIN bp.application_instances ai USING(instance_id) ";
	$query .= "JOIN bp.nodes AS n ON (ai.node_no=n.node_no) ";
	$query .= "WHERE schedule_id = ".$sched["schedule_id"];
	$query .= " AND queued IN (0,2) AND status_time >= ".$begin;
	$query .= " ORDER BY status_time";
	if ($debug) print $query."\n";

	$sres = pg_query($dbconn, $query);
	if ($sres === false) {
		print "ERROR: failed to access schedule_history table:\n";
		print pg_last_error($dbconn)."\n";
		die(1);
	}
	if (pg_num_rows($sres) == 0) {
		// There are no failures during this time period for this schedule.
		pg_free_result($sres);
		continue;
	}
	$nschedules++;
	$client = pg_fetch_result($sres,0,"node_name");

	// Construct the schedule information section.
	$str  = '	  <table ' . $superTableWidth . ' >' . "\n";
	$str .= "      <tr >\n";
	$str .= "			<td $classSubtabletitle $bgColorGray align=\"left\" >\n";
	$str .= "				<p><strong>Failures for schedule '".$sched["sched_name"]."'</strong></p>\n";
	$str .= "			</td>\n";
	if ($sched["app_type"] != "file-level") {
		$str .="        <tr>\n";
		$str .="          <td $classBody $bgColorGray ><strong>Client name:  </strong>".$client."</td>\n";
		$str .="        </tr>\n";
	}
	$str .="        <tr>\n";
	$str .="          <td $classBody $bgColorGray  ><strong>Application name:  </strong>".$sched["app_name"]."</td>\n";
	$str .="        </tr>\n";
	$str .="        <tr>\n";
	$str .="          <td $classBody $bgColorGray ><strong>Description:  </strong>".$sched["description"]."</td>\n";
	$str .="        </tr>\n";
	$str .="        <tr>\n";
	$str .="          <td $emptyCell>&nbsp;</td>\n";
	$str .="        </tr>\n";
	$str .="      </table>\n";

	fprintf($fp, "%s", $str);
	$str = "";

	// These are the column headings for the table of failures that
	// occurred during the reporting period (last hour).
	$colhead = "   <table $superTableWidth>" . "\n";
	$colhead .="      <thead>\n";
	$colhead .="        <tr>\n";
	$colhead .="          <td $classColhead $bgColorGray width=\"30%\"><strong>".$objcol[$sched["app_type"]]."</strong></td>\n";
	// For Archive schedules, we don't currently have linkage between the
	// data in the archive tables and the schedule_history to provide some
	// kind of identifier for the user to look for on the Archive Status
	// page.  The closest correspondence would be the scheduled_start_time,
	// since that will likely be nearly the same as the creation_timestamp
	// in the archive_sets table.  This is the timestamp that is displayed
	// in brackets next to the "Archive Set" label on the status page.  The
	// failed item in that set will be marked with a red X icon, so that
	// will hopefully make it identifiable.
	if ($sched["app_type"] == "Archive") {
		$colhead .="          <td $classColhead $bgColorGray width=\"15%\"><strong>Archive Set</strong></td>\n";
	}
	else {
		$colhead .="          <td $classColhead $bgColorGray width=\"15%\"><strong>Backup Id</strong></td>\n";
	}
	$colhead .="          <td $classColhead $bgColorGray width=\"15%\"><strong>Backup Type</strong></td>\n";
	$colhead .="          <td $classColhead $bgColorGray width=\"40%\"><strong>Status</strong></td>\n";
	$colhead .="        </tr>\n";
	$colhead .="      </thead>\n";
	// The column titles
	fprintf($fp, "%s", $colhead);
	$colhead = "";

	// The rows correspond to each failure.
	while ($row = pg_fetch_assoc($sres)) {
		// Construct the row values for each backup.
		// The instance name is composed of "keyN" values.
		// Add code here to handle new application-specific name types.
		if ($sched["app_type"] == "file-level") {
			$name = $row["node_name"];
		}
		elseif ($sched["app_type"] == "Archive") {
			$name = "Job ".$row["job_no"];
		}
		else {
			$name = $row["key1"];
		}
		if ($sched["app_type"] == "SQL Server") {
			$name .= "/".$row["key2"];
		}
		if ($sched["app_type"] == "VMware") {
			$name .= ", VM ".$row["key2"];
		}
		if ($sched["app_type"] == "UCS Service Profile") {
			$name = $client;
			$name .= "/".$row["key1"];
		}
		if ($sched["app_type"] == "Volume") {
			$name = $client;
			$name .= "/".$row["key1"];
		}
		if ($sched["app_type"] == "Xen") {
			$name .= ", VM ".$row["key2"];
		}

		// Backup types -- these are the LCST_* values
		switch ($row["type"]) {
			case 1:
				$bkup_type = "Master";
				break;
			case 5:
			case 10:
			case 14:
			case 18:
				$bkup_type = "Full";
				break;
			case 2:
			case 6:
			case 11:
			case 15:
			case 19:
				$bkup_type = "Differential";
				break;
			case 7:
			case 16:
			case 17:
			case 20:
				$bkup_type = "Incremental";
				break;
			case 12:
				$bkup_type = "Transaction Log";
				break;
			case 13:
				$bkup_type = "Archive";
				break;
			case 3:
				$bkup_type = "Selective";
				break;
			case 4:
				$bkup_type = "BareMetal";
				break;
			case 21:
				$bkup_type = "Oracle Full";
				break;
			case 22:
				$bkup_type = "Oracle Incremental";
				break;
			case 23:
				$bkup_type = "SharePoint Full";
				break;
			case 24:
				$bkup_type = "SharePoint Differential";
				break;
			case 25:  // LCST_RAE_FULL
				if ($sched["app_type"] == "UCS Service Profile") {
					$bkup_type = "UCS Service Profile Full";
				} else {
					$bkup_type = "Unknown";
				}
				break;
			case 26:  //LCST_RAE_DIFF
			case 27:  // LCST_RAE_INCR
				$bkup_type = "Unknown";
				break;
			case 28:
				$bkup_type = "NDMP Full";
				break;
			case 29:
				$bkup_type = "NDMP Differential";
				break;
			case 30:
				$bkup_type = "NDMP Incremental";
				break;
			default:  //just in case
				$bkup_type = "Unknown";
				break;

		}

		// In case there are no keys, use job type as the name
		if (!isset($name)) {
			$name = $bkup_type;
		}


		// Set the Backup Id.  For Archive schedule items, the column
		// (Archive Set) displays the start time, which is "close" to the
		// time displayed in the RRC to identify an Archive Set.
		if ($sched["app_type"] == "Archive") {
			$bkup_no = date("m/d g:iA", $row["scheduled_start_time"]);
		}
		else {
			$bkup_no = $row["backup_no"];
		}
		// It's possible there isn't a backup number,
		// for example, if the job failed to queue.
		if (is_null($bkup_no)) {
			$bkup_no = "N/A";
		}

		$status = $row["status_string"];
		if (is_null($status)) {
			$status = "Failed to queue the job.";
		}
		$nfailed++;

		// Compose the row with the values
		$str = "        <tr $idFooter>\n";
		$str .="          <td $classCell>".$name."</td>\n";
		$str .="          <td $classCell>".$bkup_no."</td>\n";
		$str .="          <td $classCell>".$bkup_type."</td>\n";
		$str .="          <td $classCell>".$status."</td>\n";
		$str .="        </tr>\n";

		// print the row for this backup
		fprintf($fp, "%s", $str);
		$str = "";
	}
	// close the last table and print the divider
	$tmp = "	</table>\n";
	if($useHtml == true){
		$tmp .= '	<table ' . $superTableWidth . '> <tr> <td style="padding:.5em 0;"> ';
		$tmp .= '	<div align="center"> <img src="http://www.unitrends.com/reports/notifications/hr.png"> <div> </td> </tr> </table>' . "\n";
	} else {
		$tmp .= '	<hr />';
	}
	fprintf($fp, "%s", $tmp);
	$tmp = "";
	
	
	pg_free_result($sres);
}


$from = "Failure Report@".$dpu;
// Footer identifies appliance name and asset tag.
if($useHtml == true){
	$str = "	<table width=\"720px\" $idFooter>\n";
		$str .= '			<tr style="border:0;">' . "\n";
		$str .= '				<td style ="padding:.4em;"> ' . "\n";
	$asset = trim(shell_exec($bpdir . "/bin/cmc_command run 'Asset Tag'"));
	if (isset($asset) && !empty($asset)){
		$str .= '					<p style="color:#555555;font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif; text-align:left;font-size:80%;word-spacing:1px;"> ' . "\n";
		$str .= '						Asset Tag: ' . $asset . '</p>' . "\n";
	}
		$str .= '					<p style="color:#555555;font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif; text-align:left;font-size:80%;word-spacing:1px;"> ' . "\n";
		$str .= '						Failure Report generated: ' . date("F j, Y, g:i a") . "\n";
		$str .= '					</p>' . "\n";
		$str .= '					<p style="color:#555555;font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif; text-align:left;font-size:80%;word-spacing:1px;"> ' . "\n";
		$str .= '						Report preferences can be set through the user interface via <strong>Settings -> Clients, Networking and Notifications </strong>' . "\n";
		$str .= '					</p>' . "\n";
		$str .= '					<p style="color:#555; font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif; text-align:left;font-size:80%;word-spacing:1px;"> ' . "\n";
		$str .= '						Additional resources are available at the following:' . "\n";
		$str .= '					</p>';
		$str .= '						<ul style="display:block;">' . "\n";
		$str .= '							<li style="color:#555;font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif; text-align:left;font-size:80%;word-spacing:1px;"><a href="http://www.unitrends.com/documents/appliance-agent-user-manual.pdf">Manual</a></li>' . "\n";
		$str .= '							<li style="color:#555;font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif; text-align:left;font-size:80%;word-spacing:1px;"><a href="http://support.unitrends.com/ikm/">Knowledge Base</a></li>' . "\n";
		$str .= '							<li style="color:#555;font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif; text-align:left;font-size:80%;word-spacing:1px;"><a href="http://www.unitrends.com/support/customer-care-center/video-tutorials.html">Training Videos</a></li>' . "\n";
		$str .= '						</ul>' . "\n";
		$str .= '				</td>' . "\n";
		$str .= '				<td style = "padding:.2em;"> ' . "\n";
		$str .= '					<p style="color:#555;font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif; text-align:left;font-size:80%;"> Connect with us. <br />' . "\n";
		$str .= '						<a href="http://community.spiceworks.com/pages/unitrends"><img float="right"src="http://www.unitrends.com/reports/spiceworks.jpg"';
		$str .= '						border="0" alt="Spiceworks" width="42" height="42" style="margin: 3px 3px 0px 3px;" /></a><a href="http://www.facebook.com/unitrends">';
		$str .= '						<img float="right"src="http://www.unitrends.com/reports/facebook.jpg" border="0"alt="Be our friends on Facebok" width="42" height="42" style="margin:';
		$str .= ' 					3px 3px 0px 3px;" /></a><br /><a href="http://www.linkedin.com/company/25406?trk=tyah"><img float="right"src="http://www.unitrends.com/reports/linkd-in.jpg" border="0" alt="Linked In"width="42" height="42" style="margin: 3px 3px 0px 3px;" /></a><a href="http://twitter.com/#!/Unitrends"><img float="right"src="http://www.unitrends.com/reports/twitter.jpg" border="0" alt="Twitter" width="42" height="42" ';
		$str .= '						style="margin: 3px 3px 0px 3px;" /></a><a href="http://www.youtube.com/user/UnitrendsInc"><img float="right"src="http://www.unitrends.com/reports/you-tube.jpg" border="0"alt="You Tube" width="42" height="42" style="margin: 3px 3px 0px 3px;" /></a>' . "\n";
		$str .= '					</p>';
		$str .= '			</tr>' . "\n";
	$str .= '		</table>';
} else{
	$asset = trim(shell_exec($bpdir . "/bin/cmc_command run 'Asset Tag'"));
	if (isset($asset) && !empty($asset)){
		$str = '					<p align="left"><strong> System Version: </strong>' . $vers . ', <strong>Asset Tag:</strong> ' . $asset . '</p>' . "\n";
	}
}
	fprintf($fp, "%s", $str);

fprintf($fp, "  </div></body>\n</html>\n");
fclose($fp);

if ($nschedules == 0) {
	// Either there were no events processed, or no failures occurred.
	echo "No failures occurred during the report period.\n";
	unlink($tmpfile);
	die(0);
}

$rptfile = $bpdir . "/reports.dir/failure_report.html";
if (!$rptfp = fopen($rptfile, "w")) {
	echo "An error occurred opening $rptfile for writing.\n";
	die(1);
}

// Construct the report general info section.
$fstr = "<span style = \"color:#c00000\" color=\"#c00000\"><strong><em>There ";
if ($nfailed == 1) {
	$fstr .= "was one failure.</em></span>";
}
else {
	$fstr .= "were ".$nfailed." failures.</em></span>";
}


//Generate HTML Header, top table with logo, title, System name, number of failures, etc.
$str ='<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"	"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd"><html xmlns="http://www.w3.org/1999/xhtml"> ' . "\n" . '<head><meta type="text/css"></meta>' . "\n";
$str .="    <title>SCHEDULE FAILURE REPORT</title>\n";
$str .="  </head>\n";
$str .="  <body>\n";
if($useHtml == true){
	$str .="    <div align=\"center\" >";
	$str .= '		<table style="border:0; border-collapse: collapse; border-spacing: 0;" '. $superTableWidth . 'cellpadding="0" cellspacing="0">'. "\n";
	$str .= '			<tr style="border:0;"> ' . "\n";
	$str .= '				<td style="border:0;padding:1.5em 2em;"> ' . "\n";
	$str .= '						<img src="http://www.unitrends.com/reports/color/logo.png" align="left" width="209px" height="37px">' . "\n";
	$str .= ' 				</td>' . "\n";
	$str .= '			</tr>';

	$str .= '	  <tr >' . "\n";
	$str .= '		<td ' . $colSpan2 . 'style="margin:0px; border:0px;"> <img src="http://www.unitrends.com/reports/notifications/roundedrectangletop.png" width="720px" height="26px" style="vertical-align:bottom;margin:0px 0px -2px;padding:0px;float:left;"></td>';
	$str .="      </tr>\n";

$str .="      <tr>\n";
$str .="        <td $colSpan2 bgcolor=\"#999999\" style=\"background-color:#999999; padding:0 3em 1.5em;\">\n";
$str .='         <div align="center"> <strong><span style="color:#333333;font-family: Calibri, Candara, Segoe, \"Segoe UI\", Optima, Arial, sans-serif; font-size:14px;">SCHEDULE FAILURE REPORT</span></strong></div>' . "\n";
$str .="        </td>\n";
$str .="      </tr>\n";
} else {
	$str .='	<table ' . $superTableWidth . ' style="border-collapse: collapse; border-spacing:0;">' . "\n";
	$str .="      <tr>\n";
	$str .="        <td $colSpan2 >\n";
	$str .='         <div align="center"> <strong><p>SCHEDULE FAILURE REPORT</p></strong></div>' . "\n";
	$str .="        </td>\n";
	$str .="      </tr>\n";
}
$str .="      <tr>\n";
$str .="        <td $colSpan2 $bgColorGray $classBody >Schedule events that failed in the hour prior to:  ";
$str .="          <strong>".date("F j, Y, g:i a")."</strong></td>\n";
$str .="      </tr>\n";

$str .="      <tr >\n";
$str .="        <td $colSpan2 $bgColorGray $classBody >System name:  <strong>".$dpu."</strong></td>\n";
$str .="      </tr >\n";

$str .="      <tr >\n";
$str .="        <td $colSpan2 $bgColorGray $classBody >".$fstr."</td>\n";
$str .="      </tr>\n";
if($useHtml == true){
	$str .="	  <tr > \n";
	$str .='		 <td ' . $colSpan2 . 'style="padding:.5em 0;"><div align="center"> <img src="http://www.unitrends.com/reports/notifications/hr.png"> </div></td>' . "\n";
	$str .="	  </tr>\n";
	$str .="    </table>\n";
} else {
	$str .="	</table>\n";
	$str .="	<hr />";
}



fprintf($rptfp, "%s", $str);
fclose($rptfp);


// read the temporary file and append to the report
$ret = file_put_contents($rptfile, file_get_contents($tmpfile), FILE_APPEND);

// clean up the temporary file
unlink($tmpfile);

bp_bypass_cookie(3, 'failure_report');
$report_addr = bp_get_ini_value('Unattended Backup Information',
		'FailureReportMailto');
if ($report_addr !== false && !empty($report_addr)) {
	$report = file_get_contents($rptfile);
	if (isset($report)) {
		$subject = $dpu." Schedule Failure Report ";
		
		// create PDF if needed
		$systemID = bp_get_local_system_id();
		$pdfFilename = $dpu."-ScheduleFailureReport.pdf";
		$pdfFile = createPDF($systemID, $rptfile, $pdfFilename);
		
		// empty array of images
		$img = array();
		mail_report($from, $report_addr, $subject, $report, $img, $pdfFile);
		
		clearTempReports($pdfFile);
	}
}
else {
	printf("No email recipients are configured (FailureReportMailto in master.ini).\n");
}




bp_destroy_cookie();
printf ("Schedule Failure Report processed %d schedules and found %d failure(s).\n",
$nschedules, $nfailed);
exit(0);

// Returns full path to PDF file upon success
// Returns false upon failure or if PDF is not to be created
function createPDF($systemID, $htmlFilePath, $pdfFilename) {
	// load pdf conversion class
	$rootdir = "/var/www/html/recoveryconsole";
	$pdfScript = $rootdir . '/system/pdf_conversion.php';
	// if this method is called more than one time - it will fail without require_once
	require_once $pdfScript;
	$pdfConversion = new PDFConversion();
	
	// create PDF file if necessary
	$result = $pdfConversion->createPDF($systemID, $htmlFilePath, $pdfFilename);
	return $result;
}

// Deletes PDF reports created
function clearTempReports($pdfFile) {
	if ($pdfFile !== false && file_exists($pdfFile)) {
		unlink($pdfFile);
	}	
}

?>
