#!/usr/bin/php
<?php
// This program can run standalone -- it generates the VM Verification
// Report and attaches the screen dump image.  It is normally invoked by
// the restore daemon when a new screen dump is created.

$bpdir = trim(shell_exec('eval `grep BPDIR /etc/default/bp.ini`; echo $BPDIR'));
$dbconn_string = "port=5432 dbname=bpdb user=postgres";

// The mail_report function
require("$bpdir/bin/mail_report.php");

// Main program starts here.
if ($argc < 3) {
	fwrite(STDERR, "Usage: ".basename($argv[0])." <virtual_id> <filename>\n");
	fwrite(STDERR, "Filename is the full pathname of the screen dump jpeg to attach.\n");
	exit(1);
}

$virtual_id = (int)$argv[1];
$attachment = $argv[2];

if (!is_readable($attachment)) {
	fwrite(STDERR, "File $attachment is not readable.\n");
	exit(1);
}

$dbconn = pg_connect($dbconn_string);
if($dbconn === FALSE) {
	fwrite(STDERR, "Failed to connect to the database.\n");
	exit(1);
}

// Create an array mapping app_id to application name.
$res = pg_query($dbconn, "SELECT app_id, name from bp.application_lookup");
if ($res === false) {
	$msg = "Failed to access application_lookup table: ";
	$msg .= pg_last_error($dbconn);
	fwrite(STDERR, $msg);
	exit(1);
}
while ($row = pg_fetch_assoc($res)) {
	if ($row["name"] == "file-level") {
		$app_names[$row["app_id"]] = "File-level";
	}
	else {
		$app_names[$row["app_id"]] = $row["name"];
	}
}
pg_free_result($res);

// All BPL API calls occur between here and the call to bp_destroy_cookie()
bp_bypass_cookie(3);

$report_addr = bp_get_ini_value('Unattended Backup Information', 'MailTo');
if ($report_addr === false) {
	$msg = bp_error();
	fwrite(STDERR, "Could not determine email address for reports: $msg\n");
	bp_destroy_cookie();
	exit(1);
}

if (empty($report_addr)) {
	fwrite(STDERR, "No email recipients are configured for the Unitrends Appliance list.\n");
	bp_destroy_cookie();
	exit(1);
}

$ret = bp_get_last_virtual_restore($virtual_id);
if ($ret === false) {
	$msg = bp_error();
	fwrite(STDERR, "Failed to obtain info for last restores report: $msg\n");
	bp_destroy_cookie();
	exit(1);
}

if (empty($ret)) {
	fwrite(STDERR,"No virtual restores have been performed for this client.\n");
	bp_destroy_cookie();
	exit(1);
}

// End of BPL API calls.
bp_destroy_cookie();

exec("uname -n", $hostname, $retval);
$hostname = $hostname[0];
$client = $ret[0]["cname"];
if ($ret[0]["grandclient"] == true) {
		$client .= " (replicated from ". $ret[0]["system_name"] .")";
}

// Failover Virtual Client Recovery Verification Report
$str = "<HTML>\n";
$str .= "  <HEAD>\n";
$str .="    <TITLE>VIRTUAL CLIENT RECOVERY VERIFICATION REPORT</TITLE>\n";
$str .="  </HEAD>\n";
$str .="  <BODY>\n";
$str .="    <TABLE width=\"100%\">\n";
$str .="      <TR>\n";
$str .="        <TD colspan=\"2\" style=\"text-align: center\">\n";
$str .="          <B>VIRTUAL CLIENT RECOVERY VERIFICATION REPORT</B>";
$str .="        </TD>\n";
$str .="      </TR>\n";
$str .="      <TR>\n";
$str .="        <TD>Client name:  <B>".$client."</B></TD>\n";
$str .="      </TR>\n";
$str .="      <TR>\n";
$str .="        <TD>System name:  <B>".$hostname."</B></TD>\n";
$str .="      </TR>\n";
$str .="      <TR>\n";
$str .="        <TD>To keep the Instant Recovery client up to date, the backups of the file system and applications are replayed on the Instant Recovery client. This report shows the current up-to-date status of the client.</TD>\n";
$str .="      </TR>\n";
$str .="    </TABLE>\n";
$str .="    <HR>\n";
// end of head area of report

$str .="    <TABLE width=\"100%\">\n";
$str .="    <THEAD>\n";
$str .="        <TR>\n";
$str .="          <TD width=\"50%\"><U>Component</U></TD>\n";
$str .="          <TD width=\"20%\"><U>Application</U></TD>\n";
$str .="          <TD width=\"30%\"><U>Up-to-date Time</U></TD>\n";
$str .="        </TR>\n";
$str .="    </THEAD>\n";
// end of column headings

// Construct the rows for each "last" virtual restore.
foreach ($ret as $item) {
	$str .= "        <TR>\n";

	// Component column
	if ($item["instance_name"] == "file-level") {
		$str .="          <TD>File System</TD>\n";
	}
	else {
		$str .="          <TD>".$item["instance_name"]."</TD>\n";
	}

	// Application column
	$str .="          <TD>".$app_names[$item["app_id"]]."</TD>\n";

	// Up-to-date column
	$timestr = date('m/d/Y h:i A', $item["start_time"]);
	$str .="          <TD>".$timestr."</TD>\n";

	// End of table row
	$str .="        </TR>\n";
}

$str .="    </TABLE>\n";
$str .= "    <HR>\n";

// Footer identifies appliance name and asset tag.
$str .= "<P>This email was sent from ". $hostname;
$asset = trim(shell_exec($bpdir . "/bin/cmc_command run 'Asset Tag'"));
if (isset($asset) && !empty($asset)) {
	$str .=", Asset Tag ". $asset;
}
$str .=".</P>\n";

// The End
$str.="  </BODY>\n</HTML>\n";

$from = "Instant Recovery Verification@".$hostname;
$subject = "Recovery Verification Report for $client";
$images = array();
$fname = basename($attachment);
$images[] = array("filename" => $attachment, "cid" => $fname ."@". $hostname,
		"type" => 'jpeg', "attach" => $fname);

$ret = mail_report($from, $report_addr, $subject, $str, $images);
if ($ret == FALSE) {
	fwrite(STDERR, "Failed to queue verification email for $client.\n");
	exit(1);
}
else {
	exit(0);
}

?>

