#!/usr/bin/perl

BEGIN {
}
use strict;
use warnings;
use DBI;
use IO::Handle;
use Sys::Hostname;
use Fcntl ':mode';

$\ = "\n";
$, = ' ';

if ($#ARGV < 1) {
	my $name = $0;
	$name =~ s/.*\/([^\/]*)/$1/ if $name =~ /\//;
	print "Usaege: ".$name." <command> <dumpPath>";
	exit 1;
}

my ($command, $path) = (shift, shift);

my $index = 0;
print STDERR "Performing action $command";

if (!$command) {
	print "Error: command must be provided";
	exit 1;
}
if (!$path) {
	print "Error: path to dump file must be provided";
	exit 1;
}
if (! -e $path) {
	print "Error: cannot open path $path";
	exit 1;
}
if ($command eq 'checkDevices') {
	&checkDevices();
} elsif ($command eq 'checkNodes') {
	&checkNodes();
}

print STDERR "Finished processing command $command";
exit 0;

sub checkDevices() {
	#First get what we have in dbdump.sql
	if (! open(FILE, "$path")) {
		print "Cannot open state db dump $path"; 
		exit 1;
	}

	# Read through the dump until we get to the application_instances table
	my $section = '';
	my @ldlines;
	my $found = 0;
	my $allFound = 0;
	my @dumpEntries;
	my $end = "/\/.";
	while (<FILE>) {
		chomp;
		my @singleEntry;
		if (!$found && m/^COPY devices /) {
			$found = 1; next;
		}
		next if !$found;

		if (/^\s*$/ || $_ eq '\.') {
			# We reached end of table
			$allFound = 1;
			close FILE;
			last;
		} else {
			#@print "Line: $_\n";
			# This is the one, return the 2, 15, 29, 30, 48,49fields
			my @fields = split ('\t' );
		
		#	print "@fields";
		if ($fields[1] =~ /^\\N/) {
			$fields[1] = '';
		}
		if ($fields[14] =~ /^\\N/) {
			$fields[14] = '';
		}
		if ($fields[28] =~ /^\\N/) {
			$fields[28] = '';
		}
		if ($fields[29] =~ /^\\N/) {
			$fields[29] = '';
		}
		if ($fields[47] =~ /^\\N/) {
			$fields[47] = '';
		}
		if ($fields[48] =~ /^\\N/) {
			$fields[48] = '';
		}
		@singleEntry = ($fields[1], $fields[14], $fields[28],$fields[29], $fields[47], $fields[48]);
		#print "@singleEntry";
		push @dumpEntries, [ @singleEntry ];
		}
	}
	if (!$allFound) {
		print "devices not found in dbdump";
		close FILE;
		exit 1;
	}

	# Get existing device info list 
	my $device_name;
	my $tape_no;
	my $dev_nr_name;

	my $sql = 'SELECT dev_name, dev_nr_name, tape_no FROM bp.devices WHERE device_type=128';
	my $connStr = "dbi::Pg:host=localhost dbname=bpdb user=postgres";
	my $user = 'postgres';
	my $db = 'bpdb';
	my $dbConn = DBI->connect("dbi:Pg:dbname=$db", $user, '');
	if (!defined $dbConn) {
		print "Error: cannot connect to database";
		exit 1;
	}

	my $result = $dbConn->prepare($sql);

	my $resultSql = $result->execute();
#	unless( $result->resultStatus == PGRES_TUPLES_OK) {
	if (!defined $resultSql) {
		print "Error: No devices found";
		$result->finish();
		$dbConn->disconnect();
		exit 1;
	}
	# Start loop to get all deviceInfo */
	my $nrows = $result->rows();
	my @row;

	my $returnStr = "";		
	while( ($device_name, $dev_nr_name, $tape_no) = $result->fetchrow_array()) {
		# Check if this tuple is the same from what we got 
		# from dbdump.sql
		my $devFound = 0;

		for my $i ( 0 .. $#dumpEntries) {
			my @item = $dumpEntries[$i]; 
				if ($item[0] eq $device_name) {
					# See if dev_nr_name and tape_no  match
					if ($item[1] eq $dev_nr_name &&
						$item[2] eq $tape_no) {
						next;
					} else {
						if (!length($returnStr)) {
							$returnStr = "\'$device_name\'";
						} else {
							$returnStr = $returnStr . ",\'$device_name\'";
						}
					}
					$devFound = 1;
					last;
				}
		}
		#Finished searching devices from dump
		if (!$devFound) {
			if (!length($returnStr)) {
				$returnStr = "\'$device_name\'";
			} else {
				$returnStr = $returnStr . ",\'$device_name\'";
			}
		}
	}
	$result->finish();
	$dbConn->disconnect();
	my $len = length($returnStr);
	if ($len) {
		print $returnStr;
		exit 1;
	}
}

sub checkNodes() {
	#First get what we have in dbdump.sql
	if (! open(FILE, "$path")) {
		print "Cannot open state db dump $path"; 
		exit 1;
	}

	# Read through the dump until we get to the application_instances table
	my $section = '';
	my @ldlines;
	my $found = 0;
	my $allFound = 0;
	my @dumpEntries;
	my $end = "/\/.";
	while (<FILE>) {
		chomp;
		my @singleEntry;
		if (!$found && m/^COPY nodes /) {
			$found = 1; next;
		}
		next if !$found;

		if (/^\s*$/ || $_ eq '\.') {
			# We reached end of table
			$allFound = 1;
			close FILE;
			last;
		} else {
			#@print "Line: $_\n";
			#Need first and second ones	
			my @fields = split ('\t' );
		
		#	print "@fields";
		if ($fields[0] =~ /^\\N/) {
			$fields[0] = '';
		}
		if ($fields[1] =~ /^\\N/) {
			$fields[1]= '';
		}
		@singleEntry = ($fields[1], $fields[0]);
		#print "@singleEntry";
		push @dumpEntries, [ @singleEntry ];
		}
	}
	if (!$allFound) {
		print "nodes not found in dbdump";
		close FILE;
		exit 1;
	}

	# Get existing clients 
	my $node_name;
	my $node_no;
	my $gcname;
	my $props;

	#Excluding system client
	my $sql = 'SELECT node_no, node_name, gcname, properties FROM bp.nodes WHERE options != 1 OR (properties & 32) != 0 ORDER by node_no';
	my $connStr = "dbi::Pg:host=localhost dbname=bpdb user=postgres";
	my $user = 'postgres';
	my $db = 'bpdb';
	my $dbConn = DBI->connect("dbi:Pg:dbname=$db", $user, '');
	if (!defined $dbConn) {
		print "Error: cannot connect to database";
		exit 1;
	}

	my $result = $dbConn->prepare($sql);

	my $resultSql = $result->execute();
	if (!defined $resultSql) {
		print "Error: No devices found";
		$result->finish();
		$dbConn->disconnect();
		exit 1;
	}
	# Start loop to get all deviceInfo */
	my $nrows = $result->rows();
	my @row;

	my $returnStr = "";		
	while( ($node_no, $node_name, $gcname, $props) = $result->fetchrow_array()) {
		# Check if this tuple is the same from what we got 
		# from dbdump.sql
		my $nodeFound = 0;
		#print $node_name;

		for my $i ( 0 .. $#dumpEntries) {
			my @item = $dumpEntries[$i]; 
				if ($item[0] eq $node_name) {					# See if node_no  matches
					if ($item[1] eq $node_no) {
						next;
					} else {
						if (!length($returnStr)) {
							if (($props & 32) != 0) {
								$returnStr = "\'$gcname\'";
							} else {
								$returnStr = "\'$node_name\'";
							}
						} else {
							if (($props & 32) != 0) {
								$returnStr = $returnStr . ",\'$gcname\'";
							} else {
								$returnStr = $returnStr . ",\'$node_name\'";
							}
						}
					}
					$nodeFound = 1;
					last;
				}
		}
		#Finished searching devices from dump
		if (!$nodeFound) {
			if (!length($returnStr)) {
				if (($props & 32) != 0) {
					$returnStr = "\'$gcname\'";
				} else {
					$returnStr = "\'$node_name\'";
				}
			} else {
				if (($props & 32) != 0) {
					$returnStr = $returnStr . ",\'$gcname\'";
				} else {
					$returnStr = $returnStr . ",\'$node_name\'";
				}
			}
		}
	}
	$result->finish();
	$dbConn->disconnect();
	my $len = length($returnStr);
	if ($len) {
		print $returnStr;
		exit 1;
	}
}
