#!/bin/sh -u

# Prepares archive media

. /etc/default/bp.ini 2> /dev/null
. ${BPDIR}/bin/uarchive-scripts/constants.incl

. ${BPDIR}/bin/uarchive-scripts/capabilities.incl
. ${BPDIR}/bin/uarchive-scripts/media_3ware.incl
. ${BPDIR}/bin/uarchive-scripts/media_removable.incl
. ${BPDIR}/bin/uarchive-scripts/mount.incl


device="$1" # "3ware", "3ware MDA", "Recovery Archive", or device node or external 
label="$2"

echo "Preparing device $device for archives with label \"$label\"" 1>&2

# Prepare device
if [[ "$device" = "3ware MDA" ]]; then
	Identify3WareMedia || exit 1
	if [ "$TwMediaFound" != "y" ] || [ "$TwMDA" = "n" ]; then
		echo "The archiving system did not find a 3ware MDA unit connected to the system" | $DUP
		exit 1
	fi
	Build3WareArchive "$label"
	ret=$?
elif [[ "$device" = "3ware" ]]; then
	Identify3WareMedia || exit 1
	if [ "$TwMediaFound" != "y" ] || [ "$TwMDA" = "y" ]; then
		echo "The archiving system did not find any removable 3ware drives in the system" | $DUP
		exit 1
	fi
	Build3WareArchive "$label"
	ret=$?
elif [[ "$device" = "Recovery Archive" ]]; then
	FindArchiveTargets
	PrepareRemovableArchiveDevice "$device" "$label"
	ret=$?
elif [[ "$device" = "external" ]]; then
	# Don't create any file system. Remove everything except .asset and .unitrends_serial
	if [ $# -ne 3 ]; then
		echo "The archiving system did not find the mount path for external device connected to the system" | $DUP
		exit 1
	fi
	pushd $3 >/dev/null
	# Write label 
	`echo $2 > $3/.label`
	`rm -rf *`
	if [ $? -ne 0 ]; then 
		echo "Error preparing external media for use"
		exit 1
	fi
	popd >/dev/null 
	ret=$?
else
	FindArchiveTargets
	if [ ! -e "$device" ]; then
		echo "The archiving system did not find device \"$device\" connected to the system" | $DUP
		exit 1
	fi
	PrepareRemovableArchiveDevice "$device" "$label"
	ret=$?
fi


[ "$ret" -ne 0 ] && echo "Failed to prepare archive media for use" | $DUP
[ "$ret" -eq 0 ] && echo "Successfully prepared archive device \"$device\" for use"
exit $ret



#TODO fix SignArchiveMedia
###############################################################################
# Assumes archive media mounted and first parameter contains 
# name of directory that we're archiving to.
# Places archive software version info and DPU Asset # on the backup media.
SignArchiveMedia()
{
	archiveDirectory=$1
	echo $ARCHIVE_VERSION > $archiveDirectory/$ARCHIVE_VERSION_FILE
	cat /etc/unitrends-asset > /archives/.asset
}

