<?php

class JobsHistory
{
    private $BP;

    public function __construct($BP, $sid, $Functions)
    {
        $this->BP = $BP;
        $this->now = time();
		$this->sid = $sid;
	    $this->itemCount = 0;

        $this->functions = $Functions;

        require_once('jobs.functions.php');
        $this->jobsFunctions = new JobsFunctions($this->BP);
    }


    public function get_jobs_history($which, $filter, $sid)
    {
        $allJobs = array();
        $startDate = isset($_GET['start_date']) ? strtotime($_GET['start_date']) : strtotime(Constants::DATE_ONE_WEEK_AGO);
        $endDate = isset($_GET['end_date']) ? $this->functions->formatEndDate($_GET['end_date']) : strtotime('now');
        $failed = isset($_GET['result']) ? $_GET['result'] : null;
        $includeOnDemand = (isset($_GET['include_on_demand']) and $_GET['include_on_demand'] === 'false') ? false : true;
        $includeDisabled = (isset($_GET['include_disabled']) and $_GET['include_disabled'] === 'false') ? false : true;
        $includeSystemMetadata = (isset($_GET['include_metadata']) and $_GET['include_metadata'] === 'true') ? true : false;

        //did they pass in sid, cid, or iid?
        if ($sid !== false) {
            $historyFilter = array(
                'start_time' => $startDate,
                'end_time' => $endDate,
                'include_on_demand' => $includeOnDemand,
                'include_disabled' => $includeDisabled
            );

            $name = $this->functions->getSystemNameFromID($sid);
            $jobHistory = $this->BP->get_schedule_history($historyFilter, $sid);
            $replicationJobHistory = $this->BP->get_replication_job_history_info(array_merge($historyFilter, array('system_id' => $sid)));

            if ($jobHistory !== false) {
                foreach ($jobHistory as $history) {
                    $data = $this->buildHistoryOutput($history, $name, $sid, null, $which, $startDate, $failed);
                    $filtered = $this->buildReturn($data, $filter);
                    if ($filtered != NULL) {
                        $allJobs[] = $filtered;
                    }
                }
            } else {
                // This will fail for any un-managed source.  Need to filter them out of $systems list, then this can be uncommented.
                // global $Log;
                //$Log->writeError("job history:" . $this->BP->getError(), true);
            }

            $replicationHistoryArray = array();
            if ($replicationJobHistory !== false) {
                foreach  ($replicationJobHistory as $replicationHistory){
                    if (!$includeSystemMetadata && isset($replicationHistory['instance_name']) &&
                        $replicationHistory['instance_name'] == "System Metadata") {
                        continue;
                    }
                    $data = $this->buildReplicationHistoryOutput($replicationHistory, $sid, $startDate, $failed);
                    if (!empty($data)) {
                        $replicationHistoryArray[] = $data;
                    }
                }
            } else {
                // This will fail for any un-managed source.  Need to filter them out of $systems list, then this can be uncommented.
                // global $Log;
                //$Log->writeError("replication job history:" . $this->BP->getError(), true);
            }

            $allJobs = array_merge($allJobs, $replicationHistoryArray);

        } else {
            $systems = $this->functions->selectSystems();
            foreach ($systems as $sid => $name) {
                $historyFilter = array(
                    'start_time' => $startDate,
                    'end_time' => $endDate,
                    'include_on_demand' => $includeOnDemand,
                    'include_disabled' => $includeDisabled
                );

                $jobHistory = $this->BP->get_schedule_history($historyFilter, $sid);
                $replicationJobHistory = $this->BP->get_replication_job_history_info(array_merge($historyFilter, array('system_id' => $sid)));

                $systemHistory = array();
                if ($jobHistory !== false) {
                    foreach ($jobHistory as $history) {
                        $data = $this->buildHistoryOutput($history, $name, $sid, null, $which, $startDate, $failed);
                        if (!empty($data)) {
                            $systemHistory[] = $data;
                        }
                    }
                } else {
                    // This will fail for any un-managed source.  Need to filter them out of $systems list, then this can be uncommented.
                    // global $Log;
                    //$Log->writeError("job history:" . $this->BP->getError(), true);
                }
                $allJobs = array_merge($allJobs, $systemHistory);

                $replicationHistoryArray = array();
                if ($replicationJobHistory !== false) {
                    foreach  ($replicationJobHistory as $replicationHistory){
                        if (!$includeSystemMetadata && isset($replicationHistory['instance_name']) &&
                            $replicationHistory['instance_name'] == "System Metadata") {
                            continue;
                        }
                        $data = $this->buildReplicationHistoryOutput($replicationHistory, $sid, $startDate, $failed);
                        if (!empty($data)) {
                            $replicationHistoryArray[] = $data;
                        }
                    }
                } else {
                    // This will fail for any un-managed source.  Need to filter them out of $systems list, then this can be uncommented.
                    // global $Log;
                    //$Log->writeError("replication job history:" . $this->BP->getError(), true);
                }

                $allJobs = array_merge($allJobs, $replicationHistoryArray);
            }
        }
        return $allJobs;
    }

    public function buildHistoryOutput($jobData, $sysName, $sid, $clientName,$which, $historyStartDate = null, $failed = null)
    {
        $data = array();
        $hasRun = false;

        $scheduleName = isset($jobData['schedule_name']) ? $jobData['schedule_name'] : null;
        // Do this as the core is not returning duration.  Format duration as a string.
        $duration = isset($jobData['start_time']) ? $this->functions->formatTimeDelta($this->now - $jobData['start_time']) : "n/a";
        $appName = isset($jobData['application_name']) ? $jobData['application_name'] : "";
        $appType = isset($jobData['app_type']) ? $jobData['app_type'] : "";
        $scheduleDescription = isset($jobData['schedule_description']) ? $jobData['schedule_description'] : "";
        $failures = isset($jobData['failures']) ? $jobData['failures'] : "";
        $status = ((int)($failures) > 0) ? 'Error' : 'Successful';
        $historyStartDate = $this->functions->formatDateTime($historyStartDate);
        $backups = isset($jobData['backups']) ? $this->jobsFunctions->buildBackupsOutput($jobData['backups'], $failed, $sid) : "";
        if($backups){
            $hasRun = true;
        }

        if ($failed == null or ($failed == "failed" and $failures !== 0)) {
            if ($appName == "on_demand") {
                $clientName = "";
            } else {
                $clientName = isset($jobData['client_name']) ? $jobData['client_name'] : $this->jobsFunctions->getClientNameFromBackup($backups);
            }

            if ($hasRun) {
                $data = array(
                    'client_name' => $clientName,
                    'app_name' => $appName == "Archive" ? "Backup Copy" : $appName,
                    'description' => $scheduleDescription,
                    'id' => $sid . '.' . $this->itemCount++,
                    'duration' => $duration,
                    'type' => $appType == "Archive" ? "Backup Copy" : "Backup",
                    'status' => $status,
                    'failures' => $failures,
                    'name' => $scheduleName,
                    'start_date' => $this->jobsFunctions->getStartDateFromBackup($backups, $historyStartDate),
                    'system_name' => $sysName,
                    'system_id' => $sid,
                    'backups' => $backups
                );
            }
        }

        return $data;
    }

    function buildReplicationHistoryOutput($replicationHistory, $sid, $startDate, $failed){
        $data = array();

       // var_dump($replicationHistory);
        $replicationInstanceID = isset($replicationHistory['instance_id']) ? $replicationHistory['instance_id'] : null;

        if($replicationInstanceID) {
            $appInst = $this->BP->get_appinst_info($replicationInstanceID, $sid);
            foreach($appInst as $instance) {
                $appName = isset($instance['app_name']) ? $instance['app_name'] : "n/a";
                if ($appName != "System Metadata" && $appName != 'Archive') {
                    $data = array(
                        'type' => isset($replicationHistory['type']) ? $replicationHistory['type'] : "",
                        'mode' => isset($replicationHistory['mode']) ? $this->functions->getBackupTypeDisplayName($replicationHistory['mode']) : "",
                        'client_name' => isset($replicationHistory['client_name']) ? $replicationHistory['client_name'] : "",
                        'app_name' => $appName,
                	    'backup_id' => isset($replicationHistory['backup_id']) ? $replicationHistory['backup_id'] : "N/A",
                	    'id' => isset($replicationHistory['id']) ? $replicationHistory['id'] : "N/A",
                        'duration' => isset($replicationHistory['start_time']) ? $this->functions->formatTimeDelta($this->now - $replicationHistory['start_time']) : "n/a",
                        'status' => isset($replicationHistory['status']) ? $this->getReplicationStatusName($replicationHistory['status']) : "n/a",
                        'start_time' => isset($replicationHistory['start_time']) ? $this->functions->formatDatetime($replicationHistory['start_time']) : "n/a",
                        'system_name' => isset($replicationHistory['system_name']) ? $replicationHistory['system_name'] : "n/a",
                        'system_id' => $sid,
                        'target' => isset($replicationHistory['target']) ? $replicationHistory['target'] : "n/a",
                        'primary_name' => isset($instance['primary_name']) ? $instance['primary_name'] : "n/a",
                        'secondary_name' => isset($instance['secondary_name']) ? $instance['secondary_name'] : "n/a",
                        'name' => isset($replicationHistory['schedule_name']) ? $replicationHistory['schedule_name'] : "n/a",
                    );
                    break;
                }
            }
        }

        return $data;
    }
    //this needs to be moved job functions as it's used in both active and history
    function buildReturn($data, $filter){
        $skip = false;

        if ($filter != "") {
            switch ($filter) {

                // get them all from jobs list.
                case 'active':
                    break;
                // see if type matches user selection.  If not, skip it.
                default:
                    if ($data['type'] != $filter) {
                        $skip = true;
                    }
                    break;
            }
        }
        if ($skip) {
            $data = NULL;
        }

        return $data;
    }

    function getReplicationStatusName($status) {
        $value = $status;
        switch ($status) {
            case 'completed':
                $value = 'Success';
                break;
            case 'failed':
            case 'aborted':
            case 'terminated':
                $value = 'Error';
                break;
            case 'active':
                $value = 'Active';
                break;
            case 'quued':
                $value = 'Queued';
                break;
        }
        return $value;
    }

} //end JobsHistory class


