<?php

$dir = '/api/includes/reports';

class DataReductionReport
{
    private $BP;

    public function __construct( $BP, $functionsClass )
    {
        $this->BP = $BP;
        $this->functions = $functionsClass;
    }

    public function get_data_reduction_report($systems, $data)
    {
        $report = array();
        $report_count = 0;
        //Temporary until more data is available
        $report_total = $report_count;
        $report_start = 1;
        $report_data = array();

        $start_time = false;
        if ( array_key_exists('start_date', $data) )
        {
            $start_time = strtotime($data['start_date']);
        }
        $end_time = false;
        if ( array_key_exists('end_date', $data) )
        {
            $end_time = $this->functions->formatEndDate($data['end_date']);
        }

        $isValidDate = $this->functions->isValidDateRange($start_time, $end_time, false);

        if($isValidDate) {
            foreach ($systems as $system_id => $system_name)
            {

                $data_reductions_stats = $this->BP->get_data_reduction_stats($system_id);
                if ( $data_reductions_stats !== false and count($data_reductions_stats) > 0 )
                {
                    foreach ($data_reductions_stats as $data_stat)
                    {
                        $temp_time = $data_stat['sample_time'];

                        if ( $temp_time !== 0 and ( $start_time === false or $start_time <= $temp_time ) and ( $end_time === false or $end_time >= $temp_time ) )
                        {
                            $temp_data_stat = array();
                            $temp_data_stat['date'] = date(Constants::DATE_FORMAT_DATE_THEN_12HR_TIME, $temp_time);
                            $temp_data_stat['dedup'] = $data_stat['deduped'];
                            $temp_data_stat['data_reduction'] = $data_stat['compressed'];
                            $temp_data_stat['system_name'] = $system_name;

                            $report_data[] = $temp_data_stat;
                            $report_count += 1;
                        }
                    }
                }
            }
            $report_total = $report_count;  //Temporary

            $report['count'] = $report_count;
            $report['total'] = $report_total;
            $report['start'] = $report_start;
            $report['data'] = $report_data;
        } else {
            $report['error'] = 500;
            $report['message'] = "Invalid date range provided";
        }
        return $report;
    }
}