<?php

$dir = '/api/includes/reports';

class ProtectionSummaryReport
{
    private $BP;

    public function __construct( $BP, $functionsClass )
    {
        $this->BP = $BP;
        $this->functions = $functionsClass;
    }


    public function get_protection_summary_report( $systems, $data )
    {
        $report = array();
        $report_count = 0;
        //Temporary until more data is available
        $report_total = $report_count;
        $report_start = 1;
        $report_data = array();


        $start_time = false;
        if ( array_key_exists('start_date', $data) )
        {
            $start_time = strtotime($data['start_date']);
        }
        $end_time = false;
        if ( array_key_exists('end_date', $data) )
        {
            $end_time = $this->functions->formatEndDate($data['end_date']);
        }

        $isValidDate = $this->functions->isValidDateRange( $start_time, $end_time, false);

        if($isValidDate)
        {
            $assets_with_a_backup_count = 0;
            $assets_with_a_backup_copy_count = 0;
            $total_number_of_assets = 0;
            $number_of_backups_count_array = array('successes'=>0, 'warnings'=>0, 'failures'=>0);
            $number_of_all_backup_copies_count_array = array('successes'=>0, 'failures'=>0);
            $number_of_backup_copies_per_target_count_array = array();  //Each target is array('successes'=>0, 'failures'=>0)
            $assets_table = array();
            $backup_assets_table = array();
            $backup_copy_assets_tables = array();  // array of tables for each target

            foreach ($systems as $system_id => $system_name)
            {
                $assetsByInstanceID = array(); //instance_id is the key
                $fileLevelInstanceIDByClientID = array();  //client_id is the key and the file-level instance id is the value

                $clients = $this->BP->get_client_list($system_id);
                if ( $clients !== false )
                {
                    foreach ( $clients as $client_id => $client_name )
                    {
                        $clientInfo = $this->BP->get_client_info($client_id, $system_id);
                        if ( $clientInfo !== false )
                        {
                            // Do not show a client entry for vCenter RRC, Generic Clients (because they do not have a file-level entry), or the local system
                            if ( $clientInfo['name'] !== $system_name and $clientInfo['name'] !== Constants::CLIENT_NAME_VCENTER_RRC and $clientInfo['os_type_id'] !== Constants::OS_GENERIC )
                            {
                                $fileLevelInstanceIDByClientID[$client_id] = $clientInfo['file_level_instance_id'];
                                $assetsByInstanceID[$clientInfo['file_level_instance_id']] = $this->initializeAssetArray( $clientInfo['name'], '', $clientInfo['os_type'] );
                            }
                            foreach ( $clientInfo['applications'] as $applicationID => $applicationArray )
                            {
                                switch ($applicationArray['type'])
                                {
                                    case Constants::APPLICATION_ID_FILE_LEVEL:
                                        break;  //Do nothing for File-Level
                                    case Constants::APPLICATION_TYPE_NAME_EXCHANGE:
                                        $exchangeInfo = $this->BP->get_exchange_info($client_id, true, $system_id);
                                        if ( $exchangeInfo !== false )
                                        {
                                            foreach ($exchangeInfo as $exchangeInstance)
                                            {
                                                $assetsByInstanceID[$exchangeInstance['instance_id']] = $this->initializeAssetArray( $clientInfo['name'], $exchangeInstance['name'], Constants::APPLICATION_TYPE_NAME_EXCHANGE );
                                            }
                                        }
                                        break;
                                    case Constants::APPLICATION_TYPE_NAME_SQL_SERVER:
                                        $sqlInfo = $this->BP->get_sql_info($client_id, $applicationID, true, $system_id);
                                        if ( $sqlInfo !== false )
                                        {
                                            foreach ($sqlInfo as $sqlInstance)
                                            {
                                                $assetsByInstanceID[$sqlInstance['instance_id']] = $this->initializeAssetArray( $clientInfo['name'], $sqlInstance['database'], Constants::APPLICATION_TYPE_NAME_SQL_SERVER );
                                            }
                                        }
                                        break;
                                    case Constants::APPLICATION_TYPE_NAME_VMWARE:
                                        if ( array_key_exists('servers', $applicationArray) and count($applicationArray['servers']) > 0 )
                                        {
                                            foreach ($applicationArray['servers'] as $serverUUID => $serverHostname)
                                            {
                                                $vmInfo = $this->BP->get_vm_info($serverUUID, NULL, true, false, $system_id);
                                                if ( $vmInfo !== false )
                                                {
                                                    foreach ($vmInfo as $vmInstance)
                                                    {
                                                        $assetsByInstanceID[$vmInstance['instance_id']] = $this->initializeAssetArray( $serverHostname, $vmInstance['name'], Constants::APPLICATION_TYPE_NAME_VMWARE );
                                                    }
                                                }
                                            }
                                        }
                                        break;
                                    case Constants::APPLICATION_TYPE_NAME_HYPER_V:
                                        $hyperVInfo = $this->BP->get_hyperv_info($client_id, true, $system_id);
                                        if ( $hyperVInfo !== false )
                                        {
                                            foreach ($hyperVInfo as $hyperVInstance)
                                            {
                                                $assetsByInstanceID[$hyperVInstance['instance_id']] = $this->initializeAssetArray( $clientInfo['name'], $hyperVInstance['name'], Constants::APPLICATION_TYPE_NAME_HYPER_V );
                                            }
                                        }
                                        break;
                                    case Constants::APPLICATION_TYPE_NAME_ORACLE:
                                        $oracleInfo = $this->BP->get_oracle_info($client_id, $applicationID, true, $system_id);
                                        if ( $oracleInfo !== false )
                                        {
                                            foreach ($oracleInfo as $oracleInstance)
                                            {
                                                $assetsByInstanceID[$oracleInstance['instance_id']] = $this->initializeAssetArray( $clientInfo['name'], $oracleInstance['name'], Constants::APPLICATION_TYPE_NAME_ORACLE );
                                            }
                                        }
                                        break;
                                    case Constants::APPLICATION_TYPE_NAME_SHAREPOINT:
                                        $sharePointInfo = $this->BP->get_sharepoint_info($client_id, $applicationID, true, $system_id);
                                        if ( $sharePointInfo !== false )
                                        {
                                            foreach ($sharePointInfo as $sharePointInstance)
                                            {
                                                $assetsByInstanceID[$sharePointInstance['instance_id']] = $this->initializeAssetArray( $clientInfo['name'], $sharePointInstance['name'], Constants::APPLICATION_TYPE_NAME_SHAREPOINT );
                                            }
                                        }
                                        break;
                                    case Constants::APPLICATION_TYPE_NAME_UCS_SERVICE_PROFILE:
                                        break;  //Do nothing in 9.0
                                    case Constants::APPLICATION_TYPE_NAME_VOLUME:  //NDMP Volume
                                    case Constants::APPLICATION_TYPE_NAME_NDMP_DEVICE:
                                        break; //Do nothing in 9.0
                                    case Constants::APPLICATION_TYPE_NAME_XEN:
                                        $xenInfo = $this->BP->get_xen_vm_info($client_id, true, false, $system_id);
                                        if ( $xenInfo !== false )
                                        {
                                            foreach ($xenInfo as $xenInstance)
                                            {
                                                $assetsByInstanceID[$xenInstance['instance_id']] = $this->initializeAssetArray( $clientInfo['name'], $xenInstance['name'], Constants::APPLICATION_TYPE_NAME_XEN );
                                            }
                                        }
                                        break;
                                }
                            }
                        }
                    }
                }

                $total_number_of_assets += count( $assetsByInstanceID );  // Add the total number of assets for this system

                $backup_status_result_format = array();
                $backup_status_result_format['system_id'] = $system_id;
                $backup_status_result_format['type'] = array(
                    Constants::BACKUP_TYPE_MASTER,
                    Constants::BACKUP_TYPE_DIFFERENTIAL,
                    Constants::BACKUP_TYPE_INCREMENTAL,
                    Constants::BACKUP_TYPE_BAREMETAL,
                    Constants::BACKUP_TYPE_SELECTIVE,
                    Constants::BACKUP_TYPE_MSSQL_FULL,
                    Constants::BACKUP_TYPE_MSSQL_DIFFERENTIAL,
                    Constants::BACKUP_TYPE_MSSQL_TRANSACTION,
                    Constants::BACKUP_TYPE_EXCHANGE_FULL,
                    Constants::BACKUP_TYPE_EXCHANGE_DIFFERENTIAL,
                    Constants::BACKUP_TYPE_EXCHANGE_INCREMENTAL,
                    Constants::BACKUP_TYPE_LEGACY_MSSQL_FULL,
                    Constants::BACKUP_TYPE_LEGACY_MSSQL_DIFF,
                    Constants::BACKUP_TYPE_LEGACY_MSSQL_TRANS,
                    Constants::BACKUP_TYPE_VMWARE_FULL,
                    Constants::BACKUP_TYPE_VMWARE_DIFFERENTIAL,
                    Constants::BACKUP_TYPE_VMWARE_INCREMENTAL,
                    Constants::BACKUP_TYPE_HYPER_V_FULL,
                    Constants::BACKUP_TYPE_HYPER_V_INCREMENTAL,
                    Constants::BACKUP_TYPE_HYPER_V_DIFFERENTIAL,
                    Constants::BACKUP_TYPE_ORACLE_FULL,
                    Constants::BACKUP_TYPE_ORACLE_INCR,
                    Constants::BACKUP_TYPE_SHAREPOINT_FULL,
                    Constants::BACKUP_TYPE_SHAREPOINT_DIFF,
                    Constants::BACKUP_TYPE_XEN_FULL ); //,  The following backup types are not needed for 9.0
                //Constants::BACKUP_TYPE_SYSTEM_METADATA,  // Do NOT include system metadata in this report
                //Constants::BACKUP_TYPE_UCS_SERVICE_PROFILE_FULL,
                //Constants::BACKUP_TYPE_NDMP_FULL,
                //Constants::BACKUP_TYPE_NDMP_DIFF,
                //Constants::BACKUP_TYPE_NDMP_INCR );

                if($start_time !== false) {
                    $backup_status_result_format['start_time'] = $start_time;
                }
                if($end_time !== false) {
                    $backup_status_result_format['end_time'] = $end_time;
                }
                $backup_status_result_format['grandclients'] = false;
                /*
                if ( array_key_exists('finish_interval_start', $data) )
                {
                    $backup_status_result_format['finish_interval_start'] = $data['finish_interval_start'];
                }
                if ( array_key_exists('Finish_interval_end', $data) )
                {
                    $backup_status_result_format['Finish_interval_end'] = $data['Finish_interval_end'];
                }
                */

                $backup_status_result_array = $this->BP->get_backup_status($backup_status_result_format);

                if ($backup_status_result_array !== false)
                {
                    foreach ($backup_status_result_array as &$backup)
                    {
                        if ( $backup['complete'] === true )
                        {
                            $backupInstanceID = NULL;
                            $backupType = NULL;
                            switch ( $backup['type'] )
                            {
                                case Constants::BACKUP_TYPE_MASTER:
                                    $backupInstanceID = $fileLevelInstanceIDByClientID[$backup['client_id']];
                                    $backupType = Constants::BACKUP_DISPLAY_TYPE_FULL;
                                    break;
                                case Constants::BACKUP_TYPE_MSSQL_FULL:
                                case Constants::BACKUP_TYPE_EXCHANGE_FULL:
                                case Constants::BACKUP_TYPE_LEGACY_MSSQL_FULL:
                                case Constants::BACKUP_TYPE_VMWARE_FULL:
                                case Constants::BACKUP_TYPE_HYPER_V_FULL:
                                case Constants::BACKUP_TYPE_ORACLE_FULL:
                                case Constants::BACKUP_TYPE_SHAREPOINT_FULL:
                                case Constants::BACKUP_TYPE_XEN_FULL:
                                    $backupInstanceID = $backup['instance_id'];
                                    $backupType = Constants::BACKUP_DISPLAY_TYPE_FULL;
                                    break;
                                case Constants::BACKUP_TYPE_DIFFERENTIAL:
                                    $backupInstanceID = $fileLevelInstanceIDByClientID[$backup['client_id']];
                                    $backupType = Constants::BACKUP_DISPLAY_TYPE_DIFFERENTIAL;
                                    break;
                                case Constants::BACKUP_TYPE_MSSQL_DIFFERENTIAL:
                                case Constants::BACKUP_TYPE_EXCHANGE_DIFFERENTIAL:
                                case Constants::BACKUP_TYPE_LEGACY_MSSQL_DIFF:
                                case Constants::BACKUP_TYPE_VMWARE_DIFFERENTIAL:
                                case Constants::BACKUP_TYPE_HYPER_V_DIFFERENTIAL:
                                case Constants::BACKUP_TYPE_SHAREPOINT_DIFF:
                                    $backupInstanceID = $backup['instance_id'];
                                    $backupType = Constants::BACKUP_DISPLAY_TYPE_DIFFERENTIAL;
                                    break;
                                case Constants::BACKUP_TYPE_INCREMENTAL:
                                    $backupInstanceID = $fileLevelInstanceIDByClientID[$backup['client_id']];
                                    $backupType = Constants::BACKUP_DISPLAY_TYPE_INCREMENTAL;
                                    break;
                                case Constants::BACKUP_TYPE_EXCHANGE_INCREMENTAL:
                                case Constants::BACKUP_TYPE_VMWARE_INCREMENTAL:
                                case Constants::BACKUP_TYPE_HYPER_V_INCREMENTAL:
                                case Constants::BACKUP_TYPE_ORACLE_INCR:
                                    $backupInstanceID = $backup['instance_id'];
                                    $backupType = Constants::BACKUP_DISPLAY_TYPE_INCREMENTAL;
                                    break;
                                case Constants::BACKUP_TYPE_BAREMETAL:
                                    $backupInstanceID = $fileLevelInstanceIDByClientID[$backup['client_id']];
                                    $backupType = Constants::BACKUP_TYPE_BAREMETAL;
                                    break;
                                case Constants::BACKUP_TYPE_SELECTIVE:
                                    $backupInstanceID = $fileLevelInstanceIDByClientID[$backup['client_id']];
                                    $backupType = Constants::BACKUP_DISPLAY_TYPE_SELECTIVE;
                                    break;
                                case Constants::BACKUP_TYPE_MSSQL_TRANSACTION:
                                case Constants::BACKUP_TYPE_LEGACY_MSSQL_TRANS:
                                    $backupInstanceID = $backup['instance_id'];
                                    $backupType = Constants::BACKUP_DISPLAY_TYPE_TRANSACTION;
                                    break;
                            }

                            switch ( $backup['status'] )
                            {
                                case Constants::BACKUP_STATUS_SUCCESS:
                                    if ( $assetsByInstanceID[$backupInstanceID]['protected'] === false )
                                    {
                                        $assetsByInstanceID[$backupInstanceID]['protected'] = true;
                                        $assets_with_a_backup_count += 1;
                                    }
                                    if ( $backup['start_time'] > $assetsByInstanceID[$backupInstanceID]['last_backup_timestamp'] )
                                    {
                                        $assetsByInstanceID[$backupInstanceID]['last_backup_timestamp'] = $backup['start_time'];
                                    }
                                    $assetsByInstanceID[$backupInstanceID]['backup_types'][$backupType] = true;
                                    $assetsByInstanceID[$backupInstanceID]['backups_count_by_status']['successes'] += 1;
                                    $number_of_backups_count_array['successes'] += 1;
                                    break;
                                case Constants::BACKUP_STATUS_WARNINGS:
                                    $assetsByInstanceID[$backupInstanceID]['backups_count_by_status']['warnings'] += 1;
                                    $number_of_backups_count_array['warnings'] += 1;
                                    break;
                                case Constants::BACKUP_STATUS_FAILURE:
                                    $assetsByInstanceID[$backupInstanceID]['backups_count_by_status']['failures'] += 1;
                                    $number_of_backups_count_array['failures'] += 1;
                                    break;
                            }
                        }
                    }
                }
                else
                {
                    $report = false;  //Fix this logic
                }

                // ----Replication Backup Copy Logic----

                $backup_status_result_format['type'] = array(
                    Constants::BACKUP_TYPE_SECURESYNC_MASTER,
                    Constants::BACKUP_TYPE_SECURESYNC_DIFFERENTIAL,
                    Constants::BACKUP_TYPE_SECURESYNC_INCREMENTAL,
                    Constants::BACKUP_TYPE_SECURESYNC_BAREMETAL,
                    //Constants::BACKUP_TYPE_SECURESYNC_DPU_STATE, // Do NOT include system metadata in this report
                    //Constants::BACKUP_TYPE_SECURESYNC_LOCAL_DIRECTORY, // Do NOT include system metadata in this report
                    Constants::BACKUP_TYPE_SECURESYNC_MS_SQL,
                    Constants::BACKUP_TYPE_SECURESYNC_EXCHANGE,
                    Constants::BACKUP_TYPE_SECURESYNC_MSSQL_FULL,
                    Constants::BACKUP_TYPE_SECURESYNC_MSSQL_DIFFERENTIAL,
                    Constants::BACKUP_TYPE_SECURESYNC_MSSQL_TRANSACTION,
                    Constants::BACKUP_TYPE_SECURESYNC_EXCHANGE_FULL,
                    Constants::BACKUP_TYPE_SECURESYNC_EXCHANGE_DIFFERENTIAL,
                    Constants::BACKUP_TYPE_SECURESYNC_EXCHANGE_INCREMENTAL,
                    Constants::BACKUP_TYPE_SECURESYNC_VMWARE_FULL,
                    Constants::BACKUP_TYPE_SECURESYNC_VMWARE_DIFFERENTIAL,
                    Constants::BACKUP_TYPE_SECURESYNC_VMWARE_INCREMENTAL,
                    Constants::BACKUP_TYPE_SECURESYNC_HYPER_V_FULL,
                    Constants::BACKUP_TYPE_SECURESYNC_HYPER_V_INCREMENTAL,
                    Constants::BACKUP_TYPE_SECURESYNC_HYPER_V_DIFFERENTIAL,
                    Constants::BACKUP_TYPE_SECURESYNC_ORACLE_FULL,
                    Constants::BACKUP_TYPE_SECURESYNC_ORACLE_INCR,
                    Constants::BACKUP_TYPE_SECURESYNC_SHAREPOINT_FULL,
                    Constants::BACKUP_TYPE_SECURESYNC_SHAREPOINT_DIFF,
                    Constants::BACKUP_TYPE_SECURESYNC_XEN_FULL ); //,  The following backup types are not needed for 9.0
                //Constants::BACKUP_TYPE_SECURESYNC_UCS_SERVICE_PROFILE_FULL,
                //Constants::BACKUP_TYPE_SECURESYNC_NDMP_FULL,
                //Constants::BACKUP_TYPE_SECURESYNC_NDMP_DIFF,
                //Constants::BACKUP_TYPE_SECURESYNC_NDMP_INCR);

                $backup_status_result_array = $this->BP->get_backup_status($backup_status_result_format);

                if ($backup_status_result_array !== false)
                {
                    foreach ($backup_status_result_array as &$backup)
                    {
                        if ( $backup['complete'] === true )
                        {
                            $backupInstanceID = NULL;
                            $backupType = NULL;
                            $targetName = $backup['destination'];
                            if ( array_key_exists( $targetName, $number_of_backup_copies_per_target_count_array) === false )
                            {
                                $number_of_backup_copies_per_target_count_array[$targetName] = array('successes'=>0, 'failures'=>0);
                                $backup_copy_assets_tables['backup_copies_to_'.$targetName] = array();
                            }

                            switch ( $backup['type'] )
                            {
                                case Constants::BACKUP_TYPE_SECURESYNC_MASTER:
                                    $backupInstanceID = $fileLevelInstanceIDByClientID[$backup['client_id']];
                                    $backupType = Constants::BACKUP_DISPLAY_TYPE_FULL;
                                    break;
                                case Constants::BACKUP_TYPE_SECURESYNC_MS_SQL:
                                case Constants::BACKUP_TYPE_SECURESYNC_EXCHANGE:
                                case Constants::BACKUP_TYPE_SECURESYNC_EXCHANGE_FULL:
                                case Constants::BACKUP_TYPE_SECURESYNC_MSSQL_FULL:
                                case Constants::BACKUP_TYPE_SECURESYNC_VMWARE_FULL:
                                case Constants::BACKUP_TYPE_SECURESYNC_HYPER_V_FULL:
                                case Constants::BACKUP_TYPE_SECURESYNC_ORACLE_FULL:
                                case Constants::BACKUP_TYPE_SECURESYNC_SHAREPOINT_FULL:
                                case Constants::BACKUP_TYPE_SECURESYNC_XEN_FULL:
                                    $backupInstanceID = $backup['instance_id'];
                                    $backupType = Constants::BACKUP_DISPLAY_TYPE_FULL;
                                    break;
                                case Constants::BACKUP_TYPE_SECURESYNC_DIFFERENTIAL:
                                    $backupInstanceID = $fileLevelInstanceIDByClientID[$backup['client_id']];
                                    $backupType = Constants::BACKUP_DISPLAY_TYPE_DIFFERENTIAL;
                                    break;
                                case Constants::BACKUP_TYPE_SECURESYNC_MSSQL_DIFFERENTIAL:
                                case Constants::BACKUP_TYPE_SECURESYNC_EXCHANGE_DIFFERENTIAL:
                                case Constants::BACKUP_TYPE_SECURESYNC_VMWARE_DIFFERENTIAL:
                                case Constants::BACKUP_TYPE_SECURESYNC_HYPER_V_DIFFERENTIAL:
                                case Constants::BACKUP_TYPE_SECURESYNC_SHAREPOINT_DIFF:
                                    $backupInstanceID = $backup['instance_id'];
                                    $backupType = Constants::BACKUP_DISPLAY_TYPE_DIFFERENTIAL;
                                    break;
                                case Constants::BACKUP_TYPE_SECURESYNC_INCREMENTAL:
                                    $backupInstanceID = $fileLevelInstanceIDByClientID[$backup['client_id']];
                                    $backupType = Constants::BACKUP_DISPLAY_TYPE_INCREMENTAL;
                                    break;
                                case Constants::BACKUP_TYPE_SECURESYNC_EXCHANGE_INCREMENTAL:
                                case Constants::BACKUP_TYPE_SECURESYNC_VMWARE_INCREMENTAL:
                                case Constants::BACKUP_TYPE_SECURESYNC_HYPER_V_INCREMENTAL:
                                case Constants::BACKUP_TYPE_SECURESYNC_ORACLE_INCR:
                                    $backupInstanceID = $backup['instance_id'];
                                    $backupType = Constants::BACKUP_DISPLAY_TYPE_INCREMENTAL;
                                    break;
                                case Constants::BACKUP_TYPE_SECURESYNC_BAREMETAL:
                                    $backupInstanceID = $fileLevelInstanceIDByClientID[$backup['client_id']];
                                    $backupType = Constants::BACKUP_TYPE_BAREMETAL;
                                    break;
                                case Constants::BACKUP_TYPE_SECURESYNC_MSSQL_TRANSACTION:
                                    $backupInstanceID = $backup['instance_id'];
                                    $backupType = Constants::BACKUP_DISPLAY_TYPE_TRANSACTION;
                                    break;
                            }

                            if ( array_key_exists( ('last_backup_copy_to_'.$targetName.'_timestamp'), $assetsByInstanceID[$backupInstanceID]) === false )
                            {
                                $assetsByInstanceID[$backupInstanceID]['last_backup_copy_to_'.$targetName.'_timestamp'] = 0;
                                $assetsByInstanceID[$backupInstanceID]['backups_copies_to_'.$targetName.'_count_by_status'] = array('successes'=>0, 'failures'=>0);
                                $assetsByInstanceID[$backupInstanceID]['backup_copy_types_to_'.$targetName] = array(  Constants::BACKUP_DISPLAY_TYPE_FULL=>false,
                                    Constants::BACKUP_DISPLAY_TYPE_DIFFERENTIAL=>false,
                                    Constants::BACKUP_DISPLAY_TYPE_INCREMENTAL=>false,
                                    Constants::BACKUP_TYPE_BAREMETAL=>false, // the display type for Bare Metal is 2 words, so I used the backup type as the key
                                    Constants::BACKUP_DISPLAY_TYPE_TRANSACTION=>false);
                                $assetsByInstanceID[$backupInstanceID]['backup_copy_targets'][] = $targetName;
                                $assetsByInstanceID[$backupInstanceID]['next_backup_copy_to_'.$targetName.'_timestamp'] = 0;
                                $assetsByInstanceID[$backupInstanceID]['next_backup_copy_to_'.$targetName.'_joborder'] = '';
                            }

                            switch ( $backup['status'] )
                            {
                                case Constants::BACKUP_STATUS_SUCCESS:
                                    if ( $assetsByInstanceID[$backupInstanceID]['protected_by_backup_copy'] === false )
                                    {
                                        $assetsByInstanceID[$backupInstanceID]['protected_by_backup_copy'] = true;
                                        $assets_with_a_backup_copy_count += 1;
                                    }
                                    if ( $backup['start_time'] > $assetsByInstanceID[$backupInstanceID]['last_backup_copy_to_'.$targetName.'_timestamp'] )
                                    {
                                        $assetsByInstanceID[$backupInstanceID]['last_backup_copy_to_'.$targetName.'_timestamp'] = $backup['start_time'];
                                    }
                                    $assetsByInstanceID[$backupInstanceID]['backup_copy_types_to_'.$targetName][$backupType] = true;
                                    $assetsByInstanceID[$backupInstanceID]['backups_copies_to_'.$targetName.'_count_by_status']['successes'] += 1;
                                    $number_of_all_backup_copies_count_array['successes'] += 1;
                                    $number_of_backup_copies_per_target_count_array[$targetName]['successes'] += 1;
                                    break;
                                case Constants::BACKUP_STATUS_FAILURE:
                                    $assetsByInstanceID[$backupInstanceID]['backups_copies_to_'.$targetName.'_count_by_status']['failures'] += 1;
                                    $number_of_all_backup_copies_count_array['failures'] += 1;
                                    $number_of_backup_copies_per_target_count_array[$targetName]['failures'] += 1;
                                    break;
                            }
                        }
                    }
                }
                else
                {
                    $report = false;  //Fix this logic
                }

                //----Archiving Backup Copy Logic----


                $archive_status_result_format = array();
                $archive_status_result_format['type'] = array(
                    Constants::BACKUP_TYPE_MASTER,
                    Constants::BACKUP_TYPE_DIFFERENTIAL,
                    Constants::BACKUP_TYPE_INCREMENTAL,
                    Constants::BACKUP_TYPE_BAREMETAL,
                    Constants::BACKUP_TYPE_SELECTIVE,
                    Constants::BACKUP_TYPE_MSSQL_FULL,
                    Constants::BACKUP_TYPE_MSSQL_DIFFERENTIAL,
                    Constants::BACKUP_TYPE_MSSQL_TRANSACTION,
                    Constants::BACKUP_TYPE_EXCHANGE_FULL,
                    Constants::BACKUP_TYPE_EXCHANGE_DIFFERENTIAL,
                    Constants::BACKUP_TYPE_EXCHANGE_INCREMENTAL,
                    Constants::BACKUP_TYPE_LEGACY_MSSQL_FULL,  // Not listed in the definition of bp_get_archive_status
                    Constants::BACKUP_TYPE_LEGACY_MSSQL_DIFF,  // Not listed in the definition of bp_get_archive_status
                    Constants::BACKUP_TYPE_LEGACY_MSSQL_TRANS,  // Not listed in the definition of bp_get_archive_status
                    Constants::BACKUP_TYPE_VMWARE_FULL,
                    Constants::BACKUP_TYPE_VMWARE_DIFFERENTIAL,
                    Constants::BACKUP_TYPE_VMWARE_INCREMENTAL,
                    Constants::BACKUP_TYPE_HYPER_V_FULL,
                    Constants::BACKUP_TYPE_HYPER_V_INCREMENTAL,
                    Constants::BACKUP_TYPE_HYPER_V_DIFFERENTIAL,  // Not listed in the definition of bp_get_archive_status
                    Constants::BACKUP_TYPE_ORACLE_FULL,
                    Constants::BACKUP_TYPE_ORACLE_INCR,
                    Constants::BACKUP_TYPE_SHAREPOINT_FULL,
                    Constants::BACKUP_TYPE_SHAREPOINT_DIFF,
                    Constants::BACKUP_TYPE_XEN_FULL ); //,  The following backup types are not needed for 9.0
                //Constants::BACKUP_TYPE_SYSTEM_METADATA,  // Do NOT include system metadata in this report
                //Constants::BACKUP_TYPE_UCS_SERVICE_PROFILE_FULL,
                //Constants::BACKUP_TYPE_NDMP_FULL,
                //Constants::BACKUP_TYPE_NDMP_DIFF,
                //Constants::BACKUP_TYPE_NDMP_INCR );

                if($start_time !== false) {
                    $archive_status_result_format['job_start_time'] = $start_time;
                }
                if($end_time !== false) {
                    $archive_status_result_format['job_end_time'] = $end_time;
                }

                $archive_status_result_format = $this->BP->get_archive_status($archive_status_result_format, $system_id);

                if ($archive_status_result_format !== false)
                {
                    foreach ($archive_status_result_format as &$archive_backup)
                    {
                        if ( $archive_backup['is_imported'] === false and array_key_exists('client_id', $archive_backup) )
                        {
                            $backupInstanceID = NULL;
                            $backupType = NULL;
                            $targetName = $archive_backup['target'];
                            if ( array_key_exists( $targetName, $number_of_backup_copies_per_target_count_array) === false )
                            {
                                $number_of_backup_copies_per_target_count_array[$targetName] = array('successes'=>0, 'failures'=>0);
                                $backup_copy_assets_tables['backup_copies_to_'.$targetName] = array();
                            }

                            switch ( $archive_backup['type'] )
                            {
                                case Constants::BACKUP_TYPE_MASTER:
                                    $backupInstanceID = $fileLevelInstanceIDByClientID[$archive_backup['client_id']];
                                    $backupType = Constants::BACKUP_DISPLAY_TYPE_FULL;
                                    break;
                                case Constants::BACKUP_TYPE_MSSQL_FULL:
                                case Constants::BACKUP_TYPE_EXCHANGE_FULL:
                                case Constants::BACKUP_TYPE_LEGACY_MSSQL_FULL:
                                case Constants::BACKUP_TYPE_VMWARE_FULL:
                                case Constants::BACKUP_TYPE_HYPER_V_FULL:
                                case Constants::BACKUP_TYPE_ORACLE_FULL:
                                case Constants::BACKUP_TYPE_SHAREPOINT_FULL:
                                case Constants::BACKUP_TYPE_XEN_FULL:
                                    $backupInstanceID = $archive_backup['instance_id'];
                                    $backupType = Constants::BACKUP_DISPLAY_TYPE_FULL;
                                    break;
                                case Constants::BACKUP_TYPE_DIFFERENTIAL:
                                    $backupInstanceID = $fileLevelInstanceIDByClientID[$archive_backup['client_id']];
                                    $backupType = Constants::BACKUP_DISPLAY_TYPE_DIFFERENTIAL;
                                    break;
                                case Constants::BACKUP_TYPE_MSSQL_DIFFERENTIAL:
                                case Constants::BACKUP_TYPE_EXCHANGE_DIFFERENTIAL:
                                case Constants::BACKUP_TYPE_LEGACY_MSSQL_DIFF:
                                case Constants::BACKUP_TYPE_VMWARE_DIFFERENTIAL:
                                case Constants::BACKUP_TYPE_HYPER_V_DIFFERENTIAL:
                                case Constants::BACKUP_TYPE_SHAREPOINT_DIFF:
                                    $backupInstanceID = $archive_backup['instance_id'];
                                    $backupType = Constants::BACKUP_DISPLAY_TYPE_DIFFERENTIAL;
                                    break;
                                case Constants::BACKUP_TYPE_INCREMENTAL:
                                    $backupInstanceID = $fileLevelInstanceIDByClientID[$archive_backup['client_id']];
                                    $backupType = Constants::BACKUP_DISPLAY_TYPE_INCREMENTAL;
                                    break;
                                case Constants::BACKUP_TYPE_EXCHANGE_INCREMENTAL:
                                case Constants::BACKUP_TYPE_VMWARE_INCREMENTAL:
                                case Constants::BACKUP_TYPE_HYPER_V_INCREMENTAL:
                                case Constants::BACKUP_TYPE_ORACLE_INCR:
                                    $backupInstanceID = $archive_backup['instance_id'];
                                    $backupType = Constants::BACKUP_DISPLAY_TYPE_INCREMENTAL;
                                    break;
                                case Constants::BACKUP_TYPE_BAREMETAL:
                                    $backupInstanceID = $fileLevelInstanceIDByClientID[$archive_backup['client_id']];
                                    $backupType = Constants::BACKUP_TYPE_BAREMETAL;
                                    break;
                                case Constants::BACKUP_TYPE_SELECTIVE:
                                    $backupInstanceID = $fileLevelInstanceIDByClientID[$archive_backup['client_id']];
                                    $backupType = Constants::BACKUP_DISPLAY_TYPE_SELECTIVE;
                                    break;
                                case Constants::BACKUP_TYPE_MSSQL_TRANSACTION:
                                case Constants::BACKUP_TYPE_LEGACY_MSSQL_TRANS:
                                    $backupInstanceID = $archive_backup['instance_id'];
                                    $backupType = Constants::BACKUP_DISPLAY_TYPE_TRANSACTION;
                                    break;
                            }

                            if ( array_key_exists( ('last_backup_copy_to_'.$targetName.'_timestamp'), $assetsByInstanceID[$backupInstanceID]) === false )
                            {
                                $assetsByInstanceID[$backupInstanceID]['last_backup_copy_to_'.$targetName.'_timestamp'] = 0;
                                $assetsByInstanceID[$backupInstanceID]['backups_copies_to_'.$targetName.'_count_by_status'] = array('successes'=>0, 'failures'=>0);
                                $assetsByInstanceID[$backupInstanceID]['backup_copy_types_to_'.$targetName] = array(  Constants::BACKUP_DISPLAY_TYPE_FULL=>false,
                                    Constants::BACKUP_DISPLAY_TYPE_DIFFERENTIAL=>false,
                                    Constants::BACKUP_DISPLAY_TYPE_INCREMENTAL=>false,
                                    Constants::BACKUP_TYPE_BAREMETAL=>false, // the display type for Bare Metal is 2 words, so I used the backup type as the key
                                    Constants::BACKUP_DISPLAY_TYPE_TRANSACTION=>false);
                                $assetsByInstanceID[$backupInstanceID]['backup_copy_targets'][] = $targetName;
                                $assetsByInstanceID[$backupInstanceID]['next_backup_copy_to_'.$targetName.'_timestamp'] = 0;
                                $assetsByInstanceID[$backupInstanceID]['next_backup_copy_to_'.$targetName.'_joborder'] = '';
                            }

                            if ( $archive_backup['success'] === true )
                            {
                                if ($assetsByInstanceID[$backupInstanceID]['protected_by_backup_copy'] === false)
                                {
                                    $assetsByInstanceID[$backupInstanceID]['protected_by_backup_copy'] = true;
                                    $assets_with_a_backup_copy_count += 1;
                                }
                                if ($archive_backup['archive_time'] > $assetsByInstanceID[$backupInstanceID]['last_backup_copy_to_' . $targetName . '_timestamp'])
                                {
                                    $assetsByInstanceID[$backupInstanceID]['last_backup_copy_to_' . $targetName . '_timestamp'] = $archive_backup['archive_time'];
                                }
                                $assetsByInstanceID[$backupInstanceID]['backup_copy_types_to_' . $targetName][$backupType] = true;
                                $assetsByInstanceID[$backupInstanceID]['backups_copies_to_' . $targetName . '_count_by_status']['successes'] += 1;
                                $number_of_all_backup_copies_count_array['successes'] += 1;
                                $number_of_backup_copies_per_target_count_array[$targetName]['successes'] += 1;
                            }
                            else
                            {
                                $assetsByInstanceID[$backupInstanceID]['backups_copies_to_'.$targetName.'_count_by_status']['failures'] += 1;
                                $number_of_all_backup_copies_count_array['failures'] += 1;
                                $number_of_backup_copies_per_target_count_array[$targetName]['failures'] += 1;
                            }
                        }
                    }
                }
                else
                {
                    $report = false;  //Fix this logic
                }

                if ( $end_time === false )
                {
                    //Only do future entries if the end time is now
                    $schedules = new Schedule($this->BP);
                    $scheduleList = $schedules->get(-1, array($system_id => $system_name), true);
                    if ( $scheduleList !== false and is_array($scheduleList) and array_key_exists('data', $scheduleList) )
                    {
                        foreach ( $scheduleList['data'] as $schedule )
                        {
                            if ( array_key_exists( 'instance_ids', $schedule ) )
                            {
                                $instances = $schedule['instance_ids'];
                            }
                            else
                            {
                                $instances = array();
                            }

                            if ( array_key_exists( 'client_ids', $schedule ) )
                            {
                                foreach ( $schedule['client_ids'] as $client_id_in_schedule ) {
                                    $instances[] = $fileLevelInstanceIDByClientID[$client_id_in_schedule];
                                }
                            }

                            if ( $schedule['enabled'] == true )
                            {
                                if ( array_key_exists('next_time',$schedule) and $schedule['next_time'] !== "Never" )
                                {
                                    $nextRun = strtotime($schedule['next_time']);
                                }
                                else
                                {
                                    $nextRun = 0;
                                }

                                switch ( $schedule['type'] )
                                {
                                    case Constants::JOBORDER_TYPE_BACKUP:
                                        foreach ( $instances as $instance_id_in_schedule )
                                        {
                                            if ( $assetsByInstanceID[$instance_id_in_schedule]['next_backup_timestamp'] === 0 or $assetsByInstanceID[$instance_id_in_schedule]['next_backup_timestamp'] > $nextRun )
                                            {
                                                $assetsByInstanceID[$instance_id_in_schedule]['next_backup_timestamp'] = $nextRun;
                                                $assetsByInstanceID[$instance_id_in_schedule]['next_backup_joborder'] = $schedule['name'];
                                            }
                                        }
                                        break;
                                    case Constants::JOBORDER_TYPE_ARCHIVE:
                                    case Constants::JOBORDER_TYPE_REPLICATION:
                                        if ( array_key_exists('target',$schedule) and $schedule['target'] !== '' )
                                        {
                                            if ( array_key_exists( $schedule['target'], $number_of_backup_copies_per_target_count_array) === false )
                                            {
                                                $number_of_backup_copies_per_target_count_array[$schedule['target']] = array('successes'=>0, 'failures'=>0);
                                                $backup_copy_assets_tables['backup_copies_to_'.$schedule['target']] = array();
                                            }
                                            foreach ( $instances as $instance_id_in_schedule )
                                            {
                                                $assetsByInstanceID[$instance_id_in_schedule]['in_a_backup_copy_joborder'] = true;
                                                if ( array_key_exists( ('last_backup_copy_to_'.$schedule['target'].'_timestamp'), $assetsByInstanceID[$instance_id_in_schedule]) === false )
                                                {
                                                    $assetsByInstanceID[$instance_id_in_schedule]['last_backup_copy_to_'.$schedule['target'].'_timestamp'] = 0;
                                                    $assetsByInstanceID[$instance_id_in_schedule]['backups_copies_to_'.$schedule['target'].'_count_by_status'] = array('successes'=>0, 'failures'=>0);
                                                    $assetsByInstanceID[$instance_id_in_schedule]['backup_copy_types_to_'.$schedule['target']] = array(  Constants::BACKUP_DISPLAY_TYPE_FULL=>false,
                                                        Constants::BACKUP_DISPLAY_TYPE_DIFFERENTIAL=>false,
                                                        Constants::BACKUP_DISPLAY_TYPE_INCREMENTAL=>false,
                                                        Constants::BACKUP_TYPE_BAREMETAL=>false, // the display type for Bare Metal is 2 words, so I used the backup type as the key
                                                        Constants::BACKUP_DISPLAY_TYPE_TRANSACTION=>false);
                                                    $assetsByInstanceID[$instance_id_in_schedule]['backup_copy_targets'][] = $schedule['target'];
                                                    $assetsByInstanceID[$instance_id_in_schedule]['next_backup_copy_to_'.$schedule['target'].'_timestamp'] = $nextRun;
                                                    $assetsByInstanceID[$instance_id_in_schedule]['next_backup_copy_to_'.$schedule['target'].'_joborder'] = $schedule['name'];
                                                }
                                                elseif ( ($assetsByInstanceID[$instance_id_in_schedule]['next_backup_copy_to_'.$schedule['target'].'_timestamp'] === 0) or ($assetsByInstanceID[$instance_id_in_schedule]['next_backup_copy_to_'.$schedule['target'].'_timestamp'] > $nextRun) )
                                                {
                                                    $assetsByInstanceID[$instance_id_in_schedule]['next_backup_copy_to_'.$schedule['target'].'_timestamp'] = $nextRun;
                                                    $assetsByInstanceID[$instance_id_in_schedule]['next_backup_copy_to_'.$schedule['target'].'_joborder'] = $schedule['name'];
                                                }
                                            }
                                        }
                                        break;
                                }
                            }
                        }
                    }
                }

                foreach ( $assetsByInstanceID as $asset )
                {
                    if ( $asset['client_name'] !== NULL )
                    {
                        $temp_asset_table_array = array('system_name' => $system_name,
                            'client_name' => $asset['client_name'],
                            'instance_name' => $asset['instance_name'],
                            'asset_type' => $asset['asset_type'],
                            'protected' => $asset['protected'],
                            'protected_by_backup_copy' => $asset['protected_by_backup_copy']);
                        if ( $asset['protected'] === true or $asset['next_backup_timestamp'] !== 0 )
                        {
                            $temp_backup_table_row_array = array('system_name' => $system_name,
                                'client_name' => $asset['client_name'],
                                'instance_name' => $asset['instance_name'],
                                'asset_type' => $asset['asset_type'] );
                            $temp_backup_types_array = array();
                            if ( $asset['backup_types'][Constants::BACKUP_DISPLAY_TYPE_FULL] === true )
                            {
                                $temp_backup_types_array[] = Constants::BACKUP_DISPLAY_TYPE_FULL;
                            }
                            if ( $asset['backup_types'][Constants::BACKUP_DISPLAY_TYPE_DIFFERENTIAL] === true )
                            {
                                $temp_backup_types_array[] = Constants::BACKUP_DISPLAY_TYPE_DIFFERENTIAL;
                            }
                            if ( $asset['backup_types'][Constants::BACKUP_DISPLAY_TYPE_INCREMENTAL] === true )
                            {
                                $temp_backup_types_array[] = Constants::BACKUP_DISPLAY_TYPE_INCREMENTAL;
                            }
                            if ( $asset['backup_types'][Constants::BACKUP_TYPE_BAREMETAL] === true )
                            {
                                $temp_backup_types_array[] = Constants::BACKUP_DISPLAY_TYPE_BAREMETAL;
                            }
                            if ( $asset['backup_types'][Constants::BACKUP_DISPLAY_TYPE_SELECTIVE] === true )
                            {
                                $temp_backup_types_array[] = Constants::BACKUP_DISPLAY_TYPE_SELECTIVE;
                            }
                            if ( $asset['backup_types'][Constants::BACKUP_DISPLAY_TYPE_TRANSACTION] === true )
                            {
                                $temp_backup_types_array[] = Constants::BACKUP_DISPLAY_TYPE_TRANSACTION;
                            }
                            $temp_backup_table_row_array['backup_types'] = $temp_backup_types_array;
                            $temp_backup_table_row_array['successes'] = $asset['backups_count_by_status']['successes'];
                            $temp_backup_table_row_array['warnings'] = $asset['backups_count_by_status']['warnings'];
                            $temp_backup_table_row_array['failures'] = $asset['backups_count_by_status']['failures'];

                            if ( $asset['last_backup_timestamp'] !== 0 )
                            {
                                $temp_asset_table_array['last_backup'] = date(Constants::DATE_FORMAT_DATE_THEN_12HR_TIME, $asset['last_backup_timestamp']);
                                $temp_backup_table_row_array['last_backup'] = date(Constants::DATE_FORMAT_DATE_THEN_12HR_TIME, $asset['last_backup_timestamp']);
                            }

                            if ( $asset['next_backup_timestamp'] !== 0 )
                            {
                                $temp_backup_table_row_array['next_backup'] = date(Constants::DATE_FORMAT_DATE_THEN_12HR_TIME, $asset['next_backup_timestamp']);
                                $temp_backup_table_row_array['next_backup_joborder'] = $asset['next_backup_joborder'];
                            }
                            elseif ( $asset['next_backup_joborder'] !== '' )
                            {
                                $temp_backup_table_row_array['next_backup_joborder'] = $asset['next_backup_joborder'];
                            }

                            $backup_assets_table[] = $temp_backup_table_row_array;
                        }
                        if ( $asset['protected_by_backup_copy'] === true or $asset['in_a_backup_copy_joborder'] === true )
                        {

                            $temp_backup_copy_table_row_array_base = array(  'system_name' => $system_name,
                                'client_name' => $asset['client_name'],
                                'instance_name' => $asset['instance_name'],
                                'asset_type' => $asset['asset_type'] );
                            foreach ( $asset['backup_copy_targets'] as $backupCopyTargetName )
                            {
                                $temp_backup_copy_table_row_array_per_target = $temp_backup_copy_table_row_array_base;
                                $temp_backup_copy_types_array = array();
                                if ( $asset['backup_copy_types_to_'.$backupCopyTargetName][Constants::BACKUP_DISPLAY_TYPE_FULL] === true )
                                {
                                    $temp_backup_copy_types_array[] = Constants::BACKUP_DISPLAY_TYPE_FULL;
                                }
                                if ( $asset['backup_copy_types_to_'.$backupCopyTargetName][Constants::BACKUP_DISPLAY_TYPE_DIFFERENTIAL] === true )
                                {
                                    $temp_backup_copy_types_array[] = Constants::BACKUP_DISPLAY_TYPE_DIFFERENTIAL;
                                }
                                if ( $asset['backup_copy_types_to_'.$backupCopyTargetName][Constants::BACKUP_DISPLAY_TYPE_INCREMENTAL] === true )
                                {
                                    $temp_backup_copy_types_array[] = Constants::BACKUP_DISPLAY_TYPE_INCREMENTAL;
                                }
                                if ( $asset['backup_copy_types_to_'.$backupCopyTargetName][Constants::BACKUP_TYPE_BAREMETAL] === true )
                                {
                                    $temp_backup_copy_types_array[] = Constants::BACKUP_DISPLAY_TYPE_BAREMETAL;
                                }
                                if ( $asset['backup_copy_types_to_'.$backupCopyTargetName][Constants::BACKUP_DISPLAY_TYPE_SELECTIVE] === true )
                                {
                                    $temp_backup_copy_types_array[] = Constants::BACKUP_DISPLAY_TYPE_SELECTIVE;
                                }
                                if ( $asset['backup_copy_types_to_'.$backupCopyTargetName][Constants::BACKUP_DISPLAY_TYPE_TRANSACTION] === true )
                                {
                                    $temp_backup_copy_types_array[] = Constants::BACKUP_DISPLAY_TYPE_TRANSACTION;
                                }
                                $temp_backup_copy_table_row_array_per_target['backup_copy_types'] = $temp_backup_copy_types_array;
                                $temp_backup_copy_table_row_array_per_target['backup_copy_successes'] = $asset['backups_copies_to_'.$backupCopyTargetName.'_count_by_status']['successes'];
                                $temp_backup_copy_table_row_array_per_target['backup_copy_failures'] = $asset['backups_copies_to_'.$backupCopyTargetName.'_count_by_status']['failures'];

                                if ( $asset['last_backup_copy_to_'.$backupCopyTargetName.'_timestamp'] !== 0 )
                                {
                                    $temp_asset_table_array['last_backup_copy_to_'.$backupCopyTargetName] = date(Constants::DATE_FORMAT_DATE_THEN_12HR_TIME, $asset['last_backup_copy_to_'.$backupCopyTargetName.'_timestamp']);
                                    $temp_backup_copy_table_row_array_per_target['last_backup_copy'] = date(Constants::DATE_FORMAT_DATE_THEN_12HR_TIME, $asset['last_backup_copy_to_'.$backupCopyTargetName.'_timestamp']);
                                }

                                if ( $asset['next_backup_copy_to_'.$backupCopyTargetName.'_timestamp'] !== 0 )
                                {
                                    $temp_backup_copy_table_row_array_per_target['next_backup_copy_to_'.$backupCopyTargetName] = date(Constants::DATE_FORMAT_DATE_THEN_12HR_TIME, $asset['next_backup_copy_to_'.$backupCopyTargetName.'_timestamp']);
                                    $temp_backup_copy_table_row_array_per_target['next_backup_copy_to_'.$backupCopyTargetName.'_joborder'] = $asset['next_backup_copy_to_'.$backupCopyTargetName.'_joborder'];
                                }
                                elseif ( $asset['next_backup_copy_to_'.$backupCopyTargetName.'_joborder'] !== '' )
                                {
                                    $temp_backup_copy_table_row_array_per_target['next_backup_copy_to_'.$backupCopyTargetName.'_joborder'] = $asset['next_backup_copy_to_'.$backupCopyTargetName.'_joborder'];
                                }

                                // The target name is returned twice, so this might be unnecessary and is only included for 100% clarity.
                                //$temp_backup_copy_table_row_array_per_target['target'] = $backupCopyTargetName;

                                $backup_copy_assets_tables['backup_copies_to_'.$backupCopyTargetName][] = $temp_backup_copy_table_row_array_per_target;
                            }
                        }
                        $assets_table[] = $temp_asset_table_array;
                    }
                }
            }

            foreach ( $number_of_backup_copies_per_target_count_array as $backup_copy_target_name => $backup_copy_count_per_target )
            {
                $backup_copy_assets_tables['backup_copies_to_'.$backup_copy_target_name] = array_merge( $backup_copy_count_per_target, array('target'=>$backup_copy_target_name, 'assets'=>$backup_copy_assets_tables['backup_copies_to_'.$backup_copy_target_name]) );
            }

            $report_data['protected_assets'] = $assets_with_a_backup_count;
            $report_data['unprotected_assets'] = $total_number_of_assets - $assets_with_a_backup_count;
            $report_data['assets_protected_by_backup_copy'] = $assets_with_a_backup_copy_count;
            $report_data['assets_not_protected_by_backup_copy'] = $total_number_of_assets - $assets_with_a_backup_copy_count;

            $report_data['assets'] = $assets_table;
            $report_data['backups'] = array_merge( $number_of_backups_count_array, array('assets'=>$backup_assets_table) );
            $report_data['backup_copies'] = array_merge( $number_of_all_backup_copies_count_array, array('assets'=>$backup_copy_assets_tables) );

            //Temporary logic
            $report_count = $total_number_of_assets;

            $report['count'] = $report_count;
            $report['total'] = $report_count;//$report_total;
            $report['start'] = $report_start;
            $report['data'] = $report_data;
        } else {
            $report['error'] = 500;
            $report['message'] = "Invalid date range provided";
        }
        return $report;
    }

    private function initializeAssetArray( $clientName, $instanceName, $type )
    {
        return array(  'client_name'=>$clientName,
            'instance_name'=>$instanceName,
            'asset_type'=>$type,
            'protected'=>false,
            'protected_by_backup_copy'=>false,
            'last_backup_timestamp'=>0,
            'next_backup_timestamp'=>0,
            'next_backup_joborder'=>'',
            'in_a_backup_copy_joborder'=>false,
            'backups_count_by_status'=>array('successes'=>0, 'warnings'=>0, 'failures'=>0),
            'backup_types'=>array(  Constants::BACKUP_DISPLAY_TYPE_FULL=>false,
                Constants::BACKUP_DISPLAY_TYPE_DIFFERENTIAL=>false,
                Constants::BACKUP_DISPLAY_TYPE_INCREMENTAL=>false,
                Constants::BACKUP_TYPE_BAREMETAL=>false, // the display type for Bare Metal is 2 words, so I used the backup type as the key
                Constants::BACKUP_DISPLAY_TYPE_SELECTIVE=>false,
                Constants::BACKUP_DISPLAY_TYPE_TRANSACTION=>false),
            'backup_copy_targets'=>array() );
    }
}