<?php

$dir = '/api/includes/reports';

class ReplicationCapacityReport
{
    private $BP;

    public function __construct( $BP, $functionsClass )
    {
        $this->BP = $BP;
        $this->functions = $functionsClass;
    }

    public function get_replication_capacity_report($sid, $data)
    {
        $systems = $this->functions->selectReplicatingSystems($sid);

        $report = array();
        $report_count = 0;
        //Temporary until more data is available
        $report_total = $report_count;
        $report_start = 1;
        $report_data = array();

        $localSystemID = $this->BP->get_local_system_id();

        foreach ($systems as $system_id => $system_name)
        {
            $grandclients = $this->BP->get_grandclient_list($system_id);
            if ( $grandclients !== false and count($grandclients) > 0 )
            {
                foreach ($grandclients as $grandclientID => $grandclientName)
                {
                    $fileLevelRetentionInfoList = $this->BP->get_grandclient_retention_info(array('system_id' => $system_id,
                        'client_id' => $grandclientID,
                        'app_id' => Constants::APPLICATION_ID_FILE_LEVEL));
                    if ($fileLevelRetentionInfoList !== false and count($fileLevelRetentionInfoList) > 0)
                    {
                        foreach ($fileLevelRetentionInfoList as $retentionInfo)
                        {
                            $report_data[] = $this->buildReplicationCapacityPerInstance($retentionInfo['app_name'],
                                $grandclientName,
                                $retentionInfo['mb_size'],
                                $system_name);
                            $report_count += 1;
                        }
                    }

                    $hyperVRetentionInfoList = $this->BP->get_grandclient_retention_info( array('system_id' => $system_id,
                        'client_id' => $grandclientID,
                        'app_id' => Constants::APPLICATION_ID_HYPER_V_2008_R2) );

                    $hyperVRetentionInfoList = array_merge($hyperVRetentionInfoList, $this->BP->get_grandclient_retention_info( array('system_id' => $system_id,
                        'client_id' => $grandclientID,
                        'app_id' => Constants::APPLICATION_ID_HYPER_V_2012) ));

                    if ($hyperVRetentionInfoList !== false and count($hyperVRetentionInfoList) > 0)
                    {
                        foreach ($hyperVRetentionInfoList as $retentionInfo)
                        {
                            $nameArray = explode('|', $retentionInfo['name']);
                            $report_data[] = $this->buildReplicationCapacityPerInstance( $retentionInfo['app_name'],
                                $grandclientName,
                                $retentionInfo['mb_size'],
                                $system_name,
                                $nameArray[0] );
                            $report_count += 1;
                        }
                    }

                    $VMwareRetentionInfoList = $this->BP->get_grandclient_retention_info(array('system_id' => $system_id,
                        'client_id' => $grandclientID,
                        'app_id' => Constants::APPLICATION_ID_VMWARE));

                    if ($VMwareRetentionInfoList !== false and count($VMwareRetentionInfoList) > 0)
                    {
                        foreach ($VMwareRetentionInfoList as $retentionInfo)
                        {
                            $nameArray = explode('|', $retentionInfo['name']);
                            $report_data[] = $this->buildReplicationCapacityPerInstance( $retentionInfo['app_name'],
                                $nameArray[0],
                                $retentionInfo['mb_size'],
                                $system_name,
                                $nameArray[1] );
                            $report_count += 1;
                        }
                    }
                }
            }
        }
        $report_total = $report_count;  //Temporary

        $report['count'] = $report_count;
        $report['total'] = $report_total;
        $report['start'] = $report_start;
        $report['data'] = $report_data;
        return $report;

    }

    private function buildReplicationCapacityPerInstance( $appName, $clientName, $size, $systemName, $instanceName = '', $databaseName = '')
    {
        $replication_capacity_info = array();
        $replication_capacity_info['app_name'] = $appName;
        $replication_capacity_info['client_name'] = $clientName;
        $replication_capacity_info['instance_name'] = $instanceName;
        $replication_capacity_info['database_name'] = $databaseName;
        $replication_capacity_info['size'] = $size;
        $replication_capacity_info['system_name'] = $systemName;
        //$replication_capacity_info['retention_policy'] = 'CoreWorkNeeded'; //Not available for 1st Out
        //$replication_capacity_info['retention_id'] = 0; //Not available for 1st Out

        return $replication_capacity_info;
    }

}