<?php

$dir = '/api/includes/reports';

class UpdateHistoryReport
{
    private $BP;

    public function __construct( $BP, $functionsClass )
    {
        $this->BP = $BP;
        $this->functions = $functionsClass;
    }

    public function get_system_update_history($systems, $data)
    {
        $report = array();
        $report_count = 0;
        //Temporary until more data is available
        $report_total = $report_count;
        $report_start = 1;
        $report_data = array();

        $start_time = false;
        if (array_key_exists('start_date', $data)) {
            $start_time = strtotime($data['start_date']);
        }
        $end_time = false;
        if (array_key_exists('end_date', $data)) {
            $end_time = $this->functions->formatEndDate($data['end_date']);
        }

        $isValidDate = $this->functions->isValidDateRange($start_time, $end_time, false);

        if($isValidDate) {
            foreach ($systems as $system_id => $system_name)
            {

                $update_history_result_format = array();
                $update_history_result_format['system_id'] = $system_id;

                if ($data !== NULL and count($data) > 0)
                {
                    if ($start_time !== false) {
                        $update_history_result_format['start_time'] = $start_time;
                    }
                    if ($end_time !== false) {
                        $update_history_result_format['end_time'] = $end_time;
                    }
                    if ( array_key_exists('order', $data) )
                    {
                        $update_history_result_format['order'] = $data['order'];
                    }
                    /*
                     ***************** IMPLEMENTED IN FUTURE *******************
                     if ( array_key_exists('format', $data) )
                    {
                        $update_history_result_format['format'] = $data['format'];
                    }
                    */
                    if ( array_key_exists('start_row', $data) )
                    {
                        $update_history_result_format['start_row'] = (int)$data['start_row'];
                    }
                    if ( array_key_exists('count', $data) )
                    {
                        $update_history_result_format['count'] = (int)$data['count'];
                    }
                }

                $update_history = $this->BP->get_update_history($update_history_result_format);

                if ( $update_history !== false and count($update_history) > 0 )
                {
                    foreach($update_history as $update)
                    {
                        $temp_update_history = array();
                        $temp_update_history['start_date'] = date(Constants::DATE_TIME_FORMAT_US, $update['start_date']);
                        $temp_update_history['end_date'] = ( $update['end_date'] !== 0 ? date(Constants::DATE_TIME_FORMAT_US, $update['end_date']) : "" );
                        $temp_update_history['system_name'] = $update['system_name'];
                        $temp_update_history['version_from'] = $update['version_from'];
                        $temp_update_history['version_to'] = $update['version_to'];
                        $temp_update_history['status'] = $update['status'];
                        $temp_update_history['error_count'] = $update['error_count'];
                        $temp_update_history['messages'] = ( array_key_exists('messages', $update) ? $update['messages'] : array() );

                        if ( array_key_exists('error_list', $update) and count($update['error_list']) > 0 )
                        {
                            foreach ( $update['error_list'] as $update_error_message )
                            {
                                $temp_update_history['messages'][] = $update_error_message;
                            }
                        }


                        $report_data[] = $temp_update_history;
                        $report_count += 1;
                    }
                }
                /*  Comment this out until partial failure is implemented
                else
                {
                    $report = false;
                }*/
            }
            $report_total = $report_count;  //Temporary

            $report['count'] = $report_count;
            $report['total'] = $report_total;
            $report['start'] = $report_start;
            $report['data'] = $report_data;
        } else {
            $report['error'] = 500;
            $report['message'] = "Invalid date range provided";
        }
        return $report;
    }

}