<?php

class Summary
{
    private $BP;
     
    public function __construct($BP, $sid)
    {
		$this->BP = $BP;
        $this->now = time();
        $this->sid = $sid == false ? $sid = $this->BP->get_local_system_id() : $sid;
    }
 
    public function get($which, $filter) {
        $sid = $this->sid;
        switch ($which){
            case "current":
                $summaryCurrent = $this->BP->get_summary_current($sid);
                if ($summaryCurrent !== false){
                    $data = $this->buildOutput($summaryCurrent);
                } else {
                    $data = false;
                }

                break;
            case "counts" :
                $summaryCounts = $this->BP->get_summary_counts($filter, $sid);
				// If the call fails, return false so that buildResult will get the error message.
				if ($summaryCounts !== false) {
					$data = $this->buildCountsOutput($summaryCounts, $filter);
				} else {
					$data = false;
				}
				break;
            case "days" :
                $subsystem = $filter;
                $days = isset($_GET['days']) ? $_GET['days'] : 7;
                $summaryDays = $this->BP->get_summary_days($subsystem, $days, $sid);
                if ($summaryDays !== false) {
                    foreach ($summaryDays as $id => $day){
                        $data['data'][] = $this->buildDaysOutput($id, $day);
                    }
                } else {
                    $data = false;
                }
                break;

            case "target-days" :
                $subsystem = $filter;
                $days = isset($_GET['days']) ? $_GET['days'] : 7;
                $summaryDays = $this->BP->get_target_summary_days($subsystem, $days, $sid);
                if ($summaryDays !== false) {
                    $i = 1;
                    foreach ($summaryDays as $replicatingSources){
                        $daysData = array();
                        $sourceKey = "source." . $i++;
                        foreach ($replicatingSources as $id => $day) {
                            if ($id === 'source') {
                                $sourceKey = $day;
                            } else {
                                $daysData[] = $this->buildDaysOutput($id, $day);
                            }
                        }
                        $data['data'][$sourceKey] = $daysData;
                    }
                } else {
                    $data = false;
                }
                break;
            }

        return $data;
    }

    function buildOutput($summary) {

        include_once 'function.lib.php';
        $functions = new Functions();

        $alerts = isset($summary['alert_count']) ? $summary['alert_count'] : 0;
        // Don't return time of a sync unless there has been one (sync_last > 0)).
        $syncLast = isset($summary['sync_last']) && $summary['sync_last'] > 0 ? $functions->formatDateTime($summary['sync_last']) : null;
        $syncProgress = isset($summary['sync_progress']) ? $summary['sync_progress'] : 0;
        $syncRunning = isset($summary['sync_running']) ? $summary['sync_running'] : false;
        $syncStatus = isset($summary['sync_status']) ? $summary['sync_status'] : 'n/a';
        // Only return a message if there is one.
        $message = isset($summary['message']) && $summary['message'] != '(No message available.)' ? $summary['message'] : false;

        $data = array(
            'alert_count' => $alerts,
            'sync_last' => $syncLast,
            'sync_progress' => $syncProgress,
            'sync_running' => $syncRunning,
            'sync_status' => $syncStatus,
            'message' => $message
        );

        return $data;
    }

    function buildCountsOutput($summary, $filter) {
        // if the API failed, $data is false.
        if ($summary === false) {
            $data = $summary;
            return;
        }

        $backupErrors = isset($summary['backup_errors']) ? $summary['backup_errors'] : null;
        $backupNotProtected = isset($summary['backup_not_protected']) ? $summary['backup_not_protected'] : null;
        $backupProtected = isset($summary['backup_protected']) ? $summary['backup_protected'] : null;
        $backupProtected_RPO = isset($summary['rpo_protected']) ? $summary['rpo_protected'] : 0;  //recovery point objective - the instances with backups in the last $backupRPODays
        $backupRPODays = isset($summary['rpo_days']) ? $summary['rpo_days'] : null;
        $replicationErrors = isset($summary['replication_errors']) ? $summary['replication_errors'] : null;
        $replicationProtected = isset($summary['replication_protected']) ? $summary['replication_protected'] : null;
        $replicatedBackupErrors = isset($summary['replicated_backup_errors']) ? $summary['replicated_backup_errors'] : null;
        $replicatedBackupProtected = isset($summary['replicated_backup_protected']) ? $summary['replicated_backup_protected'] : null;
        $replicatedBackupNotProtected = isset($summary['replicated_backup_not_protected']) ? $summary['replicated_backup_not_protected'] : null;
        $restoreActive = isset($summary['restore_active']) ? $summary['restore_active'] : null;
        $restoreActiveFLR = isset($summary['restore_active_flr']) ? $summary['restore_active_flr'] : null;
        $restoreActiveIR = isset($summary['restore_active_ir']) ? $summary['restore_active_ir'] : null;
        $restoreRecent = isset($summary['restore_recent']) ? $summary['restore_recent'] : null;
        $archiveErrors = isset($summary['archive_errors']) ? $summary['archive_errors'] : null;
        $archiveProtected = isset($summary['archive_protected']) ? $summary['archive_protected'] : null;
        $WIRErrors = isset($summary['wir_errors']) ? $summary['wir_errors'] : null;
        $WIRProtected = isset($summary['wir_protected']) ? $summary['wir_protected'] : null;


        $data = array();

        $backup = array(
            'backup_errors' => $backupErrors,
            'backup_not_protected' =>($backupNotProtected+$backupProtected-$backupProtected_RPO), //UNIBP-6122 - Changing numbers until core work associated with UNIBP-6219 is in
            'backup_protected' => $backupProtected_RPO,  //UNIBP-6122 - Changing numbers until core work associated with UNIBP-6219 is in
            'rpo_protected' => $backupProtected_RPO,
            'rpo_days' => $backupRPODays,
            'backup_not_protected-not_adjusted' =>$backupNotProtected, //UNIBP-6122 - Changing numbers until core work associated with UNIBP-6219 is in
            'backup_protected-not_adjusted' => $backupProtected //UNIBP-6122 - Changing numbers until core work associated with UNIBP-6219 is in
        );
        $replication = array(
            'replication_errors' => $replicationErrors,
            'replication_protected' => $replicationProtected
        );
        $replicatedBackup = array(
            'replicated_backup_errors' => $replicatedBackupErrors,
            'replicated_backup_protected' => $replicatedBackupProtected,
            'replicated_backup_not_protected' => $replicatedBackupNotProtected
        );
        $restore = array(
            'restore_active' => $restoreActive,
            'restore_active_flr' => $restoreActiveFLR,
            'restore_active_ir' => $restoreActiveIR,
            'restore_recent' => $restoreRecent
        );
        $archive = array(
            'archive_errors' => $archiveErrors,
            'archive_protected' => $archiveProtected
        );
        $wir = array(
            'wir_errors' => $WIRErrors,
            'wir_protected' => $WIRProtected
        );


        switch($filter){
            case 'backup':
                $data = $backup;
                break;
            case 'replication':
                $data = $replication;
                break;
            case 'replicated_backup':
                $data = $replicatedBackup;
                break;
            case 'restore':
                $data = $restore;
                break;
            case 'archive':
                $data = $archive;
                break;
            case 'wir':
                $data = $wir;
                break;
            case null:
                $data = array_merge($backup, $replication, $replicatedBackup, $restore, $archive, $wir);
                break;
            default:
                $data = "The filter '" + $filter + "' is not valid.";
                break;
        }

        return $data;
    }

    function buildDaysOutput($id, $day){
        $data = array();

        if ($id === false) {
            $data = $id;
            return $data;
        }

        if (is_array($day)) {
            foreach ($day as $d) {
                $avgSpeed = isset($d['avg_speed']) ? round($d['avg_speed'],2) : null;
                $units = isset($d['units']) ? $d['units'] : null;
                $avgTime = isset($d['avg_time']) ? $d['avg_time'] : null;
                $singleDay = isset($d['day']) ? $d['day'] : null;
                $read = isset($d['read']) ? round($d['read'],2) : null;
                $write = isset($d['write']) ? round($d['write'], 2) : null;
                $compress = isset($d['compress']) ? round($d['compress'], 2) : null;
                $avgStartLag = isset($d['avg_start_lag']) ? round($d['avg_start_lag'], 2) : null;
                $avgProtectionGap = isset($d['avg_prot_gap']) ? round($d['avg_prot_gap'], 2) : null;
                $successes = isset($d['successes']) ? $d['successes'] : null;
                $retries = isset($d['retries']) ? $d['retries'] : null;

                $backupSpeed = array(
                    'avg_speed' => $avgSpeed,
                    'units' => $units,
                    'avg_time' => $avgTime,
                    'day' => $singleDay
                );

                $backupSize = array(
                    'read' => $read,
                    'write' => $write,
                    'compress' => $compress,
                    'units' => $units,
                    'day' => $singleDay
                );

                $backupDataSpeed = array(
                    'read' => $read,
                    'write' => $write,
                    'compress' => $compress,
                    'units' => $units,
                    'day' => $singleDay
                );

                $replicationSpeed = array(
                    'read' => $read,
                    'write' => $write,
                    'units' => $units,
                    'day' => $singleDay
                );

                $replicationSize = array(
                    'read' => $read,
                    'write' => $write,
                    'units' => $units,
                    'day' => $singleDay
                );

                $replicationMetrics = array(
                    'start_lag' => $avgStartLag,
                    'protection_gap' => $avgProtectionGap,
                    'units' => $units,
                    'day' => $singleDay
                );

                $replicationStatistics = array(
                    'successes' => $successes,
                    'retries' => $retries,
                    'day' => $singleDay
                );

                $restoreSpeed = array(
                    'avg_speed' => $avgSpeed,
                    'units' => $units,
                    'avg_time' => $avgTime,
                    'day' => $singleDay
                );

                $archiveSpeed = array(
                    'avg_speed' => $avgSpeed,
                    'units' => $units,
                    'avg_time' => $avgTime,
                    'day' => $singleDay
                );

                $archiveSize = array(
                    'write' => $write,
                    'units' => $units,
                    'day' => $singleDay
                );

                switch ($id) {
                    case 'backup_speed':
                        $data[$id][] = $backupSpeed;
                        break;
                    case 'backup_size':
                        $data[$id][] = $backupSize;
                        break;
                    case 'backup_data_speed':
                        $data[$id][] = $backupDataSpeed;
                        break;
                    case 'replication_speed':
                    case 'replicating_speed':
                        $data[$id][] = $replicationSpeed;
                        break;
                    case 'replication_size':
                    case 'replicating_size':
                        $data[$id][] = $replicationSize;
                        break;
                    case 'replication_metrics':
                    case 'replicating_metrics':
                        $data[$id][] = $replicationMetrics;
                        break;
                    case 'replication_stats':
                    case 'replicating_stats':
                        $data[$id][] = $replicationStatistics;
                        break;
                    case 'restore_speed':
                        $data[$id][] = $restoreSpeed;
                        break;
                    case 'archive_speed':
                        $data[$id][] = $archiveSpeed;
                        break;
                    case 'archive_size':
                        $data[$id][] = $archiveSize;
                        break;
                }
            }
        }

        return $data;
    }

} // end Summary

?>
