<?php
// defines BP, xml, action, cookie
require 'header.php';

// general library functions
require 'library.php';
$lib = new Library();

// VCenter-RRC is a special client.
define("VCENTER_RRC", "vCenter-RRC");

$localID = $BP->get_local_system_id();
$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $localID;

// figure out which sysID to used based on view.
$sysID = $grandClientView ? $localID : $dpuID;

switch ($action) {

	case "list":	// list clients
			$instanceID = isset($_GET['id']) ? (int)$_GET['id'] : -1;
			$appInfo = $BP->get_application_clients($instanceID, $sysID);
			if ($appInfo !== false) {
				$xml->push("root");
				$appName = isset($appInfo['app_name']) ? $appInfo['app_name'] : 'unknown';
				$xml->element('Name', $appName);
				$clients = array();
				if (array_key_exists('clients', $appInfo)) {
					$clients = $appInfo['clients'];
				}
				foreach ($clients as $id => $name) {
					$xml->emptyElement('Client', array("Identification" => $id,
														 "Name" => $name));
				}
				$xml->pop();
			} else {
				$BP->buildResult($xml, false);
			}					
			echo($xml->getXml());			
			break;
			
	case "group":	// restore group
			$id = isset($_GET['id']) ? (int)$_GET['id'] : -1;
			$restoreGroup = $BP->get_restore_group($id, $sysID);
			if ($restoreGroup !== false) {
				if (isset($_GET['short'])) {
					// Display in a short format, backup IDs only, no info about each backup.
					$xml->push('Backups');
					foreach ($restoreGroup as $backup) {
						$xml->element('ID', $backup['id']);
					}
					$xml->pop();
				} else {
					if ($grandClientView) {
						$clients = $BP->get_grandclient_list($dpuID);
					} else {
						$clients = $BP->get_client_list($dpuID);
					}
					$systems = $BP->get_system_list();
					if ($restoreGroup !== false && $clients !== false && $systems !== false) {
						$dpuName = $systems[$dpuID];
						$xml->push('Backups');
						foreach ($restoreGroup as $backup) {
							if (array_key_exists($backup['client_id'], $clients)) {
								$clientName = $clients[$backup['client_id']];
							} else {
								$clientName = "unknown";
							}
							createShortBackupStatusXML($xml, $backup, $dpuName, $clientName);		
						}
						$xml->pop(); // Backups
					} else {
						$BP->buildResult($xml, false);
					}
				}
			} else {
				$BP->buildResult($xml, false);
			}					
			echo($xml->getXml());			
			break;
			
	case "backup":	// Run an application backup now.
			$clientID = isset($_GET['cid']) ? (int)$_GET['cid'] : -1;
			$backupType = $_GET['btype'];
			$metanames = isset($_GET['metanames']) ? $_GET['metanames'] : NULL;
			// move this code lower to handle app file-level backup (no instance IDs).
			//$instanceIDs = isset($_GET['ids']) ? $lib->getIDs($_GET['ids']) : "";
			$device = isset($_GET['dev']) ? $_GET['dev'] : NULL;
			$bVerify = isset($_GET['verify']) ? true : false;
			$clientInfo = array();
			$clientInfo['id'] = $clientID;
			if ($device != NULL) {
				$clientInfo['dev_name'] = $device;
			}
			if (isset($_GET['metanames'])) {
				$metaList = $_GET['metanames'];
				if ($metaList != "") {
					$metanames = explode(",", $metaList);
					$clientInfo['metanames'] = $metanames;
				} else {
					$clientInfo['metanames'] = array();
				}
			}
			$clientInfo['verify_level'] = $bVerify ? 3 : 0;
			if (isset($_REQUEST['ids'])) {
				// If instance ID's are present, pass them in.
				$instanceIDs = $lib->getIDs($_REQUEST['ids']);
				$clientInfo['appinst_ids'] = $instanceIDs;
				$clientInfoArray = $clientInfo;
			} else {
				// File level backup may have a file list.
				$files = getFiles('excludes');
				if (count($files) > 0) {
					$clientInfo['excl_list'] = $files;
				}
				$files = getFiles('includes');
				if (count($files) > 0) {
					$clientInfo['incl_list'] = $files;
				}
			}
			$clientInfoArray = $clientInfo;
			$resultArray = $BP->backup_now($backupType, $clientInfoArray, $dpuID);
			if ($resultArray === false) {
				$BP->buildResult($xml, false);
			} else {
				$xml->push("root");
				foreach ($resultArray as $result) {
					$attributes = array("ClientID" => $result['client_id'],
										"JobID" => $result['job_id']);			
					if (array_key_exists('appinst_id', $result)) {
						$attributes = array_merge($attributes, array("ID" => $result['appinst_id']));
					}
					if (array_key_exists('msg', $result)) {
						$attributes = array_merge($attributes, array("Message" => $result['msg']));
					}
					$xml->emptyelement("Backup", $attributes);
				}
				$xml->pop(); // root
			}
								
			echo($xml->getXml());			
			break;

	case "restore":	// Restore application
			if (isset($_REQUEST['ids'])) {
				$ids = $_REQUEST['ids'];
				
				if (isset($_REQUEST['includes'])) {
					$includes = $_REQUEST['includes'];
					$includesArr = explode(",", $includes);
				}

				$options = array('target' => 'target',
								 'directory' => 'dir',
								 'datastore' => 'ds',
								 'before_command' => 'bcmd',
								 'after_command' => 'acmd');				 
				

				$bOptions = array();
				$iOptions = array('client_id' => 'cid',
								  'point_in_time' => 'time');
				$appOptions = array();
				$appOptions = buildOptions($appOptions, $options, $bOptions);
				$appOptions = addIntegerOptions($appOptions, $iOptions);
				$appOptions['includes'] = $includesArr;
				
				$valid = true;
				if ($grandClientView && isVMwareBackup($ids, $sysID)) {
					// If grand client view, see if this is a VMware backup we are restoring.
					// If so, set client to a non grandclient.
					$appOptions['client_id'] = getVMwareClient($sysID);
					if ($appOptions['client_id'] == -1) {
						$valid = false;
					}
				}
				if ($valid) {
					$resultArray = $BP->restore_application($ids, $appOptions, $sysID);
					if ($resultArray !== false) {
						$xml->push("root");
						foreach ($resultArray as $result) {
							$cid = isset($result['client_id']) ? $result['client_id'] : "";
							$attributes = array("ClientID" => $cid,
												"JobID" => $result['job_id']);			
							if (array_key_exists('type', $result)) {
								$attributes = array_merge($attributes, array("Type" => getBackupTypeString($result['type'])));
							}
							if (array_key_exists('msg', $result)) {
								$attributes = array_merge($attributes, array("Message" => $result['msg']));
							}
							$xml->emptyelement("Restore", $attributes);
						}
						$xml->pop(); // root
					} else {		
						$BP->buildResult($xml, false);
					}
				} else {
					$BP->buildResult($xml, false, "Cannot find vCenter-RRC client on this system.  Please register a VMware host to this system.");
				}
			} else {
				$BP->buildResult($xml, false, "must specify ID(s) of backup(s) to restore.");
			}
			echo($xml->getXml());
			break;
			
		case "restore_app":	// Restore application
			if (isset($_REQUEST['ids'])) {
				$ids = $_REQUEST['ids'];
				//obtain client id from client name
				$clientName = isset($_GET['res']) ? $_GET['res'] : "";
				
				$cid = $BP->get_client_id($clientName, $sysID);		
				$aOptions = array();
				$appOptions = addIntegerOptions($aOptions, $cid);
				$resultArray = $BP->rae_restore_application($ids, $aOptions, $sysID);
				if ($resultArray !== false) {
					$xml->push("root");
					foreach ($resultArray as $result) {
						$cid = isset($result['client_id']) ? $result['client_id'] : " ";
						$attributes = array("ClientID" => $cid,
											"JobID" => $result['job_id']);			
						if (array_key_exists('type', $result)) {
							$attributes = array_merge($attributes, array("Type" => getBackupTypeString($result['type'])));
						}
						if (array_key_exists('msg', $result)) {
							$attributes = array_merge($attributes, array("Message" => $result['msg']));
						}
						$xml->emptyelement("Restore", $attributes);
					}
					$xml->pop(); // root
				} else {		
					$BP->buildResult($xml, false);
				}
			} else {
				$BP->buildResult($xml, false, "must specify ID(s) of backup(s) to restore.");
			}
			echo($xml->getXml());
			break;
			
	case "name":	// get name that corresponds to provided app instance.
			$aid = isset($_GET['aid']) ? (int)$_GET['aid'] : -1;
			$appInstName = $BP->get_appinst_name($aid, $sysID);
			if ($appInstName !== false) {
				$xml->push('root');
				$xml->emptyElement('Instance', array('ID' => $aid, 'Name' => $appInstName));
				$xml->pop();
			} else {
				$BP->buildResult($xml, false);
			}					
			echo($xml->getXml());			
			break;
			
	case "names":	// get names that correspond to provided app instances.
			$aids = $lib->getIDs($_GET['aids']);
			$xml->push('root');
			foreach ($aids as $aid) {
				$appInstName = $BP->get_appinst_name($aid, $sysID);
				if ($appInstName !== false) {
					$xml->emptyElement('Instance', array('ID' => $aid, 'Name' => $appInstName));
				} else {
					$xml->emptyElement('Instance', array('ID' => $aid, 'Name' => 'id ' . $aid . ', name unknown'));
				}
			}
			$xml->pop();
			echo($xml->getXml());			
			break;
			
	case "vaulting":
		  $instanceInfo = xmlToInstances($_REQUEST['instances']);		
		  $BP->buildResult($xml, $BP->save_app_vaulting_info($instanceInfo, $dpuID));  
		  echo($xml->getXml());
		  break;			
}

/*
<Instances>
  <Instance Identification="60" Name="Ashu_build_x64">
	<Priority>500</Priority>
	<Synchable>True</Synchable>
  </Instance>
</Instances>
*/
function xmlToInstances($xml)
{
	$parser = xml_parser_create();
	xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING, 0);
	xml_parser_set_option($parser, XML_OPTION_SKIP_WHITE, 1);
	xml_parse_into_struct($parser, $xml, $values, $tags);
	xml_parser_free($parser);

	$returnArray = array();
	$oneInstance = array();

	$stack = array();
	$i = 0;
	foreach ($values as $val) {
		if ($val['type'] == "open") {
			if ($val['tag'] != "Instances" && $val['tag'] != "Instance") {
				$field = buildFieldName($val['tag']);
				array_push($stack, $field);
			}

		} else if ($val['type'] == "close") {
			if ($val['tag'] != "Instances" && $val['tag'] != "Instance") {
				array_pop($stack);
			}
		} else if ($val['type'] == "complete") {
			$field = buildFieldName($val['tag']);
			
			array_push($stack, $field);
			$bInteger = isInteger($val['tag']);
			$bBoolean = isBoolean($val['tag']);
			setArrayValue($oneInstance, $stack, $val['value'], $bInteger, $bBoolean);
			array_pop($stack);
				
			if ($val['tag'] == 'Synchable') {
				$returnArray[] = $oneInstance;
				$oneInstance = array();			
			}						
		}
	}
	return $returnArray;	
}

function setArrayValue(&$array, $stack, $value, $bInteger, $bBoolean) {
	if ($stack) {
		$key = array_shift($stack);
		setArrayValue($array[$key], $stack, $value, $bInteger, $bBoolean);
	} else {
		if ($bInteger) {
			$array = (int)$value;
		} else if ($bBoolean) {
			$array = $value == 'true' ? true : false;
		} else {
			$array = $value;
		}
	}
}

function buildFieldName($tag)
{
	$field = "";
	switch ($tag) {	
		case "ID":
			$field = "instance_id";
			break;
		
		case "Priority":
			$field = "priority";
			break;
			
		case "Synchable":
			$field = "is_synchable";
			break;
	}		
	return $field;
}

function isInteger($tag)
{
	$bInteger = false;
	switch ($tag) {
		case "Priority":
			$bInteger = true;
			break;
	}
	return $bInteger;
}

function isBoolean($tag)
{
	$bBoolean = false;
	switch ($tag) {
		case "Synchable":
			$bBoolean = true;
			break;
	}
	return $bBoolean;
}

function getFiles($type) {
	$files = array();
	if (isset($_REQUEST[$type])) {
		$fileList = $_REQUEST[$type];
		if ($fileList != "") {
			$filenames = explode(",", $fileList);
			for ($i = 0; $i < count($filenames); $i++) {
				$filenames[$i] = urldecode($filenames[$i]);					
			}
			$files = $filenames;
		}
	}
	return $files;
}

//
// Given the backup IDs, returns true if a VMware backup and false otherwise.
//
function isVMwareBackup($ids, $sysID) {
	$isVMware = false;
	global $BP;
	$aIDs = split(',', $ids);
	if (is_array($aIDs) && count($aIDs) > 0) {
		$bid = array((int)$aIDs[0]);
		$result_format = array('backup_ids' => $bid, 'system_id' => $sysID);
		$statusArray = $BP->get_backup_status($result_format);
		if (is_array($statusArray) && count($statusArray) > 0) {
			$status = $statusArray[0];
			if (isset($status['app_name'])) {
				// Yes, it's VMware.
				if ($status['app_name'] == 'VMware') {
					$isVMware = true;
				}
			}
		}
	}
	return $isVMware;
}

//
// Get the vCenter-RRC client for the target.
//
function getVMwareClient($sysID) {
	$clientID = -1;
	global $BP;
	$cid = $BP->get_client_id(VCENTER_RRC, $sysID);
	if ($cid !== false) {
		$clientID = $cid;
	}
	return $clientID;
}

$BP->destroyCookie();
?>
