<?php
define( "OPEN_SSL_ENCRYPTION_METHOD", "AES-256-CBC" );

class Cipher {
    private $securekey, $iv, $useMCrypt, $useOpenSSL;
    function __construct($textkey) {
        global $Log;

        $this->securekey  = hash('sha256',$textkey,TRUE);
        $this->useMCrypt  = false; //extension_loaded( "mcrypt" );
        $this->useOpenSSL = false; //function_exists( "openssl_encrypt" ) && function_exists( "openssl_decrypt" );
        
        if ( $this->useMCrypt ) {
            $this->iv = mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_128, MCRYPT_MODE_CBC), MCRYPT_RAND);
        } else if ( $this->useOpenSSL ) {
            list($usec, $sec) = explode(' ', microtime());
            srand( (float) $sec + ((float) $usec * 100000) );
            $this->iv = hash('md5', rand());
        }
    }

    function encrypt($input) {
        if ( $this->useMCrypt ) {
            return $this->urlsafe_b64encode(mcrypt_encrypt(MCRYPT_RIJNDAEL_128, $this->securekey, $input, MCRYPT_MODE_ECB, $this->iv));
        } else if ( $this->useOpenSSL ) {
            return $this->urlsafe_b64encode( openssl_encrypt( $input, OPEN_SSL_ENCRYPTION_METHOD, $this->securekey, 0, $this->iv) );
        } else {
            return $this->urlsafe_b64encode( $this->rc4( $input, $this->securekey ));
        }
    }

    function decrypt($input) {
        if ( $this->useMCrypt ) {
            return trim(mcrypt_decrypt(MCRYPT_RIJNDAEL_128, $this->securekey, $this->urlsafe_b64decode($input), MCRYPT_MODE_ECB, $this->iv));
        } else if ( $this->useOpenSSL ) {
            return openssl_decrypt($this->urlsafe_b64encode( $input ), OPEN_SSL_ENCRYPTION_METHOD, $this->securekey, 0, $this->iv );
        } else {
            return $this->rc4( $this->urlsafe_b64decode($input), $this->securekey );
        }
    }

    function urlsafe_b64encode($string) {
        $data = base64_encode($string);
        $data = str_replace(array('+','/','='),array('-','_','.'),$data);
        return $data;
    }
    function urlsafe_b64decode($string) {
        $data = str_replace(array('-','_','.'),array('+','/','='),$string);
        $mod4 = strlen($data) % 4;
        if($mod4) $data .= substr('====', $mod4);
        return base64_decode($data);
    }

    function rc4( $str, $key ) {
        $s = array();
        for ($i = 0; $i < 256; $i++) {
            $s[$i] = $i;
        }
        $j = 0;
        for ($i = 0; $i < 256; $i++) {
            $j = ($j + $s[$i] + ord($key[$i % strlen($key)])) % 256;
            $x = $s[$i];
            $s[$i] = $s[$j];
            $s[$j] = $x;
        }
        $i = 0;
        $j = 0;
        $res = '';
        for ($y = 0; $y < strlen($str); $y++) {
            $i = ($i + 1) % 256;
            $j = ($j + $s[$i]) % 256;
            $x = $s[$i];
            $s[$i] = $s[$j];
            $s[$j] = $x;
            $res .= $str[$y] ^ chr($s[($s[$i] + $s[$j]) % 256]);
        }
        return $res;
    }


}

?>