<?php

// Changes:
// bug 16757, LD - initial version for legal hold

// defines BP, xml, action, cookie
require '../header.php';

// defines functions for backup status processing.
require $rootdir . 'bpl/basebackupstatus.php';

global $Log;

$localID = $BP->get_local_system_id();
$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $localID;

// figure out which sysID to used based on view.
$sysID = $grandClientView ? $localID : $dpuID;

switch ($action) {
	case "get":	// get legal hold for one backup
		$error = NULL;
		if (isset($_REQUEST['ids'])) {
			$ids = $_REQUEST['ids'];
			$getRetention = isset($_GET['getretention']) ? ($_GET['getretention'] == '1') : false;
			$result = $BP->get_legalhold_backup_info($ids, $sysID);
			if ($result != NULL && $result !== false && (count($result) > 0)) {
				$xml->push('LegalHoldBackupInformation');
				foreach ($result as $backup) {
					if ($getRetention === true) {
						if (isset($backup["instance_id"])) {
							$instanceId=$backup['instance_id'];
							$retention = $BP->get_retention_settings(array('instance_id' => $instanceId), 
											$sysID);
						}
					}
					
					getLegalHoldBackupXML($xml, $backup, $retention);
				}
				$xml->pop();
			}
			else {
				$msg = $BP->getError();
				$error = "Error getting legal hold information for backups $ids.";
				$Log->writeError($error . " System error: " . $msg);
			}
		}
		else {
			$error = "Must specify ID of backup to get legal hold information.";
		}
		
		if ($error) {
			$BP->buildResult($xml, false, $error);
		}
		
		echo($xml->getXml());			
		break;
		
		
	case "save": // save legal hold for a backup
		$backupID = isset($_GET['id']) ? (int)$_GET['id'] : -1;
		$legalholddays = isset($_GET['legalholddays']) ? (int)$_GET['legalholddays'] : 0;
		$instanceID = isset($_GET['instance_id']) ? (int)$_GET['instance_id'] : -1;
		
		$saveInstance = isset($_GET['saveinstance']) ? ($_GET['saveinstance'] == '1') : false;
		$holdInstance = isset($_GET['legalholddaysinstance']) ? (int)$_GET['legalholddaysinstance'] : 0;
		$min = isset($_GET['min']) ? (int)$_GET['min'] : 0;
		$max = isset($_GET['max']) ? (int)$_GET['max'] : 0;
		
		$result = $BP->save_legalhold_per_backup($backupID, $legalholddays, $sysID);
		if ($result !== false && $saveInstance === true) {
			$retention = array();
			$retention['instance_id'] = $instanceID;
			$retention['retention_min'] = $min;
			$retention['retention_max'] = $max;
			$retention['legal_hold'] = $holdInstance;
			$result = $BP->save_retention_settings(array($retention), $sysID);
		}
		
		$BP->buildResult($xml, $result);
		
		echo($xml->getXml());			
		break;
		
	case "list":	// get legal hold for all backups, populates report
		$cid = isset($_GET['cid']) ? (int)$_GET['cid'] : -1;
		$app_id = isset($_GET['appid']) ? (int)$_GET['appid'] : -1;
		$instance_ids = isset($_REQUEST['instance_ids']) ? $_REQUEST['instance_ids'] : -1;

		$filter = array();
		
		if ($cid != -1 ) {
			$filter['client_id'] = $cid;
		}
		if ($app_id != -1 ) {
			$filter['app_id'] = $app_id;
		}
		if ($grandClientView) {
			$filter['grandclients'] = 1;
		} 
		
		$originalFilter = $filter;
		
		// Loop through all systems.  Will be only one in the list for a DPU, multiple systems on a Vault.
		$systems = $BP->get_system_list();
		
		if ($systems === false) {
			$BP->buildResult($xml, false);
			echo($xml->getXml());
			break;
		}
		
		$localID = $BP->get_local_system_id();
		$bVault = $BP->local_system_is_vault();
			
		$bManager = $BP->local_system_is_mgr();
		$bMDPU = isset($_GET['mdpu']) ? true : false;
		
		// If an error occurs, skip this DPU and continue.
		$xml->push('LegalHoldBackupInformation');
		foreach ($systems as $systemID => $systemName) {
			if ($systemID == $dpuID || $dpuID == $localID) {
				// If this system is a vault and we are looking at this system, skip it and go to the next.
				if ($bVault && ($systemID == $localID)) {
					continue;
				}
				// If this system is a manager and we are not running on the MDPU node nor are
				// not looking at the selected system, skip it.
				if (!$bVault && $bManager && !$bMDPU && ($systemID != $dpuID)) {
					continue;
				}				
				// for regular view check whether legal hold is supported
				// if false, skip this system
				if ($grandClientView === false) {
					$legalHoldSupported = $BP->legalhold_supported($systemID);
					if ($legalHoldSupported === false) {
						continue;
					}
				}
				
				// determine which sys id to use for API calls
				$sysID_API = $grandClientView ? $sysID : $systemID;
				
				$deviceList = $BP->get_device_list($sysID);
					
				$clientList = NULL;
				if ($grandClientView) {
					$clientList = $BP->get_grandclient_list($systemID);
				} else {
					$clientList = $BP->get_client_list($systemID);
				}
				
				if ($instance_ids == -1) {
					$filter = $originalFilter;
					$filter['system_id'] = $systemID;
					$legalHoldBackups = $BP->get_legalhold_backups($filter, $sysID_API);
				}
				else {
					$tempLegalHoldBackups = array();
					$instanceIdsArray = explode(",", $instance_ids);
					foreach ($instanceIdsArray as $instID) {
						$filter = $originalFilter;
						$filter['system_id'] = $systemID;
						$filter['instance_id'] = (int)$instID;
						$a = $BP->get_legalhold_backups($filter, $sysID_API);
						$tempLegalHoldBackups = array_merge($tempLegalHoldBackups, $a);
					}
					$legalHoldBackups = $tempLegalHoldBackups;
				}
				
				if ($legalHoldBackups !== false) {
					$backupStatusArray = array();
					if (count($legalHoldBackups) > 0) {
						$backupIDs = filterBackupsThatAreNotOnLegalHold($legalHoldBackups, 'backup_no');
						$backupStatusArray = getBackupStatusForBackups($BP, $sysID_API, $backupIDs);
					}
					
					if ($backupStatusArray !== false) {
						// Merge the two results (legal hold and info) into one array and output as XML.
						$result = array();
						foreach ($legalHoldBackups as &$legalHoldBackup) {
							foreach ($backupStatusArray as &$backupStatus) {
								if ($backupStatus['id'] == $legalHoldBackup['backup_no']) {
									$result = &$legalHoldBackup;
									$attributes = getBackupStatusAttributes();
											
									foreach ($attributes as $attribute) {
										if (isset($backupStatus[$attribute])) {
											$result[$attribute] = &$backupStatus[$attribute];
										}
									}
									createBackupExpandedStatusXML($xml, $result, $deviceList, $clientList,
															  $systemID, $systemName);
									break;
								} // end if ($backupInfo['id'] == $backup['id'])
							}
						}
					}
					else {
						createDPUErrorXML($xml, $BP, $systemName);
					}
				} else {
					createDPUErrorXML($xml, $BP, $systemName);
				}				
			}
		}
		$xml->pop(); // LegalHoldBackupInformation
		echo($xml->getXml());
		
		break;
}


/*
*	[backup_no]                 => long
*	[instance_id]               => long
*	[is_on_hold]                => bool
*	[start_time]                => long
*	[curr_time]                 => long
*	[hold_days_backup]          => long
*	[hold_days_instance]        => long
*	[hold_expire]          		=> long
*/
function getLegalHoldBackupXML($xml, $data, $retention = false) {
	$xml->push('LegalHoldBackup', array('id'=>$data['backup_no'], 'instanceId'=>$data['instance_id']));
	if (isset($data["is_on_hold"])) {
		$xml->element("OnHold", $data["is_on_hold"] == true ? "1" : "0");
	}
	if (isset($data['start_time'])) {
		$startTimeEpoch = $data['start_time'];
		// convert timestamp to local time/day
		$day = date('m/d/Y', $data['start_time']);
		$time = date(TIME_FORMAT_12HR, $data['start_time']);
		$xml->element('Date', $day);
		$xml->element('Time', $time);
	}
	if (isset($data['curr_time'])) {
		$currentTimeEpoch = $data['start_time'];
		// convert timestamp to local time/day
		$day = date('m/d/Y', $data['curr_time']);
		$time = date(TIME_FORMAT_12HR, $data['curr_time']);
		$xml->element('CurrentDate', $day);
		$xml->element('CurrentTime', $time);
	}
	if (isset($data["hold_days_backup"])) {
		$xml->element("LegalHoldDaysBackup", $data["hold_days_backup"]);
	}
	if (isset($data["hold_days_instance"])) {
		$xml->element("LegalHoldDaysInstance", $data["hold_days_instance"]);
	}
	if (isset($data['hold_expire_time'])) {
		// convert timestamp to local time/day
		$day = date('m/d/Y', $data['hold_expire_time']);
		$time = date(TIME_FORMAT_12HR, $data['hold_expire_time']);
		$xml->element('LegalHoldExpirationDate', $day);
		$xml->element('LegalHoldExpirationTime', $time);
	}
	
	if ($retention!==false) {
		foreach ($retention as $ret) {
			if (isset($ret['retention_min'])) {
				$xml->element('RetentionMin', $ret['retention_min']);
			}
			if (isset($ret['retention_max'])) {
				$xml->element('RetentionMax', $ret['retention_max']);
				break;
			}
		}
	}
	
	$xml->pop(); // LegalHoldBackup
}

/*
* Filter out backups that are not on legal hold from the return
* For a given array, return an int array of backup ids as specified under the given key
* backups - associative array containing backup ids
* key - the key for the backup ids in 'backups'
* returns - int array of backup ids
*/
function filterBackupsThatAreNotOnLegalHold(&$backups, $key) {
	$backupIDs = array();
	$legalHoldBackups = array();
	foreach ($backups as &$backup) {
		if ($backup['is_on_hold'] === true) {
			$legalHoldBackups[] = $backup;
			$backupIDs[] = (int)$backup[$key];
		}
	}
	$backups = $legalHoldBackups;
	return $backupIDs;
}

/*
* For a given int array, return an int array of backup ids as specified under the given key
* systemID - system id that holds these backups
* backupIDs - int array of backup ids
* returns - an associative array of backup statuses for the given backups on the given system
*/
function getBackupStatusForBackups($BP, $sysID, $backupIDs) {
	// Retrieve all backup types
	global $GLOBAL_BACKUP_REPORT_TYPES;
	
	$result_format = array( 'system_id' => $sysID, 
							'backup_ids' => $backupIDs, 
							'type' => $GLOBAL_BACKUP_REPORT_TYPES);

	$backupStatusArray = array();
	// Only call the info API if we have backup IDs to process.
	if (count($backupIDs) > 0) {
		$backupStatusArray = $BP->get_backup_status($result_format);
	}
	
	return $backupStatusArray;
}

/*
* Array of attributes that a backup has as returned from get_backup_status
* returns - attribute array
*/
function getBackupStatusAttributes() {
	return array(	'type', 'complete', 'id', 'client_id',
					'start_time', 'elapsed_time',
					'status', 'size', 'sync_size', 'files', 'encrypted',
					'database_name', 'server_instance_name',
					'oracle_sid', 'sharepoint_guid',
					'instance_id', 'app_name', 'vm_name',
					'sync_status', 'verify_status', 'sync_message', 'vmware_template', 'support_vss_diff',
					'synthesized', 'disk_metadata', 'is_cluster', 'guid',
					'restored_bkup_type', 'replicated', 'client_config', STATUS_CERTIFY_API_KEY);
}

//
// This function displays an error message for the row if the system is not accessible.
function createDPUErrorXML($xml, $BP, $dpuName)
{
	global $Log;
	$message = " Could not get backup information for system: " . $dpuName;
	$message = $BP->getError() . ":" . $message;
	$Log->writeError($message, true);
							
	$xml->push("Backup", array('Type' => '', "Identification" => '', "DPUName" => $dpuName, "ClientName" => '{system offline}'));
	$xml->element('Date', '');
	$xml->element('Time', '');
	$xml->pop(); // Backup
}

$BP->destroyCookie();
?>