<?php

// Changes:
// bug 16757, LD - Legal Hold changes

// defines BP, xml, action, cookie
require '../header.php';

$localID = $BP->get_local_system_id();
$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $localID;

// figure out which sysID to used based on view.
$sysID = $grandClientView ? $localID : $dpuID;

//
// .ini setting that determines if BareMetal backups are included in retention calculations.
//
define("STORAGE_SECTION", 'StorageManager');
define("BAREMETAL", 'EnforceBareMetalRetention');

// The XML tree structure is of this form:
/*
<root>
	<Client Name="MyWindows" Directory="" InstanceID="2" Min="2" Max="10" Actual="3" LegalHold="10">
		<Application Name="SQL Server 2005" Type="SQL" Directory="MyWindows/" Min="10" Max="3" LegalHold="10">
			<Instance Name="SQLExpress" Type="SQL" Directory="MyWindows/SQL Server 2005/"  Min="10" Max="3" LegalHold="0">
				<Database Name="Test1" Directory="MyWindows/SQL Server 2005/SQLExpress/" InstanceID="4" Min="0" Max="20" LegalHold="0"/>
				<Database Name="test2" Directory="MyWindows/SQL Server 2005/SQLExpress/" InstanceID="5" Min="4" Max="20" LegalHold="0"/>
			</Instance>
		</Application>
		<Application Name="SQL Server 2008" Type="SQL" Directory="MyWindows/"  Min="0" Max="0" LegalHold="0">
			<Instance Name="SQLExpress" Type="SQL" Directory="MyWindows/SQL Server 2008/"  Min="0" Max="0" LegalHold="0">
				<Database Name="Test1" Directory="MyWindows/SQL Server 2008/SQLExpress/" InstanceID="4" Min="0" Max="20" LegalHold="0"/>
				<Database Name="test2" Directory="MyWindows/SQL Server 2008/SQLExpress/" InstanceID="5" Min="4" Max="20" LegalHold="0"/>
			</Instance>
		</Application>
	</Client>
	<Client Name="SQL-Client" Directory="" InstanceID="5" Min="0" Max="0" LegalHold="0">
		<Application Name="SQL Server 2008" Type="SQL" Directory="SQL-Client/" Min="0" Max="0" LegalHold="0">
			<Instance Name="SQL-Client" Type="SQL" Directory="SQL-Client/SQL Server 2008/" Min="0" Max="0" LegalHold="0">
				<Database Name="test3" Directory="SQL-Client/SQL Server 2008/SQL-Client/" InstanceID="6" Min="2" Max="10" LegalHold="0"/>
				<Database Name="test4" Directory="SQL-Client/SQL Server 2008/SQL-Client/" InstanceID="7" Min="2" Max="10" LegalHold="0"/>
			</Instance>
			<Instance Name="SQL-Express" Type="SQL" Directory="SQL-Client/SQL Server 2008/" Min="-- Not Set --" Max="-- Not Set --">
				<Database Name="test10" Directory="SQL-Client/SQL Server 2008/SQL-Express/" InstanceID="6" Min="2" Max="10" LegalHold="0"/>
				<Database Name="test11" Directory="SQL-Client/SQL Server 2008/SQL-Express/" InstanceID="7" Min="2" Max="10" LegalHold="0"/>
			</Instance>			
		</Application>
	</Client>
	<Client Name="vCenter-RRC" Directory="" InstanceID="8"  Min="0" Max="0" LegalHold="0">
		<Application Name="esx41" Type="VMware" Directory="vCenter-RRC/"  Min="0" Max="0" LegalHold="0">
			<VM Name="VM1" Directory="vCenter-RRC/esx41/" InstanceID="9" Min="13" Max="30" LegalHold="0"/>
			<VM Name="vm2" Directory="vCenter-RRC/esx41/" InstanceID="10" Min="12" Max="14" LegalHold="0"/>
		</Application>
		<Application Name="myvCenter" Type="VMware" Directory="vCenter-RRC/" Min="0" Max="0" LegalHold="0">
			<VM Name="VM3" Directory="vCenter-RRC/myvCenter/" InstanceID="11" Min="0" Max="30" LegalHold="0"/>
			<VM Name="vm4" Directory="vCenter-RRC/myvCenter/" InstanceID="12" Min="2" Max="19" LegalHold="0"/>
		</Application>
	</Client>
	<Client Name="exchangeAD" Directory="" InstanceID="15"  Min="0" Max="0" LegalHold="0">
		<Application Name="Exchange 2007" Type="Exchange" Directory="exchangeAD/"  Min="0" Max="0" LegalHold="0">
			<Database Name="First Storage Group" Directory="exchangeAD/Exchange 2007/" InstanceID="20" Min="1" Max="3" LegalHold="0"/>
			<Database Name="My Storage Group" Directory="exchangeAD/Exchange 2007/" InstanceID="21" Min="34" Max="42" LegalHold="0"/>
		</Application>
	</Client>
</root>
*/

switch ($action) {

	case "list":	// list retention settings.
		if (isset($_GET['cid'])) {
			$cid = (int)$_GET['cid'];
		}
		$clients = $grandClientView ? $BP->get_grandclient_list($dpuID) : $BP->get_client_list($dpuID);
		if ($clients != false) {
			$xml->push("root");
			
			addBareMetalRetention($BP, $xml, $dpuID);
			
			foreach ($clients as $clientID => $clientName) {
				if (isset($cid) && $cid!=$clientID) {
					continue;
				}
				$clientInfo = $BP->get_client_info($clientID, $sysID);
				$attributes = array('Name' => $clientName, 'ID' => $clientID, 'Directory' => '');
				
				$settings = $BP->get_retention_settings(array('client_id' => $clientID), $sysID);
				if ($settings === false) {
					$settings = array();
				}
				// Sort the settings so that instance names are grouped together.
				usort(&$settings, "compareSettings");
				$oldestArray = getOldest($BP, $settings, $sysID);
				$applications = isset($clientInfo['applications']) ? $clientInfo['applications'] : array();
				if (count($applications) > 0) {
					// Go ahead and add the file-level attributes to the Client.
					$attributes = addFileLevelAttributes($attributes, $clientID, $settings, $oldestArray);
				}
				$xml->push("Client", $attributes);
				foreach ($applications as $appID => $application) {
					if (isset($application['servers'])) {
						createVMInstances($xml, $clientName, $settings, $application, $application['servers'], $oldestArray);
					} else if($application['type'] == "Hyper-V") {
						createHyperVInstances($xml, $clientID, $clientName, $settings, $appID, $application, $oldestArray);
					}else {
						createAppInstances($xml, $clientID, $clientName, $settings, $appID, $application, $oldestArray);
					}
				}
				$xml->pop(); // Client
				
			}
			
			$xml->pop(); // root
		}						
		echo($xml->getXml());
		break;
			
	case "save":	// save retention settings
		$settingsXML = isset($_REQUEST['settings']) ? '<root>' . $_REQUEST['settings'] . '</root>' : "";
		$vaultUser = isset($_GET['vu']) ? (int)$_GET['vu'] : 0;
		if (!$vaultUser) {
			$bm = isset($_GET['bm']) ? $_GET['bm'] : "0";
			$BareMetal = $bm == '1' ? "True" : "False";
			$result = $BP->set_ini_value(STORAGE_SECTION, BAREMETAL, $BareMetal, $sysID);
		}
		$retentionSettings = xmlToSettings($settingsXML);
		$BP->buildResult($xml, $BP->save_retention_settings($retentionSettings, $sysID));
		echo($xml->getXml());
		break;
		
	case "list_gfs_policy":
		if (isset($_GET['pid'])) {
			$pid = (int)$_GET['pid'];
		}
		$policies = $BP->get_gfs_policy($pid, $sysID);
		if ($policies !== false) {
			$xml->push("root");
			foreach($policies as $policy) {
				$policyID = $policy['policy_id'];
				$policyName = $policy['policy_name'];
				$attributes = array("PolicyID" => $policyID, "PolicyName" => $policyName);
				$xml->push('Policy', $attributes);
				if (isset($policy['policy_description'])) {
					$xml->element('Description', $policy['policy_description']);
				}
				$days = $policy['days'] == -1 ? 'Forever' : $policy['days']; 
				$weeks = $policy['weeks'] == -1 ? 'Forever' : $policy['weeks'];
				$months = $policy['months'] == -1 ? 'Forever' : $policy['months'];
				$years = $policy['years'] == -1 ? 'Forever' : $policy['years'];
				$xml->element('Days', $days);
				$xml->element('Weeks', $weeks);
				$xml->element('Months', $months);
				$xml->element('Years', $years);
				$xml->element('is_default', $policy['is_default'] ? 1 : 0);
				$xml->pop();	//policy
			}
			$xml->pop();		//root
		} else {
			$BP->buildResult($xml, false);
		}
		echo($xml->getXml());
		break;
	
	case "update_gfs_policy":
		if (isset($_GET['pid'])) {
			$pid = (int)$_GET['pid'];
			$gfsPolicyID = array("policy_id" => $pid);	
		} else {
			$BP->buildResult($xml, false, "Policy update must have id specified.");
			echo($xml->getXml());
			break;
		}
		// fall through to 'save_gfs_policy'
		
	case "save_gfs_policy":
		if (isset($_GET['pname'])) {
			$pname = $_GET['pname'];
			$gfs_settings = array ("policy_name" => $pname);
		}
		if ($gfsPolicyID != NULL) {
			$gfs_settings = array_merge($gfsPolicyID, $gfs_settings);
		}
		if (isset($_GET['dsc'])) {
			$dsc = $_GET['dsc'];
			$gfs_settings["policy_description"] = $dsc;
		}
		if (isset($_GET['years'])) {
			$years = (int)$_GET['years'];
			$gfs_settings["years"] = $years;
		}
		if (isset($_GET['months'])) {
			$months = (int)$_GET['months'];
			$gfs_settings["months"] = $months;
		}
		if (isset($_GET['weeks'])) {
			$weeks = (int)$_GET['weeks'];
			$gfs_settings["weeks"] = $weeks;
		}
		if (isset($_GET['days'])) {
			$days = (int)$_GET['days'];
			$gfs_settings["days"] = $days;
		}
		if (isset($_GET['is_default'])) {
			$is_default = (int)$_GET['is_default'];
			$gfs_settings["is_default"] = $is_default;
		}
		$result = $BP->save_gfs_policy($gfs_settings, $sysID);
		$BP->buildResult($xml, $result);	
		echo($xml->getXml());
		break;
		
	case "delete_gfs_policy":
		if (isset($_GET['pid'])) {
			$pid = (int)$_GET['pid'];
			$result = $BP->delete_gfs_policy($pid, $sysID);
		} else {
			$result = false;
		}
		$BP->buildResult($xml, $result);	
		echo($xml->getXml());
		break;
		
	case "list_retention_strategy":
		$retention = $BP->get_retention_strategy($sysID);
		if ($retention !== false) {
			$xml->push("root");
			$attributes = array("Strategy" => $retention);
			$xml->push('Retention', $attributes);
			$xml->pop();
			$xml->pop();		//root
		} else {
			$BP->buildResult($xml, false);
		}
		echo($xml->getXml());
		break;
		
	case "set_retention_strategy":
		if (isset($_GET['strategy'])) {
			$strategy = $_GET['strategy'];
		}
		$result = $BP->set_retention_strategy($strategy, $sysID);
		$BP->buildResult($xml, $result);	
		echo($xml->getXml());
		break;
		
	case "list_gfs_retention":
		if (isset($_GET['cid'])) {
			$cid = (int)$_GET['cid'];
		}
		$clients = $grandClientView ? $BP->get_grandclient_list($dpuID) : $BP->get_client_list($dpuID);
		if ($clients != false) {
			$xml->push("root");
			
			addBareMetalRetention($BP, $xml, $dpuID);
			
			foreach ($clients as $clientID => $clientName) {
				if (isset($cid) && $cid != $clientID) {
					continue;
				}
				$clientInfo = $BP->get_client_info($clientID, $sysID);
				$attributes = array('Name' => $clientName, 'ID' => $clientID, 'Directory' => '');
				
				$settings = $BP->get_gfs_retention(array('client_id' => $clientID), $sysID);
				if ($settings === false) {
					$settings = array();
				}
				// Sort the settings so that instance names are grouped together.
				usort(&$settings, "compareSettingsGFS");
				$applications = isset($clientInfo['applications']) ? $clientInfo['applications'] : array();
				if (count($applications) > 0) {
					// Go ahead and add the file-level attributes to the Client.
					$attributes = addFileLevelAttributesGFS($attributes, $clientID, $settings);
				}
				$xml->push("Client", $attributes);
				foreach ($applications as $appID => $application) {
					if (isset($application['servers'])) {
						createVMInstancesGFS($xml, $clientName, $settings, $application, $application['servers']);
					} else if($application['type'] == "Hyper-V") {
						createHyperVInstancesGFS($xml, $clientID, $clientName, $settings, $appID, $application);
					} else {
						createAppInstancesGFS($xml, $clientID, $clientName, $settings, $appID, $application);
					}
				}
				$xml->pop(); // Client
			}
			$xml->pop(); // root
		}						
		echo($xml->getXml());
		break;
	
	case "apply_gfs_retention":
		$settingsXML = isset($_REQUEST['settings']) ? '<root>' . $_REQUEST['settings'] . '</root>' : "";
		$vaultUser = isset($_GET['vu']) ? (int)$_GET['vu'] : 0;
		if (!$vaultUser) {
			$bm = isset($_GET['bm']) ? $_GET['bm'] : "0";
			$BareMetal = $bm == '1' ? "True" : "False";
			$result = $BP->set_ini_value(STORAGE_SECTION, BAREMETAL, $BareMetal, $sysID);
		}
		$retentionSettings = xmlToGFSSettings($settingsXML);
		$result = $BP->apply_gfs_retention($retentionSettings, $sysID);
		$BP->buildResult($xml, $result);
		echo($xml->getXml());
		break;

}

//
// Function that compares the appIDs of the elements and sorts them.
//
function compareSettings($a, $b) {
	if ($a['app_id'] == $b['app_id']) {
		$result = compareInstance(0, $a, $b);
	} else if ($a['app_id'] > $b['app_id']) {
		$result = compareInstance(1, $a, $b);
	} else {
		$result = compareInstance(-1, $a, $b);
	}
	return $result;
}

//
// Function that compares the instance names.
//
function compareInstance($initialResult, $a, $b) {
	$result = $initialResult;
	$split = split("\|", $a['name']);
	$instanceA = count($split > 1) ? $split[0] : "";
	$split = split("\|", $b['name']);
	$instanceB = count($split > 1) ? $split[0] : "";
	if ($instanceA == $instanceB) {
		$result = 0;
	} else if ($instanceA > $instanceB) {
		$result = 1;
	} else {
		$result = -1;
	}
	return $result;	
}

//
// Fined the client and application in the settings array and return the value of the specified field to the caller.
//
function getSettingValue($settings, $clientID, $applicationName, $field) {
	$result = "";
	foreach ($settings as $setting) {
		if ($setting['client_id'] == $clientID && $setting['app_name'] == $applicationName) {
			if(isset($setting[$field])) {
				$result = $setting[$field];
			}
			break;
		}
	}
	return $result;
}

//
// Retrieves the setting of whether or not retention will be enforced for BareMetal backups.
//
function addBareMetalRetention($BP, $xml, $dpuID) {
	$stringEnforce = $BP->get_ini_value(STORAGE_SECTION, BAREMETAL, $dpuID);
	if ($stringEnforce !== false) {
		$stringEnforce = strtolower($stringEnforce);
		$xml->element('BareMetal', $stringEnforce == 'true' ? 1 : 0);
	}
}

//
// Retrieves the settings for the file-level instance associated with a particular client ID, build an
// array of attributes about that instances and returns the attributes array.
//
function addFileLevelAttributes($attributes, $clientID, $settings, $oldest) {
	$attributes['Min'] = getSettingValue($settings, $clientID, 'file-level', 'retention_min');
	$attributes['Max'] = getSettingValue($settings, $clientID, 'file-level', 'retention_max');
	$legalHold = getSettingValue($settings, $clientID, 'file-level', 'legal_hold');
	if ($legalHold !== "") {
		$attributes['LegalHold'] = $legalHold;
	}
	$instanceID = getSettingValue($settings, $clientID, 'file-level', 'instance_id');
	$attributes['InstanceID'] = $instanceID;
	$attributes['Days'] = isset($oldest[$instanceID]) ? $oldest[$instanceID] : 0;
	return $attributes;
}

//
// Creates the portion of the XML tree for application instances that are VMs.  Goes through each VMware server and creates
// a wrapper for each (application), with VM instances underneath.
//
function createVMInstances($xml, $clientName, $settings, $application, $servers, $oldest) {
  foreach ($servers as $uuid => $serverName) {
	  $attributes = array('Name' => $serverName,
						  'Type' => $application['type'],
						  'Directory' => $clientName);
	  $xml->push("Application", $attributes);
	  foreach ($settings as $setting) {
		  if ($setting['app_name'] == 'VMware') {
			  $settingName = $setting['name'];
			  $split = split("\|", $settingName);
			  if ($serverName == $split[0]) {
				  $instanceName = $split[1];
				  $directory = $clientName . '/' . $serverName . '/';
				  $attributes = array('Name' => $instanceName,
									  'InstanceID' => $setting['instance_id'],
									  'Directory' => $directory,
									  'Min' => $setting['retention_min'],
								  	  'Max' => $setting['retention_max']);
								  
				  if (isset($setting['legal_hold'])) {
					  $attributes['LegalHold'] = $setting['legal_hold'];
				  }
				  
			 	 $attributes['Days'] = isset($oldest[$setting['instance_id']]) ? $oldest[$setting['instance_id']] : 0;
				 $xml->emptyElement('VM', $attributes);
			  }
			  //unset($setting);
		  }
	  }
	  $xml->pop(); // Application
  }
}

function createHyperVInstances($xml, $clientID, $clientName, $settings, $appID, $application, $oldest){
$attributes = array('Name' => $application['name'],
			        'Type' => $application['type'],
					'Directory' => $clientName);

  if ($application['name'] != 'file-level') {
	    $xml->push("Application", $attributes);
		$wrapper = "";
		$pushed = "";
		$bPop = false;
		foreach ($settings as $setting) {
		  $settingName = $setting['name'];
		  $split = split("\|", $settingName);
		  if ($setting['client_id'] == $clientID && $setting['app_id'] == $appID) {
				  
			  if (count($split) > 1) {
				  $wrapper = $split[0];
				  if (strcmp($pushed, $wrapper)) {
					  if ($pushed != "") {
						  $xml->pop(); // pop the prior Instance.
						  $bPop = false;
						  $pushed = "";
					  }
					  $directory = $clientName . '/' . $application['name'] . '/';
					  $attributes = array( 'Name' => $wrapper,
										   'Type' => $application['type'],
										   'Directory' => $directory,
										   'InstanceID' => $setting['instance_id'],
		   								   'Min' => $setting['retention_min'],
								  		   'Max' => $setting['retention_max']);
								  
					  if (isset($setting['legal_hold'])) {
						  $attributes['LegalHold'] = $setting['legal_hold'];
					  }
					  
					  $attributes['Days'] = isset($oldest[$setting['instance_id']]) ? $oldest[$setting['instance_id']] : 0;
				
					  $xml->push('Instance', $attributes);
					  $directory .= $split[0] . '/';
					  $pushed = $wrapper;
				  }
				  $instanceName = $split[1];
			  } else {
				  $instanceName = $settingName;
				  $directory = $clientName . '/' . $application['name'] . '/';
				  if ($pushed != "") {
				 	 $bPop = true;
				  }
			  }

			  if ($bPop) {
				  $xml->pop(); // instance
				  $bPop = false;
				  $pushed = "";
			  }
		  }
	  }
	  // handle last item.
	  if ($pushed != "") {
		  $xml->pop(); // instance
	  }
	  $xml->pop(); // Application
  }
}
//
// Creates the portion of the XML tree for application instances other than VMs.  For SQL, there are two levels (instance and database).
// For Exchange, there is one level (database), and for file-level, the Instance is at the top (application)level.
//
function createAppInstances($xml, $clientID, $clientName, $settings, $appID, $application, $oldest) {
  $attributes = array('Name' => $application['name'],
					  'Type' => $application['type'],
					  'Directory' => $clientName);
  /*
  if ($application['name'] == 'file-level') {
	  $attributes['Min'] = getSettingValue($settings, $clientID, 'file-level', 'retention_min');
	  $attributes['Max'] = getSettingValue($settings, $clientID, 'file-level', 'retention_max');
	  $instanceID = getSettingValue($settings, $clientID, 'file-level', 'instance_id');
	  $attributes['InstanceID'] = $instanceID;
	  $attributes['Days'] = isset($oldest[$instanceID]) ? $oldest[$instanceID] : 0;
  }
  */
 // $xml->push("Application", $attributes);
  if ($application['name'] != 'file-level') {
	    $xml->push("Application", $attributes);
		$wrapper = "";
		$pushed = "";
		$bPop = false;
		foreach ($settings as $setting) {
		  $settingName = $setting['name'];
		  $split = split("\|", $settingName);
		  if ($setting['client_id'] == $clientID && $setting['app_id'] == $appID) {
				  
			  if (count($split) > 1) {
				  $wrapper = $split[0];
				  if (strcmp($pushed, $wrapper)) {
					  if ($pushed != "") {
						  $xml->pop(); // pop the prior Instance.
						  $bPop = false;
						  $pushed = "";
					  }
					  $directory = $clientName . '/' . $application['name'] . '/';
					  $attributes = array( 'Name' => $wrapper,
										   'Type' => $application['type'],
										   'Directory' => $directory);
					  $xml->push('Instance', $attributes);
					  $directory .= $split[0] . '/';
					  $pushed = $wrapper;
				  }
				  $instanceName = $split[1];
			  } else {
				  $instanceName = $settingName;
				  $directory = $clientName . '/' . $application['name'] . '/';
				  if ($pushed != "") {
				 	 $bPop = true;
				  }
			  }
			  $attributes = array('Name' => $instanceName,
								  'InstanceID' => $setting['instance_id'],
								  'Directory' => $directory,
								  'Min' => $setting['retention_min'],
								  'Max' => $setting['retention_max']);
								  
			  if (isset($setting['legal_hold'])) {
				  $attributes['LegalHold'] = $setting['legal_hold'];
			  }
			  
			  $attributes['Days'] = isset($oldest[$setting['instance_id']]) ? $oldest[$setting['instance_id']] : 0;
			
				$xml->emptyElement('Database', $attributes);
							  
			  if ($bPop) {
				  $xml->pop(); // instance
				  $bPop = false;
				  $pushed = "";
			  }
		  }
	  }
	  // handle last item.
	  if ($pushed != "") {
		  $xml->pop(); // instance
	  }
	  $xml->pop(); // Application
  }
  //$xml->pop(); // Application
}


//
// Gets the oldest backup for the instances specified in the settings array, and builds a return array.
// The return array is indexed by the instance ID of the instance, and the value is the days in days of the oldest backup.
//
function getOldest($BP, $settings, $dpuID) {

  $sep = "";
  $instances = "";
  $returnArray = array();
  foreach ($settings as $setting) {
	  $instances .= $sep . $setting['instance_id'];
	  $sep = ",";
  }
  $oldestArray = $BP->get_oldest_backup($instances, $dpuID);
  if ($oldestArray !== false) {
	  foreach ($oldestArray as $oldest) {
		  $startTime = $oldest['start_time'];
		  $now = time();
		  $days = ($now - $startTime) / 3600 / 24;
		  // Do not round up, or actual retention could appear slightly longer than it is.
		  // $returnArray[$oldest['instance_id']] = round($days);
		  $returnArray[$oldest['instance_id']] = floor($days);
	  }
  } else {
	  global $Log;
	  $Log->writeVariable('failure getting the oldest backup: ' . $BP->getError());
  }
  return $returnArray;
}

//
// Convers the XML tree to a for suitable for the "save settings" API.
//
function xmlToSettings($xml)
{
	$parser = xml_parser_create();
	xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING, 0);
	xml_parser_set_option($parser, XML_OPTION_SKIP_WHITE, 1);
	xml_parse_into_struct($parser, $xml, $values, $tags);
	xml_parser_free($parser);

	$returnArray = array();
	$oneInstance = array();

	$stack = array();
	foreach ($values as $val) {
		if ($val['type'] == "open" || $val['type'] == "complete") {
			if ($val['tag'] == 'root') {
				; // skip
			} else {
				$attributes = $val['attributes'];
				// If we have an instanceID, create a retention array element to save.
				if (isset($attributes['InstanceID'])) {
					$oneInstance = array('instance_id' => (int)$attributes['InstanceID']);
					if (isset($attributes['Min'])) {
						$oneInstance['retention_min'] = (int)$attributes['Min'];
					}
					if (isset($attributes['Max'])) {
						$oneInstance['retention_max'] = (int)$attributes['Max'];
					}
					if (isset($attributes['LegalHold'])) {
						$oneInstance['legal_hold'] = (int)$attributes['LegalHold'];
					}
					$returnArray[] = $oneInstance;
				}
			}
		}
	}

	return $returnArray;
	
}

//
// Function that compares the appIDs of the elements and sorts them.
//
function compareSettingsGFS($a, $b) {
	if ($a['app_id'] == $b['app_id']) {
		$result = compareInstance(0, $a, $b);
	} else if ($a['app_id'] > $b['app_id']) {
		$result = compareInstanceGFS(1, $a, $b);
	} else {
		$result = compareInstanceGFS(-1, $a, $b);
	}
	return $result;
}

//
// Function that compares the instance names.
//
function compareInstanceGFS($initialResult, $a, $b) {
	$result = $initialResult;
	$split = split("\|", $a['instance_name']);
	$instanceA = count($split > 1) ? $split[0] : "";
	$split = split("\|", $b['instance_name']);
	$instanceB = count($split > 1) ? $split[0] : "";
	if ($instanceA == $instanceB) {
		$result = 0;
	} else if ($instanceA > $instanceB) {
		$result = 1;
	} else {
		$result = -1;
	}
	return $result;	
}

//
// Retrieves the GFS settings for the file-level instance associated with a particular client ID, build an
// array of attributes about that instances and returns the attributes array.
//
function addFileLevelAttributesGFS($attributes, $clientID, $settings) {
	$attributes['PolicyID'] = getSettingValue($settings, $clientID, 'file-level', 'policy_id');
	$attributes['PolicyName'] = getSettingValue($settings, $clientID, 'file-level', 'policy_name');
	$attributes['Days'] = getSettingValue($settings, $clientID, 'file-level', 'days');
	$attributes['Weeks'] = getSettingValue($settings, $clientID, 'file-level', 'weeks');
	$attributes['Months'] = getSettingValue($settings, $clientID, 'file-level', 'months');
	$attributes['Years'] = getSettingValue($settings, $clientID, 'file-level', 'years');
	$instanceID = getSettingValue($settings, $clientID, 'file-level', 'instance_id');
	$attributes['InstanceID'] = $instanceID;
	return $attributes;
}

//
// Creates the portion of the XML tree for application instances that are VMs.  Goes through each VMware server and creates
// a wrapper for each (application), with VM instances underneath.
//
function createVMInstancesGFS($xml, $clientName, $settings, $application, $servers) {
  foreach ($servers as $uuid => $serverName) {
	  $attributes = array('Name' => $serverName,
						  'Type' => $application['type'],
						  'Directory' => $clientName);
	  $xml->push("Application", $attributes);
	  foreach ($settings as $setting) {
		  if ($setting['app_name'] == 'VMware') {
			  $settingName = $setting['instance_name'];
			  $split = split("\|", $settingName);
			  if ($serverName == $split[0]) {
				  $instanceName = $split[1];
				  $directory = $clientName . '/' . $serverName . '/';
				  $attributes = array('Name' => $instanceName,
									  'InstanceID' => $setting['instance_id'],
									  'Directory' => $directory,
									  'PolicyID' => $setting['policy_id'],
								  	  'PolicyName' => $setting['policy_name'],
									  'Days' => $setting['days'],
									  'Weeks' => $setting['weeks'],
									  'Months' => $setting['months'],
									  'Years' => $setting['years']);
								  
				  $xml->emptyElement('VM', $attributes);
			  }
		  }
	  }
	  $xml->pop(); // Application
  	}
}

function createHyperVInstancesGFS($xml, $clientID, $clientName, $settings, $appID, $application){
$attributes = array('Name' => $application['name'],
			        'Type' => $application['type'],
					'Directory' => $clientName);

  if ($application['name'] != 'file-level') {
	    $xml->push("Application", $attributes);
		$wrapper = "";
		$pushed = "";
		$bPop = false;
		foreach ($settings as $setting) {
		  $settingName = $setting['instance_name'];
		  $split = split("\|", $settingName);
		  if ($setting['client_id'] == $clientID && $setting['app_id'] == $appID) {
				  
			  if (count($split) > 1) {
				  $wrapper = $split[0];
				  if (strcmp($pushed, $wrapper)) {
					  if ($pushed != "") {
						  $xml->pop(); // pop the prior Instance.
						  $bPop = false;
						  $pushed = "";
					  }
					  $directory = $clientName . '/' . $application['name'] . '/';
					  $attributes = array( 'Name' => $wrapper,
										   'Type' => $application['type'],
										   'Directory' => $directory,
										   'InstanceID' => $setting['instance_id'],
		   								   'PolicyID' => $setting['policy_id'],
								  	  	   'PolicyName' => $setting['policy_name'],
									  	   'Days' => $setting['days'],
									  	   'Weeks' => $setting['weeks'],
									  	   'Months' => $setting['months'],
									  	   'Years' => $setting['years']);
				
					  $xml->push('Instance', $attributes);
					  $directory .= $split[0] . '/';
					  $pushed = $wrapper;
				  }
				  $instanceName = $split[1];
			  } else {
				  $instanceName = $settingName;
				  $directory = $clientName . '/' . $application['name'] . '/';
				  if ($pushed != "") {
				 	 $bPop = true;
				  }
			  }

			  if ($bPop) {
				  $xml->pop(); // instance
				  $bPop = false;
				  $pushed = "";
			  }
		  }
	  }
	  // handle last item.
	  if ($pushed != "") {
		  $xml->pop(); // instance
	  }
	  $xml->pop(); // Application
  }
}
//
// Creates the portion of the XML tree for application instances other than VMs.  For SQL, there are two levels (instance and database).
// For Exchange, there is one level (database), and for file-level, the Instance is at the top (application)level.
//
function createAppInstancesGFS($xml, $clientID, $clientName, $settings, $appID, $application) {
  $attributes = array('Name' => $application['name'],
					  'Type' => $application['type'],
					  'Directory' => $clientName);
  if ($application['name'] != 'file-level') {
	    $xml->push("Application", $attributes);
		$wrapper = "";
		$pushed = "";
		$bPop = false;
		foreach ($settings as $setting) {
		  $settingName = $setting['instance_name'];
		  $split = split("\|", $settingName);
		  if ($setting['client_id'] == $clientID && $setting['app_id'] == $appID) {	  
			  if (count($split) > 1) {
				  $wrapper = $split[0];
				  if (strcmp($pushed, $wrapper)) {
					  if ($pushed != "") {
						  $xml->pop(); // pop the prior Instance.
						  $bPop = false;
						  $pushed = "";
					  }
					  $directory = $clientName . '/' . $application['name'] . '/';
					  $attributes = array( 'Name' => $wrapper,
										   'Type' => $application['type'],
										   'Directory' => $directory);
					  $xml->push('Instance', $attributes);
					  $directory .= $split[0] . '/';
					  $pushed = $wrapper;
				  }
				  $instanceName = $split[1];
			  } else {
				  $instanceName = $settingName;
				  $directory = $clientName . '/' . $application['name'] . '/';
				  if ($pushed != "") {
				 	 $bPop = true;
				  }
			  }
			  $attributes = array( 'Name' => $instanceName,
								   'InstanceID' => $setting['instance_id'],
								   'Directory' => $directory,
								   'PolicyID' => $setting['policy_id'],
								   'PolicyName' => $setting['policy_name'],
								   'Days' => $setting['days'],
								   'Weeks' => $setting['weeks'],
								   'Months' => $setting['months'],
								   'Years' => $setting['years']);

			  $xml->emptyElement('Database', $attributes);
							  
			  if ($bPop) {
				  $xml->pop(); // instance
				  $bPop = false;
				  $pushed = "";
			  }
		  }
	  }
	  // handle last item.
	  if ($pushed != "") {
		  $xml->pop(); // instance
	  }
	  $xml->pop(); // Application
  }
  //$xml->pop(); // Application
}

//
// Convers the XML tree to a array suitable for the apply GFS retention API.
//
function xmlToGFSSettings($xml)
{
	$parser = xml_parser_create();
	xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING, 0);
	xml_parser_set_option($parser, XML_OPTION_SKIP_WHITE, 1);
	xml_parse_into_struct($parser, $xml, $values, $tags);
	xml_parser_free($parser);

	$returnArray = array();
	$oneInstance = array();

	$stack = array();
	foreach ($values as $val) {
		if ($val['type'] == "open" || $val['type'] == "complete") {
			if ($val['tag'] == 'root') {
				; // skip
			} else {
				$attributes = $val['attributes'];
				// If we have an instanceID, create a retention array element to save.
				if (isset($attributes['InstanceID'])) {
					$oneInstance = array('instance_id' => (int)$attributes['InstanceID']);
					if (isset($attributes['PolicyID'])) {
						$oneInstance['policy_id'] = (int)$attributes['PolicyID'];
					}
					$returnArray[] = $oneInstance;
				}
			}
		}
	}
	return $returnArray;
	
}

$BP->destroyCookie();
?>
