<?php
//error_reporting(E_ERROR);
// Build the XML tree, starting with root and then the level specified by the user.
// Default for dpv login is dpv level, for dpu login is dpu level.
// user may specify client to start at client level.
// If network ID specified, we only return the part of the tree that matches this ID.

define('BACKUP_XML_TAG', 'BackupStatus');
define('SYNC_XML_TAG', 'SecureSyncStatus');

// ID of the local system, used throughout tree creation.
$localID = $BP->get_local_system_id();

function buildXML($xml, $BP, $options)
{
	global $localID;
	
	$returnValue = true;
	
	$xml->push('root');
	
	// level indicates the identity of the root node of the tree to generate.
	switch ($options->getLevel()) {
		case "dpv":			// default from DPV login.  Create entire tree.
							$returnValue = addDPV(true, $xml, $BP, $options);
							break;
							
		case "dpvtop":		// DPV login, create top of tree only.
							$returnValue = addDPV(true, $xml, $BP, $options, false, false);
							break;
							
		case "mdpu":		// generate a list of DPU's, wrap with MDPU.
							$returnValue = addmDPU($xml, $BP, $options);
							break;
							
		case "mdputop":		// generate MDPU wrapper only.
							$returnValue = addmDPU($xml, $BP, $options, false, false);
							break;
							
		case "dpu":			// if the networkID is specified for dpu level, fetch that DPU.
							// Otherwise, get them all without generating a wrapper DPV node.
							if ($options->getNetworkID() != "") {
								$dpuString = $options->getNetworkID();
								$returnValue = addDPU($xml, $BP, $options, (int)$dpuString);
							} else {
								// if on a vault, get all the DPUs.  If on a DPU, get
								// this (local) DPU only.
								if ($BP->LocalSystemIsVault()) {
									$returnValue = addDPV(true, $xml, $BP, $options, true);
								} else {
									$returnValue = addDPU($xml, $BP, $options, $localID);
								}
							}
							break;
							
		case "client":		// if the networkID is specified for client level, fetch that client.
							// Otherwise get all clients of "this" dpu, which is only valid on a dpu.
							// Set the status type so that we get both backups and vaulting statuses.
							$options->setStatusType('both');
							if ($options->getNetworkID() != "") {
								$networkID = $options->getNetworkID();
								// networkID attr value is <dpuid>.<clientID>
								$pieces = explode('.', $networkID);
								// set DPU based on networkID or local if grand client view.
								$dpuID = $options->grandClientView() ? $localID : (int)$pieces[0];
								$clientID = (int)$pieces[1];
								$dpuInfo = $BP->get_system_info($dpuID);
								$clientInfo = $BP->get_client_info($clientID, $dpuID);	
								if ($clientInfo !== false) {
									addClient($options, $xml, $BP, $dpuInfo, $clientInfo, $networkID);
								} else {
									addBadElement($options, $xml, 'Client', getClientName($BP, $clientID, $dpuID));
								}				
							} else {
								// output all clients of the local DPU.  Can't be from a vault
								// as a DPU would have had to have been specified.
								if (!$BP->local_system_is_mgr()) {
									$dpuInfo = NULL;
									$result = addClients($options, $xml, $BP, $dpuInfo);
								}
							}
	}
	
	$xml->pop(); // root
	
	if ($returnValue != true) {
		$xml->clear(false);
		$BP->buildResult($xml, false);
	}
	
	return($xml->getXml());
}

//
// Add the DPV node to the XML, then all the sub-nodes (location, alert).
// Then get all DPU's for this DPV and output their XML if bRecurse is true.
//
function addDPV($outputDPVInfo, $xml, $BP, $options, $bWarning = false, $bRecurse = true)
{
	
	$returnValue = true;
	
	$dpvs = getDPVs($BP);	
	if (!is_null($dpvs)) {
		foreach ($dpvs as $dpv) {
			//$dpvInfo = $BP->get_system_info($dpv);
			$dpvInfo = array('id' => $dpv['id'], 'name' => $dpv['name']);
			if ($outputDPVInfo && $dpvInfo !== false) {
				$dpvAttributes = array(	'NetworkName' => $dpvInfo['name'],
										'NetworkIdentification' => $dpvInfo['id']);
				if ($bWarning) {
					$dpvAttributes = array_merge($dpvAttributes, array('Warning' => true));
				}
				$xml->push('DPV', $dpvAttributes);
				
				// if .dpu personality exists, skip getting alerts for the vault
				$targetDpuID = getCrossVaultDpuID($BP, $dpvInfo['name']);
				if ($targetDpuID == -1) {
					$result = addAlerts($options, $xml, $BP, $dpvInfo);
					if ($result == false) {
						addBadElement($options, $xml, 'Alert', $dpvInfo['name']);
					}
				}
			} else {
				//$returnValue = false;
				//addBadElement($options, $xml, 'DPV', getSystemName($BP, $dpv), true);
				break;
			}
			
			if ($bRecurse) {
				$dpus = getDPUsForDPV($dpv, $BP);
				$dpuString = $options->getNetworkID();
				foreach ($dpus as $dpu) {
					$dpuID = $dpu['id'];
					$dpuName = $dpu['name'];
					if ($dpuString == "") {
						// none specified. get them all
						$returnValue = addDPU($xml, $BP, $options, $dpuID, $dpuName);
					} else if ((int)$dpuString == $dpu['id']) {
						// one specified, output it and break.
						$returnValue = addDPU($xml, $BP, $options, $dpuID, $dpuName);
						break;
					}
				}
			}
			
			if ($outputDPVInfo) {
				$xml->pop(); // DPV
			}
		}
	} else {
		// no DPVs found.
	}
	
	return $returnValue;
}

//
// Add the DPV node to the XML, then all the sub-nodes (location, alert).
// Then get all DPU's for this DPV and output their XML.
//
function addmDPU($xml, $BP, $options, $bWarning = false, $bRecurse = true)
{
	global $localID;
	$returnValue = true;
		
	$systems = $BP->get_system_list();
	foreach ($systems as $id => $name) {
		if ($id == $localID) {			
			//
			// add this system info.
			//
			$xml->push("MDPU", array('NetworkName' => $name, 'NetworkIdentification' => -1));
			if ($bRecurse) {
				//
				// loop through all the other systems, output their xml info.
				//
				$dpus = $BP->get_system_list();
				foreach ($dpus as $dpuID => $dpuName) {
					$returnValue = addDPU($xml, $BP, $options, $dpuID, $dpuName);
				}
			}
			$xml->pop(); // managed DPU
			break;
		}
	}
	
	return $returnValue;
}

//
// Add the DPU node to the XML, then all the sub-nodes (location, alerts, backups).
// Then get all clients for this DPU and output their XML.
//
function addDPU($xml, $BP, $options, $dpuID, $dpuName = NULL)
{

	$returnValue = true;
	$dpuInfo = array();
	if ($dpuName == NULL) {
		$dpuInfo = $BP->get_system_info($dpuID);
	} else {
		$dpuInfo['id'] = $dpuID;
		$dpuInfo['name'] = $dpuName;
	}

	if ($dpuInfo !== false) {	
		$xml->push("DPU", array('NetworkName' => $dpuInfo['name'], 'NetworkIdentification' => $dpuInfo['id']));
		
		// Add status information for this system.  If errors, add "BAD" elements and keep going.
		$result = addAlerts($options, $xml, $BP, $dpuInfo);
		if ($result == false) {
			addBadElement($options, $xml, 'Alert', $dpuInfo['name']);
		}

		// initialize status lists.
		$BackupsList = NULL;
		$SyncList = NULL;
		// Make these static so re-usable if in grand client view.
		static $BackupsSummary = NULL;
		static $SyncSummary = NULL;
		// If not in grand client view, re-set summaries to NULL.
		if (!$options->grandClientView()) {
			$BackupsSummary = NULL;
			$SyncSummary = NULL;
		}
		
		$backupTotals = 0;
		$sType = $options->getStatusType();
		if ($sType == "") {
			addSummaries($options, $xml, $BP, BACKUP_XML_TAG, $dpuInfo, NULL, $BackupsSummary, $backupTotals);
		} else if ($sType == 'backup' || $sType == 'both') {
			$result = addBackups($options, $xml, $BP, BACKUP_XML_TAG, $dpuInfo, NULL, $BackupsList, $backupTotals);
			if ($result == false) {
				addBadElement($options, $xml, BACKUP_XML_TAG, $dpuInfo['name']);
			}
		}
			
		if ($sType == "") {
			addSummaries($options, $xml, $BP, SYNC_XML_TAG, $dpuInfo, NULL, $SyncSummary, $backupTotals);
		} else if ($sType == 'sync' || $sType == 'both') {
			$result = addBackups($options, $xml, $BP, SYNC_XML_TAG, $dpuInfo, NULL, $SyncList, $backupTotals);
			if ($result == false) {
				addBadElement($options, $xml, SYNC_XML_TAG, $dpuInfo['name']);
			}
		}
		
		addClients($options, $xml, $BP, $dpuInfo, $BackupsList, $SyncList, $BackupsSummary, $SyncSummary, $backupTotals);
		
		$xml->pop(); // DPU
		
	} else {
		addBadElement($options, $xml, 'DPU', getSystemName($BP, $dpuID));
	}
	
	return $returnValue;
}

function addBadElement($options, $xml, $stringType, $stringName = "unknown", $bPop = true) {
	// Since we are not erroring out, log in the error log.
	// The CMC will pick up on the unknown element to warn the user.
	global $Log;
	global $BP;
	$message = $BP->getError() . " (element type = " . $stringType . ", name = " . $stringName . ")";
	$Log->writeError($message, true);
	$xml->push($stringType, array('NetworkName' => $stringName,
								  'Identification' => "unknown",
								  'ID' => "unknown",
								  'NetworkIdentification' => "unknown"));
	
	if ($bPop) {
		$xml->pop(); // bad
	}
}

//
// Add the list of clients for a particular DPU to the XML.
//
function addClients($options, $xml, $BP, $dpuInfo, $BackupsList, $SyncList, $BackupsSummary, $SyncSummary, $totals)
{
	global $localID;
	
	// By default, getting  local system clients, will be changed if getting remote clients.
	$systemID = $localID;
	// Get the clients for this system either the one specified or local one.
	if ($dpuInfo !== NULL) {
		// See if we want grand Clients or local clients.
		if ($options->grandClientView()) {
			$clientList = $BP->get_grandclient_list($dpuInfo['id']);
		} else {
			// Get remote clients.
			$systemID = $dpuInfo['id'];
			$clientList = $BP->get_client_list($systemID);
		}
	} else {
		$clientList = $BP->get_client_list();
	}
	
	$returnValue = true;
	if ($clientList !== false) {
		foreach ($clientList as $id => $name) {
			// Generate a "unique" name for the client attribute, combining the DPU ID and client ID.
			$networkID = sprintf("%d.%d", $dpuInfo['id'], $id);
			$clientInfo = $BP->get_client_info($id, $systemID);
			if ($clientInfo === false) {
				$clientInfo = array('id' => $id, 'name' => $name);
			}
			addClient($options, $xml, $BP, $dpuInfo, $clientInfo, $networkID, $BackupsList, $SyncList, $BackupsSummary, $SyncSummary, $totals);
		}
	} else {
		$returnValue = $clientList;
		addBadElement($options, $xml, 'Client', '(no clients available)');
	}
	return $returnValue;
}


//
// Add the Client node to the XML.
//
function addClient($options, $xml, $BP, $dpuInfo, $clientInfo, $networkID,
				   $BackupsList = NULL, $SyncList = NULL, $BackupsSummary = NULL, $SyncSummary = NULL, $totals = -1) {
	
	$tmp = explode(".", $networkID);
	$dpuID = $tmp[0];
	
	// Build the Client XML node.  Note that Clients do not have alerts, unlike DPUs and DPVs.
	createClientXML($xml, $clientInfo, $clientInfo['id'], $networkID, false, NULL, $dpuID);

	// add Client backups
	$sType = $options->getStatusType();
	if ($sType == "") {
		addSummaries($options, $xml, $BP, BACKUP_XML_TAG, $dpuInfo, $clientInfo, $BackupsSummary, $totals);
	} else if ($sType == 'backup' || $sType == 'both') {
		$result = addBackups($options, $xml, $BP, BACKUP_XML_TAG, $dpuInfo, $clientInfo, $BackupsList, $totals); 
		if ($result == false) {
			addBadElement($options, $xml, BACKUP_XML_TAG, $clientInfo['name']);
		}
	}
	
	if ($sType == "") {
		addSummaries($options, $xml, $BP, SYNC_XML_TAG, $dpuInfo, $clientInfo, $SyncSummary, $totals);
	} else if ($sType == 'sync' || $sType == 'both') {
		$result = addBackups($options, $xml, $BP, SYNC_XML_TAG, $dpuInfo, $clientInfo, $SyncList, $totals);
		if ($result == false) {
			addBadElement($options, $xml, SYNC_XML_TAG, $clientInfo['name']);
		}
	}
	
	$xml->pop(); // Client
}

//
// Add the Alert nodes to the XML. Note: DPx's have alerts; clients do not.
//
function addAlerts($options, $xml, $BP, $dpuInfo) {

	$returnValue = true;
	if ($options->getStatusType() == "") {
		if ($dpuInfo == NULL) {
			$arrayFilter = addDaysFilter($options);
		} else {
			$arrayFilter = array('dpu' => $dpuInfo['id']);
			$arrayFilter = array_merge($arrayFilter, addDaysFilter($options));
		}
		
		$alerts = $BP->get_alerts("descending", "timestamp", $arrayFilter);
		if ($alerts === false) {
			//turn on stricter checking of alert returns.
			$returnValue = false;
			$alerts = NULL;
		}
		
		if (!is_null($alerts)) {
			foreach ($alerts as $alert) {
				createAlertXML($xml, $alert);
			}
		}
	}
	return $returnValue;
}

//
// Add backup statuses to the XML for each client/DPU
//
function addBackups($options, $xml, $BP, $XMLTag, $dpuInfo = NULL, $clientInfo = NULL, &$statusArray, &$totalStatus)
{
	$returnValue = true;
	$bDPUstatus = false;
	$bValidDPUClientCombination = true;

	if ($dpuInfo == NULL) {
		if ($clientInfo != NULL) {
			// local client backup status (logged in at DPU). No set_cookie needed.
			$result_format = array('client_id' => $clientInfo['id']);
		} else {
			// backups for this DPU; only vaulted view, so no backups will be returned.
			$bValidDPUClientCombination = false;
		}
	} else {
		// See if we want grand client or local client backups.
		if (!$options->grandClientView()) {
			$result_format = array('system_id' => $dpuInfo['id']);
		} else {
			$result_format = array();
		}
		
		if ($clientInfo != NULL) {
			// vaulted client backup status (logged in at vault).
			$result_format['client_id'] = $clientInfo['id'];
		} else {
			// vaulted DPU backup status (logged in at vault).
			$bDPUstatus = true;
		}
	}
	
	if ($bValidDPUClientCombination) { //$totalStatus < 0) {
		// indicated whether to pass in start_time/end_time to bp_get_backup_status or finish_interval_start/finish_interval_end
		$getFinishTimeInterval = false;
			
		// pass in backup types.
		if ($XMLTag == BACKUP_XML_TAG) {
			$specifiedTypes = $options->getTypes();
			//var_dump($specifiedTypes);
			if (count($specifiedTypes) > 0) {
				$backupTypes = $specifiedTypes;
			} else {
				global $GLOBAL_BACKUP_TYPES;
				$backupTypes = $GLOBAL_BACKUP_TYPES;
			}
		} else {
			// use finish_interval_start/finish_interval_end for replication history
			$getFinishTimeInterval = true;
			// an SDS backup -- change to return all SDS backup types.
			global $GLOBAL_SS_TYPES;
			$backupTypes = $GLOBAL_SS_TYPES;
		}
		// # of days
		$result_format = array_merge($result_format, addDaysFilter($options, $clientInfo, $getFinishTimeInterval));
		$result_format = array_merge($result_format, array('type' => $backupTypes));
	
		// Get the backups either from an existing backup status array, or call the API to get them.
		if ($statusArray === NULL) {
			$backups = $BP->get_backup_status($result_format);
			$statusArray = $backups;
			if ($backups === false) { 
				//turn on stricter checking of backup returns.
				$returnValue = false;
				$alerts = NULL;
				$backups = NULL;
				$statusArray = NULL;
			}
		} else {
			// Will only call this is statusArray passed in ... and clientInfo is not NULL (based on parameters).
			$backups = array();
			foreach ($statusArray as $backupStatus) {
				if (array_key_exists('client_id', $backupStatus)) {
					if ($backupStatus['client_id'] === $clientInfo['id']) {
						$backups[] = $backupStatus;
					}
				}
			}
		}
	} else {
		$backups = NULL;
	}
	
	if (!is_null($backups) && $clientInfo != NULL) { //  only output status for clients.
		foreach ($backups as $backup) {
			// Create backup status XML output, passing in the XML tag to use, the backup,
			// not to report if synth capable, and  to only include replicated system metadata backups.
			createBackupStatusXML($xml, $XMLTag, $backup, false, true);
		}
	}
	return $returnValue;
}

function addSummaries($options, $xml, $BP, $XMLTag, $dpuInfo, $clientInfo, &$summaryArray, &$totalStatus) {
	
	if ($clientInfo == NULL) {
		if (!$options->grandClientView() ||
			($options->grandClientView() && $summaryArray == NULL)) {
			$summaryArray = getStatusSummary($options, $BP, $XMLTag, $dpuInfo, $clientInfo);
		}
		if ($XMLTag == BACKUP_XML_TAG) {
			$totalStatus = computeSummaryTotals($summaryArray);
		}
	} else {
		$intervals = $summaryArray['intervals'];
		$summaryArray = $summaryArray['summary'];
		if (is_array($summaryArray) && $totalStatus != -1) {
			$i = 0;
			foreach ($summaryArray as $summary) {
				$tag = $XMLTag . 'Summary';
				$xml->push($tag, array('Time' => $intervals[$i]['start_time'],
									   'Date' => date('M d Y', $intervals[$i]['start_time'])));
				if (is_array($summary)) {
					foreach ($summary as $client) {
						if ($client['client_id'] == $clientInfo['id']) {
							createSummaryXML($xml, $client);
							break;
						}
					}
				}
				$i++;
				$xml->pop(); // Summary
			}
		}
	}
}

//
// returns the output of bp_get_backup_summary for the specified days and intervals.
//
function getStatusSummary($options, $BP, $XMLTag, $dpuInfo, $clientInfo) {

	$result_format = array();
	//
	// Set dpuID based on whether or not we want grandclient view.
	// IF grand client view, use local system.
	//
	if ($options->grandClientView()) {
		global $localID;
		$dpuID = $localID;
	} else {
		if ($dpuInfo != NULL) {
			$dpuID = $dpuInfo['id'];
		}
	}
	if ($clientInfo != NULL) {
		$result_format['client_id'] = $clientInfo['id'];
	} else {
		$result_format['grandclients'] = $options->grandClientView();
	}
	$result_format['sync_types'] = ($XMLTag == BACKUP_XML_TAG) ? false : true;

	$days = (int)$options->getDays();
	$intervals = generateIntervals($days, time());
	$result_format['intervals'] = $intervals;

	$summary = $BP->get_backup_summary($result_format, $dpuID);
	if ($summary === false) {
		global $Log;
		$Log->writeError('cannot load backup summary', true);
		$summary = NULL;
	}
	$result = array();
	$result['summary' ] = $summary;
	$result['intervals'] = $intervals;
	return $result;
}

function computeSummaryTotals($summaryStatus) {
	$totals = 0;
	foreach ($summaryStatus as $status) {
		if (is_array($status)) {
			foreach ($status as $client) {
				$totals += isset($client['successes']) ? $client['successes'] : 0;
				$totals += isset($client['failures']) ? $client['failures'] : 0;
				$totals += isset($client['warnings']) ? $client['warnings'] : 0;
			}
		}
	}
	return $totals;
}

function createSummaryXML($xml, $client) {
	if (isset($client['successes'])) {
		$xml->element('Successes', $client['successes']);
	}
	if (isset($client['warnings'])) {
		$xml->element('Warnings', $client['warnings']);
	}
	if (isset($client['failures'])) {
		$xml->element('Failures', $client['failures']);
	}
	if (isset($client['in_progress'])) {
		$xml->element('InProgress', $client['in_progress']);
	}
	if (isset($client['verify_successes'])) {
		$xml->element('VerifySuccesses', $client['verify_successes']);
	}
	if (isset($client['verify_failures'])) {
		$xml->element('VerifyFailures', $client['verify_failures']);
	}
	if (isset($client['verify_in_progress'])) {
		$xml->element('VerifyInProgress', $client['verify_in_progress']);
	}
}

//
// Generates the intervals to be passed to bp_get_backup_summary, given a number of days.
//
function generateIntervals($totalDays, $endTime) {
	$intervals = array();
	for ($i = 0, $days = $totalDays - 1; $i < $totalDays; $i++, $days--) {
		$seconds = 60 * 60 * 24 * $days;
		$start = strtotime(date("Y-m-d", $endTime - $seconds));
		$intervals[$i] = array("start_time" => $start);
		if ($i != 0) {
			$intervals[$i - 1]["end_time"] = $start - 1;
		}
	}
	$intervals[$totalDays - 1]["end_time"] = $endTime;
	return($intervals);
}


function addDaysFilter($options, $clientInfo = NULL, $getFinishTimeInterval = false)
{
	// see if the user specified start and end time.
	if ($options->getStartTime() != -1) {
		$startTime = (int)$options->getStartTime();
		if ($options->getEndTime() != -1) {
			$endTime = (int)$options->getEndTime();
		} else {
			$endTime = time();
		}
		
	} else {
	
		$days = (int)$options->getDays();
		// default -1, use defaults (week for DPUs, month for clients)
		if ($days == -1) {
			if ($clientInfo == NULL) {
				// 6 days before today gives us 1 week of data.
				$days = 6;
			} else {
				$days = 30;
			}
		} else {
			// subtract 1 day since we are going back in time (inclusive).
			// we will also go back to midnight (below).
			$days = $days - 1;
		}
			
		$endTime = time();
		// convert to start and end times (days * 24 hr/day * 60 min/hr * 60 sec/min)
		$startTime = $endTime - ($days * 24 * 60 * 60);
		// adjust the start time back to midnight of x days ago.
		$dayOffset = date('H:i:s', $startTime);
		$pieces = explode(':', $dayOffset);
		$hour = (int)$pieces[0];
		$minute = (int)$pieces[1];
		$sec = (int)$pieces[2];
		$startTime -= (($hour * 60 * 60) + ($minute * 60) + $sec);
	}
	
    $startTimeKey;
    $endTimeKey;
        
    if ($getFinishTimeInterval !== true) {
        $startTimeKey = 'start_time';
        $endTimeKey = 'end_time';
    }
    else {
        $startTimeKey = 'finish_interval_start';
        $endTimeKey = 'finish_interval_end';
    }

	//printf("end time is %d and start time is %d<br/>", $endTime, $startTime);
	$filterArray = array($startTimeKey => $startTime, $endTimeKey => $endTime);
	return $filterArray;
}

function addLocation($options, $xml, $BP, $systemInfo)
{
	// Set to Default Location if blank.
	$locationName = array_key_exists('location_name', $systemInfo) ? $systemInfo['location_name'] : "Default Location";
	$locationID = array_key_exists('location_id', $systemInfo) ? $systemInfo['location_id'] : "";

	$latitude = array_key_exists('latitude', $systemInfo) ? $systemInfo['latitude'] : "";
	$longitude = array_key_exists('longitude', $systemInfo) ? $systemInfo['longitude'] : "";
	
	$xml->push('Location');
	$xml->element('Where', $locationName);
	$xml->element('Latitude', $latitude);
	$xml->element('Longitude', $longitude);
	$xml->pop();
		
}

function getDPVs($BP)
{
	global $localID;
	$DPVs = array();

	$i = 0;
	// if the local system is a vault, look for it's ID.
	// This supports an eventuality of multiple DPV's.
	if ($BP->LocalSystemIsVault()) {
		$systems = $BP->get_system_list();
		foreach ($systems as $id => $name) {
			// only 1 DPV for Release 3.0, find it and break.
			if ($localID == $id) {
				$DPVs[$i]['id'] = $id;
				$DPVs[$i]['name'] = $name;
				$i++;
				break;
			}
		}
	}
	return $DPVs;
}

function getDPUsForDPV($dpv, $BP) {
	global $localID;
	
	$DPUs = array();
	$i = 0;
	// if the local system is not a vault, we don't have DPU's under a DPV.
	if ($BP->LocalSystemIsVault()) {
	
		$systems = $BP->get_system_list();
		foreach ($systems as $id => $name) {
			// The DPU's are everything but the local vault.
			if ($localID != $id) {
				$DPUs[$i]['id'] = $id;
				$DPUs[$i]['name'] = $name;
				$i++;
			}
		}
	}
	return $DPUs;
}

?>
