<?php

// class to wrap backup/schedule list functions.
class Credentials
{
	function __construct($credentials = NULL) {
		if ($credentials != NULL) {
			$_username = $credentials['username'];
		}
	}
	
	//
	// This function returns credentials list.
	//
	function getList($BP, $xml, $dpuID, $showRoot = false) { // : XML
		$result = $BP->get_credentials_list($dpuID);
		
		if ($result !== false) {
			if ($showRoot) $xml->push('root');
			$xml->push('Credentials');
			foreach ($result as $credentials) {
				$this->buildXML($xml, $credentials, 'Credential');
			}
			$xml->pop(); // Credentials
			if ($showRoot) $xml->pop(); // root
		} else {
			$BP->buildResult($xml, $result);
		}
	}
	
	
	//
	// This function returns credentials list.
	//
	function getNamedCredentials($BP, $xml, $dpuID, $showRoot = false) { // : XML
		$result = $BP->get_named_credentials($dpuID);
		
		if ($result !== false) {
			if ($showRoot) $xml->push('root');
			$xml->push('Credentials');
			foreach ($result as $credentials) {
				$this->buildXML($xml, $credentials, 'Credential');
			}
			$xml->pop(); // Credentials
			if ($showRoot) $xml->pop(); // root
		} else {
			$BP->buildResult($xml, $result);
		}
	}
	
	//
	// Returns XML of the specified credential ID, optionally surrounding the output with a 'root' tag.
	//
	function get($BP, $xml, $cred_id, $dpuID, $showRoot = false) { // : XML
		$result = $BP->get_credentials($cred_id, $dpuID);	
		if ($result !== false) {
			$this->buildXML($xml, $result, $showRoot == true ? 'root' : '');
		} else {
			$BP->buildResult($xml, $result);
		}
	}
	
	//
	// Returns XML of the default credentials. optionally surrounding the output with a 'root' tag.
	//
	function getDefault($BP, $xml, $dpuID, $showRoot = false) { // : XML
		$result = $BP->get_default_credentials($dpuID);	
		if ($result !== false) {
			$this->buildXML($xml, $result, $showRoot == true ? 'root' : '');
		} else {
			$BP->buildResult($xml, $result);
		}
	}

	//
	// This function saves the specified credentials.
	//
	function save($BP, $dpuID, $cred_id, $username, $password, $domain, $default, $display_name = "") // : ID
	{
		$credentials = array();
		if ($cred_id != -1) {
			$credentials['credential_id'] = $cred_id;
		}			
		if ($username != "") {
			$credentials['username'] = $username;
		}
		if ($password != "") {
			$credentials['password'] = $password;
		}
		if ($domain != "") {
			$credentials['domain'] = $domain;
		}
		if ($display_name != "") {
			$credentials['display_name'] = $display_name;
		}

		$credentials['is_default'] = $default;
		
		$result = $BP->save_credentials($credentials, $dpuID);
		return $result;
	}
	
	//
	// Delete the specified credential, and build XML of the result.
	//
	function delete($BP, $xml, $cred_id, $dpuID, $showRoot = false) { // : XML
		$result = $BP->delete_credentials($cred_id, $dpuID);	
		$BP->buildResult($xml, $result);
	}
	
	//
	// Builds an XML structure for the credentials.
	//
	function buildXML($xml, $credentials, $tag = "") {
		
		if ($tag != "") $xml->push($tag);
		
		if (isset($credentials['credential_id'])) {
			$xml->element("ID", $credentials['credential_id']);
		}
		if (isset($credentials['username'])) {
			$xml->element("Username", $credentials['username']);
		}
		if (isset($credentials['password'])) {
			$xml->element("Password", $credentials['password']);
		}
		if (isset($credentials['domain'])) {
			$xml->element("Domain", $credentials['domain']);
		}
		if (isset($credentials['is_default'])) {
			$xml->element("Default", $credentials['is_default'] == true ? "1" : "0");
		}
		if (isset($credentials['display_name'])) {
			$xml->element("Name", $credentials['display_name']);
		}
		
		if ( isset($credentials['clients'])       || 
			 isset($credentials['instances'])     || 
			 isset($credentials['applications'] ) ||
			 isset($credentials['psa'] )             ) {
			 				
			// build associations
			$xml->push('Associations');
		
			// list of clients
			if (isset($credentials['clients'])) {
				$xml->push('Clients');
				$clients = $credentials['clients'];
				foreach ($clients as $client) {
					$xml->push('Client');
					if (isset($client['client_id'])) {
						$xml->element("ClientID", $client['client_id']);
					}
					if (isset($client['client_name'])) {
						$xml->element("ClientName", $client['client_name']);
					}
					$xml->pop(); // Client					
				}
				$xml->pop(); // Clients
			}
			// list of instances
			if (isset($credentials['instances'])) {
				$xml->push('Instances');
				$instances = $credentials['instances'];
				foreach ($instances as $instance) {
					$xml->push('Instance');
					if (isset($instance['instance_id'])) {
						$xml->element("InstanceID", $instance['instance_id']);
					}
					if (isset($instance['app_id'])) {
						$xml->element("AppID", $instance['app_id']);
					}
					if (isset($instance['app_name'])) {
						$xml->element("AppName", $instance['app_name']);
					}
					if (isset($instance['instance_name'])) {
						$xml->element("InstanceName", $instance['instance_name']);
					}
					$xml->pop(); // Instance
				}
				$xml->pop(); // Instances
			}
			// list of applications
			if (isset($credentials['applications'])) {
				$xml->push('Applications');
				$applications = $credentials['applications'];
				foreach ($applications as $app) {
					$xml->push('Application');
					if (isset($app['app_id'])) {
						$xml->element("AppID", $app['app_id']);
					}
					if (isset($app['app_name'])) {
						$xml->element("AppName", $app['app_name']);
					}
					$xml->pop(); // Application								
				}
				$xml->pop(); // Applications
			}
			
			// list of PSA Tools
			if (isset($credentials['psa'])) {
				$xml->push('PSATools');
				$psaTools = $credentials['psa'];
				foreach ($psaTools as $tool) {
					$xml->push('PSATool');
					if (isset($tool['psa_id'])) {
						$xml->element("PSAID", $tool['psa_id']);
					}
					if (isset($tool['psa_tool_name'])) {
						$xml->element("PSAName", $tool['psa_tool_name']);
					}
					$xml->pop(); // PSA Tool								
				}
				$xml->pop(); // PSA Tools
			}
			
			$xml->pop(); // Associations
		}
		
		if ($tag != "") $xml->pop(); // root
	}
}
?>