<?php
$options = getopt("c");
if (isset($options['c'])) {
	$_REQUEST['auth'] = "1:";
	$_SERVER['REQUEST_URI'] = 'command line';
}
// defines BP, xml, action, cookie
require 'header.php';

// include function to parse license string
require 'systeminfo.php';

define("GIGABYTE", 1073741824);
define("VAULTING_DPU", "DPU Configured for Vaulting");

$localID = $BP->get_local_system_id();
$systems = $BP->get_system_list();

global $Log;

if ($systems !== false) {
	$vaultName = isset($_GET['vault']) ? $_GET['vault'] : $systems[$localID];
	$xml->push('root');
	
	$ApplianceType = "unknown";
	$LicensedCapacity = "unknown";
	$Clients = "unknown";
	$nRC = $nVC = 0;
	$BP->bypass_cookie(2);
	$licenseInfo = $BP->get_license_info();
	if ($licenseInfo !== false) {
		if (array_key_exists('feature_string', $licenseInfo)) {
			parseLicenseString($licenseInfo['feature_string'], $ApplianceType, $LicensedCapacity, $Clients, $nRC, $nVC);
		} else {
			if (array_key_exists('license_type', $licenseInfo) && $licenseInfo['license_type'] == "DEMO") {
				$nVC = "N/A (Demo License)";
			}
		}
	} else {
		$message = "Error retrieving license information.";
		$message = $BP->getError() . ":" . $message;
		$Log->writeError($message, true);
		$nVC = "Error reading license";
	}
	$BP->set_cookie($BP->getCookie(), true);
	
	$xml->push("DPV", array('Name' => $vaultName, "Available" => $nVC));

	//
	// Go through each system in the systems table.  If its the vault (local), skip it, otherwise
	// display the size (in GB) used on the vault.
	//
	foreach ($systems as $dpuID => $name) {
		if ($dpuID == $localID) {
			continue;
		}
		// Make sure that the system is vaulting to this vault.  Otherwise, skip it.
		// First, see if we can read the vault Name from the securesync config on the DPU.
		// If we cannot, fall back to the DPU name.
		// See if DPU is configured for vaulting.
		$info = $BP->get_system_info($dpuID);
		if ($info !== false && isset($info['role']) && $info['role'] != VAULTING_DPU) {
			continue;
		}
		
		$result = $BP->get_securesync_config($dpuID);
		if ($result !== false) {
			$settings = $result;
			
			//get clients
			if (array_key_exists('LocalDirToSync', $settings)){
				$localDirToSync = $settings['LocalDirToSync'];
			}
			// Make sure that the system is vaulting to this vault.  Otherwise, skip it.
			if (array_key_exists('SyncTo', $settings)) {
				$VaultSyncTo = $settings['SyncTo'];
				// If no vault specified for the DPU, skip it.
				if ($VaultSyncTo === "") {
					continue;
				}
				// See if the DPU is vaulting to this Vault.
				$bFoundName = matchVaultName($VaultSyncTo, $vaultName, $dpuID);
				if (!$bFoundName) {
					$Log->writeError("Vault \"$VaultSyncTo\" not found for dpu \"$name\", skipping.", false);
					continue;
				}
			}
			
			//
			// Use SyncUser to tell us where the directory is that the vaulted data is stored.
			//
			if (array_key_exists('SyncUser', $settings)) {
				$dpuUser = $settings['SyncUser'];
				$result = getVaultSpaceUsed($dpuUser);
				$VaultBytes = $result['VaultBytes'];
				$GBStr = toNumberString($VaultBytes / GIGABYTE);
				$TempBytes = $result['TempBytes'];
				$GBTempStr = toNumberString($TempBytes / GIGABYTE);
				
				$xml->push("DPU");
				$xml->element("Name", $name);
				$xml->element("Used", $GBStr);
				$xml->element("Temp", $GBTempStr);
				$xml->element("Accessible", "Yes");

				$bGetClients = isset($_GET['bgetclients']);
				
				if($bGetClients) {
				//get the clients for this dpu
					$clientSpaceUsed = getClientSpaceUsed($BP, $xml, $dpuUser);	
				}
				$xml->pop(); //dpu

			}
		} else {
			// We cannot access the DPU, try to get the home directory on the DPV.
			$Log->writeError($BP->getError(), true);
			$dpuUser = $name;
			$homeDirectory = getHomeDirectory($dpuUser);
			if ($homeDirectory != NULL) {
				$Log->writeError("Cannot contact DPU; using home directory of DPU, $homeDirectory...", false);
			} else {
				$Log->writeError("Cannot contact DPU nor determine home directory, trying DPU name, $dpuUser...", false);
			}
			$result = getVaultSpaceUsed($dpuUser, $homeDirectory);
			$VaultBytes = $result['VaultBytes'];
			$GBStr = toNumberString($VaultBytes / GIGABYTE);
			$TempBytes = $result['TempBytes'];
			$GBTempStr = toNumberString($TempBytes / GIGABYTE);
			
			$xml->push("DPU");
			$xml->element("Name", $name);
			$xml->element("Used", $GBStr);
			$xml->element("Temp", $GBTempStr);
			$xml->element("Accessible", "No");
			
			$bGetClients = isset($_GET['bgetclients']);	
			if($bGetClients) {
				$clientSpaceUsed = getClientSpaceUsed($BP, $xml, $dpuUser);	
			}
			$xml->pop(); //dpu
		}
	}

	$xml->pop();	// DPV
	$xml->pop();	// root
} else {
	$BP->buildResult($xml, false);
}
echo($xml->getXml());


//
// Returns true if we found this vault's name for the DPU or an alias; false otherwise.
// Make matches case in-sensitive.
//
function matchVaultName($SyncToVault, $ThisVault, $dpuID)
{
	global $BP;
	global $Log;
	$bMatch = false;
	
	if (strtolower($SyncToVault) == strtolower($ThisVault)) {
		$bMatch = true;
	} else {
		// See if SyncTo matches an alias of the vault's hostname.
		$hostInfo = $BP->get_host_info($ThisVault);
		if ($hostInfo !== false) {
			if (array_key_exists('aliases', $hostInfo)) {
				$aliases = $hostInfo['aliases'];
				foreach ($aliases as $alias) {
					if ((strtolower($alias) == strtolower($SyncToVault)) ||
						(strtolower($alias) == strtolower($ThisVault))) {
						$bMatch = true;
						break;
					}
				}
			}
		}
		if (!$bMatch) {
			// No match, so see if a host entry for this Vault is on the DPU.  Try all variations.
			$hostInfo = $BP->get_host_info($ThisVault, $dpuID);
			if ($hostInfo === false) {
				$Log->writeError("No host entry found for name \"$ThisVault\", trying lower case....", false);
				$hostInfo = $BP->get_host_info(strtolower($ThisVault), $dpuID);
			}
			if ($hostInfo === false) {
				$Log->writeError("No host entry found for lower-case name \"$ThisVault\", trying SyncTo vault name....", false);
				$hostInfo = $BP->get_host_info($SyncToVault, $dpuID);
			}
			if ($hostInfo === false) {
				$Log->writeError("No host entry found for name \"$SyncToVault\", trying lower case....", false);
				$hostInfo = $BP->get_host_info(strtolower($SyncToVault), $dpuID);
			}
			if ($hostInfo !== false) {
				// See if the vault name might match an alias name on the DPU.
				if (array_key_exists('aliases', $hostInfo)) {
					$aliases = $hostInfo['aliases'];
					foreach ($aliases as $alias) {
						if ((strtolower($alias) == strtolower($SyncToVault)) ||
							(strtolower($alias) == strtolower($ThisVault))) {
							$bMatch = true;
							break;
						}
					}
				}
			}
		}
	}
	return $bMatch;
}


//
// Gets the home directory of the DPU user from the passwd file.
//
function getHomeDirectory($dpuUser)
{
	$homeDirectory = NULL;

	// grep the entry from the password file and get the HOME dir.
	$command = sprintf("grep \"%s:\" /etc/passwd | awk -F: '{print $6}'", $dpuUser);
	$handle = popen($command, 'r');
	if ($handle) {
		$homeDirectory = fread($handle, 1024);
		// Remove extra newline, if nothing left, return NULL.
		$homeDirectory = trim($homeDirectory);
		if ($homeDirectory == "") {
			$homeDirectory = NULL;
		}
		pclose($handle);
	}	
	return $homeDirectory;
}

//
// This function returns the amount of space used on the vault for a given DPU, by
// running the 'du' command; the size used is in bytes.  The vaulted data is kept
// on the /backups directory, with the subdirectory of SyncUser and the 'syncd' directory
// under that.
//
function getVaultSpaceUsed($dpuUser, $homeDirectory = NULL)
{
	if ($homeDirectory == NULL) {
		$path = "/backups/" . $dpuUser . "/syncd/";
	} else {
		$path = $homeDirectory . "/syncd/";
	}
	$TotalBytes = -1;
	$VaultBytes = -1;
	$TempBytes = -1;
	
	// Get bytes used in the syncd directory for the DPU.
	$command = sprintf("sudo -S /usr/bin/du -s -b %s", $path);
	$handle = popen($command, 'r');
	if ($handle) {
		$commandReturn = fread($handle, 1024);
		pclose($handle);
	}
	// The value comes back as <size><TAB><directory name>
	$a = explode("\t", $commandReturn);
	if (is_array($a) && count($a) > 0) {
		$TotalBytes = $a[0];
	}
	
	// If we have data used, exclude temp space to get true vaulted space.
	if ($TotalBytes > 0) {
		$command = sprintf("sudo -S /usr/bin/du -s -b --exclude='new-*' --exclude='*.resume' %s", $path);
		$handle = popen($command, 'r');
		if ($handle) {
			$commandReturn = fread($handle, 1024);
			pclose($handle);
		}
		// The value comes back as <size><TAB><directory name>
		$a = explode("\t", $commandReturn);
		if (is_array($a) && count($a) > 0) {
			$VaultBytes = $a[0];
		}
	}
	
	if ($TotalBytes > 0 && $VaultBytes > 0) {
		$TempBytes = $TotalBytes - $VaultBytes;
	}
	
	$result = array("VaultBytes" => $VaultBytes, "TempBytes" => $TempBytes);
	return $result;
}

function getClientSpaceUsed($BP, $xml, $dpuUser){
	global $Log;
	$TotalBytes = -1;
	$dpuName = $BP->get_dpu_name($dpuUser);
	$clientList = $BP->get_dpu_clients($dpuUser, $dpuName);
	
	if($clientList != false) {
		foreach($clientList as $client) {
			$path = "/backups/" . $dpuUser . "/syncd/" . $client;
			$command = sprintf("sudo -S /usr/bin/du -s -b %s", $path);
			$handle = popen($command, 'r');
			if ($handle) {
				$commandReturn = fread($handle, 1024);
				pclose($handle);
			}
			// The value comes back as <size><TAB><directory name>
			$a = explode("\t", $commandReturn);
			if (is_array($a) && count($a) > 0) {
				$TotalBytes = $a[0];
			}
			$clientGB = toNumberString($TotalBytes / GIGABYTE);
						
			$xml->emptyElement("Client", array("Name" => $client,
												"Used" => $clientGB));
		}
	} else {
		$msg = 'cannot get client list for dpu: ' . $BP->getError();
		$Log->writeError($msg);
	}

}
$BP->destroyCookie();
?>