<?php
// defines BP, xml, action, cookie
require 'header.php';

// general backup list operations.
require 'listsbase.php';
$ListBase = new ListBase();

// defines functions for URL parsing.
require('urloptions.php');
global $Log;
$result_format = array();

$startTime = -1;
$endTime = -1;
$days = -1;

$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $BP->get_local_system_id();
$scheduleID = isset($_GET['scheduleID']) ? (int)$_GET['scheduleID'] : "n/a";

if (isset($_REQUEST['days']) || (isset($_REQUEST['start']) && isset($_REQUEST['end']))) {
	$returnValue = true;
	if (isset($_REQUEST['days'])) {
		$days = (int)$_REQUEST['days'];
	} else {
		$startTime = (int)$_REQUEST['start'];
		$endTime = (int)$_REQUEST['end'];
	}
}

// Loop through all systems.  Will be only one in the list for a DPU, multiple systems on a Vault or a Manager.
$systems = $BP->get_system_list();	
$localID = $BP->get_local_system_id();
$bVault = $BP->local_system_is_vault();
$bManager = $BP->local_system_is_mgr();
$bMDPU = isset($_GET['mdpu']) ? true : false;

if ($systems === false) {
	$BP->buildResult($xml, false);
	echo($xml->getXml());
	break;
	} else {	
		$xml->push('root');
		foreach ($systems as $systemID => $systemName) {
			// Either we've specified one dpu ID or all.
			if ($systemID == $dpuID || $dpuID == $localID) {				
				// If this system is a vault and we are looking at this system, skip it and go to the next.
				if ($bVault && ($systemID == $localID)) {
					;
				// If this system is a manager and we are not running on the MDPU node nor are
				// not looking at the selected system, skip it.
				} else if (!$bVault && $bManager && !$bMDPU && ($systemID != $dpuID)) {
					;
				} else {
					$xml->push("DPU", array("Name" => $systemName, "ID" => $systemID));
					$arrayFilter = array('dpu' =>$systemID);
					$arrayFilter = array_merge($arrayFilter, addDaysFilter($days, $startTime, $endTime));
					
					$scheduleHistoryArray = $BP->get_schedule_history($arrayFilter, $systemID);
					
					if ($scheduleHistoryArray === false) {
						$BP->buildResult($xml, false);
					}else{
						if ($systemID != -1) {
							if (count($scheduleHistoryArray) > 0) {
								foreach ($scheduleHistoryArray as $scheduleHistory) {
									global $Log;
									$Log->writeVariable("Schedule History");
									$Log->writeVariable($scheduleHistory);
									
									createScheduleHistoryXML($xml, $scheduleHistory, $startTime, $endTime, $systemID);
								}
							}
						}
					}
					$xml->pop(); //dpu
			}
		}
	}
	$xml->pop(); //root
	echo($xml->getXml());
}


// Creates the XML 
function createScheduleHistoryXML($xml, $scheduleHistory, $startTime, $endTime, $dpuID){

	$startTime = strftime('%m/%d/%Y %r' , $startTime);
	$endTime = strftime('%m/%d/%Y %r' , $endTime);
	
	$xml->push('ScheduleHistory', array("DPU" => $dpuID));
	$xml->element('StartTime', $startTime);
	$xml->element('EndTime', $endTime);
	$xml->element('ScheduleName', $scheduleHistory['schedule_name']);
	$xml->element('Description', $scheduleHistory['schedule_description']);
	$xml->element('AppName', $scheduleHistory['application_name']);
		
	if(array_key_exists('client_name', $scheduleHistory)){
		$xml->element('Client', $scheduleHistory['client_name']);
	}
		
	$xml->element('Failures', $scheduleHistory['failures']);
		
	if (array_key_exists('backups', $scheduleHistory)) {
		$backups = $scheduleHistory['backups'];
			
		foreach ($backups as $runStart => $backupArray) {		
			$runStart = strftime('%m/%d/%Y %r', $runStart);
			$xml->push("Backups", array("Time" => $runStart));	
				
			foreach ($backupArray as $backup) {
				$xml->push("Backup");						
				if (array_key_exists("type", $backup)){
					$xml->element("type", getBackupTypeString($backup['type']));
				}
				if (array_key_exists("backup_id", $backup)){
					$xml->element("backupId", $backup['backup_id']);
				}
				if(array_key_exists("files", $backup)){
					$xml->element("files", $backup['files']);
				}
				if(array_key_exists("size", $backup)){
					$xml->element("size", $backup['size']);
				}
				if (array_key_exists("primary_name", $backup)){
					$name = getPrimaryName($scheduleHistory['application_name']);
					$xml->push("primary_name");
					$xml->element('Name', $name);
					$xml->element('Value', $backup['primary_name']);
					$xml->pop();
				}
				if (array_key_exists("secondary_name", $backup)){
					$name = getSecondaryName($scheduleHistory['application_name']);
					$xml->push("secondary_name");
					$xml->element('Name', $name);
					$xml->element('Value', $backup['secondary_name']);
					$xml->pop();
				}
				if(array_key_exists("verify_status", $backup)){
					$xml->element("verify_status", $backup['verify_status']);
				}
				if(array_key_exists("failed", $backup)){
					$xml->element("failed", $backup['failed']);
				}
				if(array_key_exists("description", $backup)){
					$xml->element("description", $backup['description']);
				}
				$xml->pop(); // Backup
			}
			$xml->pop(); // Backups
		}
	}
			
	$xml->pop(); //scheduleHistory
	
}

//
// returns the primary name given the application name.
//
function getPrimaryName($appName) {
	$s = "";
	if (strstr($appName, 'SQL Server')) {
		$s = 'Instance';
	} else if (strstr($appName, 'VMware')) {
		$s = 'Server';
	} else if (strstr($appName, 'Exchange')) {
		$s = 'Database';
	} else if (strstr($appName, 'Hyper-V')){
		$s = 'VM';
	} else {
		$s = 'Client';
	}
	return $s;
}

//
// returns the secondary name given the application name.
//
function getSecondaryName($appName) {
	$s = "";
	if (strstr($appName, 'SQL Server')) {
		$s = 'Database';
	} else if (strstr($appName, 'VMware')) {
		$s = 'VM';
	} else {
		$s = '';
	}
	return $s;
}

function addDaysFilter($days, $startTime, $endTime)
{
	// see if the user specified start and end time.
	if ($startTime != -1) {
		if ($endTime == -1) {
			$endTime = time();
		}
	} else {
	
		//$days = -1;
		// default -1 (1 week)
		if ($days == -1) {
		// 6 days before today gives us 1 week of data.
			$days = 6;
	
			} else {
			// subtract 1 day since we are going back in time (inclusive).
			// we will also go back to midnight (below).
			$days = $days - 1;
		}
			
		$endTime = time();
		// convert to start and end times (days * 24 hr/day * 60 min/hr * 60 sec/min)
		$startTime = $endTime - ($days * 24 * 60 * 60);
		// adjust the start time back to midnight of x days ago.
		$dayOffset = date('H:i:s', $startTime);
		$pieces = explode(':', $dayOffset);
		$hour = (int)$pieces[0];
		$minute = (int)$pieces[1];
		$sec = (int)$pieces[2];
		$startTime -= (($hour * 60 * 60) + ($minute * 60) + $sec);
	}

	//printf("end time is %d and start time is %d<br/>", $endTime, $startTime);
	$filterArray = array('start_time' => $startTime, 'end_time' => $endTime);
	
	return $filterArray;
}

$BP->destroyCookie();
?>