<?php

// class to wrap general function library
class Library
{
	function __construct()
	{
	}
	
	//
	// This function converts a comma-separated string of numbers to an array of integers.
	//
	function getIDs($ids) {
		$stringArray = split(",", $ids);
		$i = 0;
		foreach ($stringArray as &$id) {
			$idArray[$i++] = (int)$id;
		}
		return $idArray;
	}
	
	//
	// This function converts a comma-separated string of strings to an array of strings.
	//
	function getStringArray($ids) {
		$idArray = split(",", $ids);
		return $idArray;
	}

	//
	// Adds an alias $aliasName to host entry $hostname on the system specified by $dpuID.
	// returns true on success and false on failure.
	//
	function addHostAlias($BP, $hostName, $aliasName, $dpuID) {
		$bValid = true;
		$hostInfo = $BP->get_host_info($aliasName, $dpuID);
		if ($hostInfo !== false) {
			; // the alias name is already present, don't need to add it.
		} else {
			// The alias is not there, add to the host entry for host name.
			$hostInfo = $BP->get_host_info($hostName, $dpuID);
			if ($hostInfo !== false) {
				// Update existing entry, add the alias if not already present.
				$hostInfo['original_ip'] = $hostInfo['ip'];
				$aliases = isset($hostInfo['aliases']) ? $hostInfo['aliases'] : array();
				if (!in_array($aliasName, $aliases)) {
					$aliases[] = $aliasName;
					$hostInfo['aliases'] = $aliases;
					$result = $BP->save_host_info($hostInfo, $dpuID);
					if ($result === false) {
						global $Log;
						$Log->writeError($BP->getError(), true);
					}
					$bValid = $result;
				} else {
					$bValid = true;
				}
			} else {
				$bValid = false;
			}
		}
		return $bValid;
	}

	//
	// Dletes alias $aliasName to host entry $hostname on the system specified by $dpuID.
	// returns true on success and false on failure.
	//
	function deleteHostAlias($BP, $hostName, $aliasName, $dpuID) {
		$bValid = true;
		$hostInfo = $BP->get_host_info($aliasName, $dpuID);
		if ($hostInfo !== false) {
			// The alias is there, remove from host entry for host name.
			// Update existing entry, add the alias if not already present.
			$hostInfo['original_ip'] = $hostInfo['ip'];
			$aliases = isset($hostInfo['aliases']) ? $hostInfo['aliases'] : array();
			if (in_array($aliasName, $aliases)) {
				$newAliases = array();
				foreach ($aliases as $alias) {
					if ($alias != $aliasName) {
						$newAliases[] = $alias;
					}
				}
				$hostInfo['aliases'] = $newAliases;
				$result = $BP->save_host_info($hostInfo, $dpuID);
				if ($result === false) {
					global $Log;
					$Log->writeError($BP->getError(), true);
				}
				$bValid = $result;
			}
		} else {
			global $Log;
			$Log->writeError($BP->getError(), true); 
		}
		return $bValid;
	}
}

?>