<?php
// defines BP, xml, action, cookie
require 'header.php';

// action will be 'vault' for vaulting operations.

$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $BP->get_local_system_id();
		
switch ($action) {				
	case "list":
	case "vault":
		$operationList = array();
		$systems = $BP->get_system_list();
		if ($systems !== false) {
			$localID = $BP->get_local_system_id();
			$bVault = $BP->local_system_is_vault();
			$bMgr = $BP->local_system_is_mgr();
		
			$operationArray = array();
			$i = 0;
			foreach ($systems as $systemID => $systemName) {
				if ($systemID == $dpuID || $dpuID == $localID) {
				
					// If this system is a manager, only look at the specified dpuID.
					if (!$bVault && $bMgr && ($systemID != $dpuID)) {
						continue;
					}
					
					// Skip replication operations when action is "vault"
					if ($action == "vault") {
						$isReplication = $BP->is_replication_configured($systemID);
						if ($isReplication === true) {
							continue;
						}
					}
		
					$operationList = $BP->get_operation_list("", $systemID);
					// filter out any non-sync operations
					$operationList = filterVautingOperations($operationList);
					if ($operationList === false) {
						continue;
					} else {
						foreach ($operationList as $operationInfo) {
							// Make sure we don't duplicate entries, particularly seen on a vault.  Don't have
							// to show "sides" of the operation since on a vault we roll up all statuses anyway.
							$bFound = false;
							foreach ($operationArray as $operationAlreadyFound) {
								if ($operationInfo['name'] == $operationAlreadyFound['name'] &&
									$operationInfo['source'] == $operationAlreadyFound['source'] &&
									$operationInfo['destination'] == $operationAlreadyFound['destination'] &&
									$operationInfo['backup_id'] == $operationAlreadyFound['backup_id'] &&
									$operationInfo['type'] == $operationAlreadyFound['type']) {
									$bFound = true;
									break;
								}
							}
							//
							// If not found already, add to our operations array for later display.
							//
							if (!$bFound) {
								$operationInfo['systemID'] = $systemID;
								$operationInfo['systemName'] = $systemName;
								$operationArray[$i++] = $operationInfo;
							}
						}
					}
				}
			}
			
			$xml->push('root');
			// Display the active operations.
			foreach ($operationArray as $operationInfo) {
				displayOperation($xml, $operationInfo, $operationInfo['systemID'], $operationInfo['systemName']);
			}
			$xml->pop();
			
		} else {
			$BP->buildResult($xml, false);
		}
		echo($xml->getXml());
		break;
		
	case "kill":
		$oid = isset($_GET['oid']) ? (int)$_GET['oid'] : -1;
		$clearSync = isset($_GET['sync']) ? true : false;
		if ($oid != -1) {
			$BP->buildResult($xml, $BP->kill_vaulting_operation($oid, $clearSync, $dpuID));
		} else {
			$stringError = sprintf("kill operation: no operation ID provided");
			$BP->buildResult($xml, false, $stringError);
		}
		echo($xml->getXml());
		break;

}

// Formats operation list XML
function displayOperation($xml, $operationInfo, $systemID, $systemName)
{
	$operationName = (array_key_exists('name', $operationInfo)) ? $operationInfo['name'] : "unknown";
	$operationID = (array_key_exists('id', $operationInfo)) ? $operationInfo['id'] : "unknown";
	$xml->push("Operation",
						array("Name" => $operationName,
							  "ID" => $operationID,
							  "SystemID" => $systemID,
							  "SystemName" => $systemName));
						 
	$xml->element('Name', getFormattedName($operationName));
	
	if (array_key_exists('type', $operationInfo)) {
		$syncType = 'securesync ' . strtolower($operationInfo['type']);
		$xml->element('Type', getBackupTypeString($syncType, true));
	}
	if (array_key_exists('backup_id', $operationInfo)) {
		$xml->element('BackupID', $operationInfo['backup_id']);
	}
	if (array_key_exists('final_size', $operationInfo)) {
		$xml->element('FinalSize', $operationInfo['final_size']);
	}
	if (array_key_exists('current_size', $operationInfo)) {
		$xml->element('CurrentSize', $operationInfo['current_size']);
	}
	if (array_key_exists('start_time', $operationInfo)) {
		$xml->element('StartTime', $operationInfo['start_time']);
		// convert timestamp to local time/day
		$day = date('m/d/Y', $operationInfo['start_time']);
		$time = date(TIME_FORMAT_12HR, $operationInfo['start_time']);
		$xml->element('Date', $day);
		$xml->element('Time', $time);
	}
	if (array_key_exists('current_time', $operationInfo)) {
		$xml->element('CurrentTime', $operationInfo['current_time']);
	}
	if (array_key_exists('on_behalf_of', $operationInfo)) {
		$xml->element('OnBehalfOf', $operationInfo['on_behalf_of']);
	}
	if (array_key_exists('source', $operationInfo)) {
		$xml->element('Source', $operationInfo['source']);
	}
	if (array_key_exists('destination', $operationInfo)) {
		$xml->element('Destination', $operationInfo['destination']);
	}
	$xml->pop(); // operation
}

//
// This function returns a formatted string of the backup type field.
//
function getFormattedName($field, $returnUnformattedName=true) {
	$name = NULL;
	if ($returnUnformattedName == true) {
		$name = $field;
	}
	
	switch ($field) {
		case "delta":
				$name = "Delta";
				break;
		case "signature":
				$name = "Signature";
				break;
		case "patch":
				$name = "Patch";
				break;
		case "copy":
				$name = "Copy";
				break;
	}
	return $name;
}


// takes return from bp_operation_progress and 
// filters out all but vaulting operations
function filterVautingOperations($operationList) {
	$newOperationList = array();
	foreach ($operationList as $key=>$operation) {
		$formattedName = getFormattedName($operation['name'], false);
		// skip any operations that don't have a match
		if ($formattedName!=NULL) {
			$newOperationList[] = $operation;
		}
	}
		
	return $newOperationList;
}

$BP->destroyCookie();
?>