<?php
//
// Define strings to parse command output.
//
define("SYSTEM_INFORMATION", "System Information");
define("PROCESSOR_INFORMATION", "Processor Information");
define("MEMORY_INFORMATION", "Memory Usage");
define("MAX_BACKUPS", "Maximum Backups");

define("DPU", "Unitrends DPU");
define("RECOVERY", "Unitrends Recovery");
define("SFF", "Unitrends Small Form Factor");
define("UEB_HYPERV", "VM-Hyper-V");
define("UEB_VMWARE", "VM-VMware");
define("VERSION", "Version");
define("INSTALL_DATE", "Installed on");
define("OS_KERNEL", "Unitrends' DPU Kernel");
define("ON_METAL", "Unitrends Metal-Platform");

define("UNKNOWN", "unknown");

define("ON_METAL_OS_TYPE", "On Metal OS Type");

//
// Amount of memory to reserve for base functionality when virtualizing clients.
//
// master.ini section/setting that defines the Appliance's Memory Reserve percentage.
define("INI_CMC", "CMC");
define("INI_MEMORY_RESERVE", "MemoryReserve");
// Default reserve percentage if master.ini value not found.
define("DEFAULT_MEMORY_RESERVE", 0.2);
// Minimum Reserve size in MB.
define("MINIMUM_RESERVE", 1024);

function buildSystemInformation($BP, $xml, $bForVirtual = false)
{

	global $Log;
	
	$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $BP->get_local_system_id();
	
	$Name = UNKNOWN;
	$IP = UNKNOWN;
	$Appliance = UNKNOWN;
	$Version = UNKNOWN;
	$Kernel = UNKNOWN;
	$OS = UNKNOWN;
	$InstallDate = UNKNOWN;
	$ApplianceType = "DPU";
	$LicensedCapacity = UNKNOWN;
	$MACAddress = UNKNOWN;
	$Clients = "Unlimited";
	$AssetTag = UNKNOWN;
	$ProcessorType = UNKNOWN;
	$ProcessorCores = 1;
	$ProcessorCache = UNKNOWN;
	$ProcessorFrequency = UNKNOWN;
	$MemorySize = UNKNOWN;
	$MemoryMB = 0;
	
	if (!$bForVirtual) {
		$hostInfo = $BP->get_hostname($dpuID);
		if ($hostInfo !== false) {
			$Name = $hostInfo['name'];
			$hostInfo = $BP->get_host_info($Name, $dpuID);
			if ($hostInfo !== false) {
				$IP = $hostInfo['ip'];
			}
		}

		$commandName = SYSTEM_INFORMATION;
		$commandOutput = $BP->run_command($commandName, "", $dpuID);
		if ($commandOutput !== false) {
			parseDPUCommandOutput($commandOutput, $Appliance, $Version, $Kernel, $InstallDate, $OS);
		} else {
			$message = "Error running System information command for system info.";
			$message = $BP->getError() . ":" . $message;
			$Log->writeError($message, true);
		}
	
		$licenseInfo = $BP->get_license_info($dpuID);
		if ($licenseInfo !== false) {
			if (array_key_exists('feature_string', $licenseInfo)) {
				$nRC = $nVC = 0;
				parseLicenseString($licenseInfo['feature_string'], $ApplianceType, $LicensedCapacity, $Clients, $nRC, $nVC);
				$commandName = MAX_BACKUPS;
				$maxBackups = $BP->run_command($commandName, "", $dpuID);
				if ($maxBackups !== false) {
					$LicensedCapacity = $maxBackups;
				}
			if (array_key_exists('daemon_host_id', $licenseInfo)){
				$MACAddress = $licenseInfo['daemon_host_id'];	
				}
			} else {
				if (array_key_exists('license_type', $licenseInfo) && $licenseInfo['license_type'] == "DEMO") {
					$LicensedCapacity = "N/A (Demo License)";
				}
			}
		} else {
			$message = "Error retrieving license information for system info.";
			$message = $BP->getError() . ":" . $message;
			$Log->writeError($message, true);
		}
	}
	
	$commandName = PROCESSOR_INFORMATION;
	$commandOutput = $BP->run_command($commandName, "", $dpuID);
	$bVirtual = $BP->is_virtual($dpuID);
	if ($commandOutput !== false) {
		parseProcessorCommandOutput($commandOutput, $bVirtual, $ProcessorType, $ProcessorCores, $ProcessorCache, $ProcessorFrequency);
	} else {
		$message = "Error getting processor information for system info.";
		$message = $BP->getError() . ":" . $message;
		$Log->writeError($message, true);
	}
	
	$commandName = MEMORY_INFORMATION;
	$commandOutput = $BP->run_command($commandName, "", $dpuID);
	if ($commandOutput !== false) {
		parseMemoryCommandOutput($commandOutput, $MemorySize, $MemoryMB);
	} else {
		$message = "Error getting memory information for system info.";
		$message = $BP->getError() . ":" . $message;
		$Log->writeError($message, true);
	}
	
	if (!$bForVirtual) {
		$result = $BP->get_asset_tag();
		if ($result !== false) {
			$AssetTag = $result;
		} else {
			$message = "Error retrieving Asset tag for system info.";
			$message = $BP->getError() . ":" . $message;
			$Log->writeError($message, true);
		}
	
		determineOSVersion($OS);
	}
	
	$xml->push('SystemInformation');
	$xml->element('Name', $Name);
	$xml->element('IP', $IP);
	$xml->element('Appliance', $Appliance);
	$xml->element('Version', $Version);
	//$xml->element('Kernel', $Kernel);
	$xml->element('OS', $OS);
	$xml->element('InstallDate', $InstallDate);
	$xml->element('ApplianceType', $ApplianceType);
	$xml->element('LicensedCapacity', $LicensedCapacity);
	$xml->element('MACAddress', $MACAddress);
							
	// Get VF allocated
	$vf = $BP->get_virtual_failover($dpuID);
	if ($vf !== false) {
		$xml->element('VF', $vf['allowed']);
	}
	
	// For virtual clients, calculate reserve.
	if ($bForVirtual) {
		$fReservePercent = 0;
		$result = $BP->get_ini_value(INI_CMC, INI_MEMORY_RESERVE, $dpuID);
		if ($result !== false) {
			$fReservePercent = (float)$result;
		}
		if ($fReservePercent <= 0 || $fReservePercent >= 1) {
			$fReservePercent = DEFAULT_MEMORY_RESERVE;
		}
		// calculate reserve based on percentage
		$Reserve = (int)($MemoryMB * $fReservePercent);
		// Make sure we are over a minimum reserve.
		if ($Reserve < MINIMUM_RESERVE) {
			$Reserve = MINIMUM_RESERVE;
		}
		$xml->element('Reserve', $Reserve);
	}
	
	$xml->element('Clients', $Clients);
	$xml->element('AssetTag', $AssetTag);
	$xml->element('Memory', $MemorySize);
	$xml->push('Processor');
	$xml->element('Type', $ProcessorType);
	$xml->element('Cores', $ProcessorCores);
	$xml->element('Cache', $ProcessorCache);
	$xml->element('Frequency', $ProcessorFrequency);
	$xml->pop(); // Processor
	$xml->pop(); // SystemInformation

}

//
// This function parses the output of the DPU command to get appliance information.
//
// This data is returned in the following format:
//
//	Unitrends DPU Recovery-300 - Thu Jan 15 15:42:59 EST 2009
//	Unitrends' DPU Kernel 2.6.26.3-2.RecoveryOS-smp
//	Version 4.0.2-1.CentOS
//	Installed on Thu 15 Jan 2009 03:44:17 PM EST
//
//
function parseDPUCommandOutput($output, &$Appliance, &$Version, &$Kernel, &$InstallDate, &$CoreOS)
{
	//global $xml;
	$outputArray = split("\n", $output);
	for ($i = 0; $i < count($outputArray); $i++) {
		$line = $outputArray[$i];
		//$xml->element('line', $line);
		if (strstr($line, DPU) && !( strstr( $line, UEB_HYPERV ) ) && !( strstr( $line, UEB_VMWARE ) ) ) {
			$lineArray = split(" ", $line);
			$Appliance = $lineArray[2];
		} else if (strstr($line, RECOVERY)) {
			$lineArray = split(" ", $line);
			$Appliance = $lineArray[1];
		} else if (strstr($line, SFF)) {
			$lineArray = split(" ", $line);
			$Appliance = "SFF";  // was $lineArray[5];
		} else if (strstr($line, UEB_HYPERV)) {
			$Appliance = "Unitrends Enterprise Backup (Hyper-V)";
		} else if (strstr($line, UEB_VMWARE)) {
			$Appliance = "Unitrends Enterprise Backup (VMware)";
		} else if (strstr($line, VERSION)) {
			$lineArray = split(" ", $line);
			$fullVersion = $lineArray[1];
			if ($location = strrpos($fullVersion, '.')) {
				$Version = substr($fullVersion, 0, $location);
                if ( $CoreOS !== ON_METAL_OS_TYPE ) {
                    $CoreOS = substr($fullVersion, $location + 1);
                }
			}
		} else if (strstr($line, INSTALL_DATE)) {
			$location = strlen(INSTALL_DATE) + 1;
			$InstallDate = substr($line, $location);
		} else if (strstr($line, OS_KERNEL)) {
			$location = strlen(OS_KERNEL) + 1;
			$Kernel = substr($line, $location);
		} else if (strstr($line, ON_METAL)) {
            $Appliance = "Unitrends Enterprise Backup Installable Software";
            $CoreOS = ON_METAL_OS_TYPE;
        }
	}
}

//
// This function parses the license string to get the type (DPU or DPV) and licensed capacity.
//
// The license string is formatted as.
//
//
// D2D=1126G,IAB=50,CSS=10,MUX=8,DPL,RC=1126G,VC=0G,RD=7  for a DPU (has RC > 0, VC = 0)
//
// D2D=1126G,IAB=50,CSS=10,MUX=8,DPL,VC=1126G,RC=0,RD=7  for a DPV (has VC > 0, RC = 0)
//
// D2D=1126G,IAB=50,CSS=10,MUX=8,DPL,VC=1126G,RC=500G,RD=7  for a DPU/DPV (has VC > 0, RC > 0)
//
// D2D=INF,IAB=50,CSS=10,MUX=8,DPL,VC=INF,RC=INF  for a DPU/DPV (has VC Infinite, RC Infinite)
//
function parseLicenseString($LicenseString, &$ApplianceType, &$TotalCapacity, &$Clients, &$nRC, &$nVC, &$bG = false)
{
	
	$stringArray = explode(",", $LicenseString);
	$nCapacity = 0;
	
	$hasRC = $hasVC = false;
	for ($i = 0; $i < count($stringArray); $i++) {
		$LicenseElement = $stringArray[$i];
		$tokenArray = explode('=', $LicenseElement);
		if (count($tokenArray) > 1) {
			if ($tokenArray[0] == "RC") {
				$n = formatCapacity($tokenArray[1]);
				if (hasSize($n)) {
					$hasRC = true;
					$nRC = $n;
				}
				$nCapacity += $n;
			} else if ($tokenArray[0] == "VC") {
				$n = formatCapacity($tokenArray[1], $bG);
				if (hasSize($n)) {
					$ApplianceType = "DPV";
					$hasVC = true;
					$nVC = $n;
				}
				$nCapacity += $n;
			} else if ($tokenArray[0] == "CNT") {
				$Clients = $tokenArray[1];
				if ($Clients == '0') {
					$Clients = "None";
				}
			}
		}
	}
	if ($nCapacity > 0) {
		$TotalCapacity = $nCapacity . ' GB';
	}
	
	if ($hasRC && $hasVC) {
		$ApplianceType = "DPU/DPV";
	}
}

//
// Report size if INF or numeric.
//
function hasSize($n) {
	return $n > 0 || strtoupper($n) == "INF";
}

//
// This function takes the capacity string as seen in the license string and formats for user
// consumption.  For example, the size on the license string is "1126G".  This function will
// return integer 1126 in this case.  The default is the same as the license string.
//
function formatCapacity($capacity, &$bG = false)
{
	$nCapacity = 0;
	// Handle infinite
	if (strtoupper($capacity) == "INF") {
		$nCapacity = "INF";
	} else {
		$GBLocation = strpos($capacity, "G");
		if ($GBLocation !== false) {
			$bG = true;
			$nCapacity = (int)substr($capacity, 0, $GBLocation);
		} else {
			$bG = false;
			$strCapacity = "";
			if (strlen($capacity) > 0) {
				$capacity = str_split($capacity);
				for ($i = 0; $i < count($capacity); $i++) {
					if ($capacity[$i] >= '0' && $capacity[$i] <= '9') {
						$strCapacity .= $capacity[$i];
					} else {
						break;
					}
				}
				if ($strCapacity != "") {
					$nCapacity = (int)$strCapacity;
				}
			}
		}
	}
	return $nCapacity;
}


//
// This function parses the output of the Processor info command to get processor information.
//
// This data is returned in the following format:
//
//	model name	: Intel(R) Core(TM)2 Duo CPU     E8400  @ 3.00GHz
//	cpu MHz		: 2997.000
//	cache size	: 6144 KB
//	cpu cores	: 2
//
//
function parseProcessorCommandOutput($output, $bVirtual, &$ProcessorType, &$ProcessorCores, &$ProcessorCache, &$ProcessorFrequency) {
	$outputArray = split("\n", $output);
	$cores = 0;
	for ($i = 0; $i < count($outputArray); $i++) {
		$line = $outputArray[$i];
		$processorArray = explode(':', $line);
		if (count($processorArray) > 1) {
			if (strstr($processorArray[0], "model name")) {
				$ProcessorType = $processorArray[1];
			} else if (strstr($processorArray[0], "cpu cores")) {
				$ProcessorCores = $processorArray[1];
			} else if (strstr($processorArray[0], "cache size")) {
				$ProcessorCache = $processorArray[1];
			} else if (strstr($processorArray[0], "cpu MHz")) {
				$frequency = (float)$processorArray[1] / 1000.0;		// convert MHz to GHz
				$ProcessorFrequency = $frequency . " GHz";
			} else if (strstr($processorArray[0], "processor")) {
				$cores++;
			}
		}
	}
	// Number of processors is total core count.
	$ProcessorCores = $cores;
}


//
// This function parses the output of the Memory info command to get memory size.
//
// This data is returned in the following format.  Mem is physical memory while
// Swap is swap memory.
//
//            	 total       used       free     shared    buffers     cached
//	Mem:          3948       3762        186          0          1       3621
//	-/+ buffers/cache:        138       3810
//	Swap:         3914          0       3914
//
//
function parseMemoryCommandOutput($output, &$MemorySize, &$MemoryMB)
{
	$outputArray = split("\n", $output);
	for ($i = 0; $i < count($outputArray); $i++) {
		$line = $outputArray[$i];
		$processorArray = explode(' ', $line);
		if (count($processorArray) > 2) {
			if (strstr($processorArray[0], "Mem:")) {
				for ($j = 1; $j < count($processorArray); $j++) {
					if ($processorArray[$j] != " " && $processorArray[$j] != "") {
						$MemoryMB = (float)$processorArray[$j]; 			// size in MB.
						$memory = $MemoryMB / 1000.0; 						// convert to GB
						$MemorySize = $memory . " GB";
						break;
					}
				}
			}
		}
	}
}

//
// This function extracts the CentOS version number from the system.
//
// output format:
//
//	RecoveryOS release 5 (Final)
//
function determineOSVersion(&$OS)
{
	ob_start();
    $result = shell_exec('cat /etc/redhat-release');
    if ( $OS === ON_METAL_OS_TYPE ) {
        $OS = $result;
    } else {
        if (!is_null($result)) {
            $resultArray = explode(' ', $result);
            if (count($resultArray) > 2) {
                $OS .=  ' ' . $resultArray[2];
            }
        }
    }
	ob_end_clean();
}
?>
