<?php
// Uses wkhtmltopdf utility for conversion of HTML files to PDF, LGPL3 license

class PDFConversion
{
	
	function __construct()
	{
	}	
	
	// Returns full path to PDF file upon success
	// Returns false upon failure or if PDF is not to be created
	// $systemID - Required. Id of the system.
	// $htmlFilePath - Required. Full path of HTML file.
	// $pdfFile - Required. Name of PDF file, can be full path if $pdfFullPathProvided is passed as true.
	// $getCookie - Optional. Default is false. Must be true if this function is called from a context that does not already have a cookie.
	// $pdfFullPathProvided - Optional. Default is false. If true, $pdfFile must have the full path provided.
	function createPDF($systemID, $htmlFilePath, $pdfFile, $getCookie = false, $pdfFullPathProvided = false) {		
		// create PDF file if necessary
		$result = $this->shouldSendPDFAttachment($systemID, $getCookie);
		if ($result !== false) {
			$result = $this->convertHtmlToPDF($htmlFilePath, $pdfFile, $pdfFullPathProvided);
		}
	
		if ($result !== false) {
			return $result;
		} else {
			return false;
		}
	}
	
	// Returns true if pdf should and can be created, false if it cannot
	// Checks Unattended Backup Information  -> NotificationsSendPDF in master.ini settings
	// Checks whether utility is installed on the system
	// $systemID - Required. Id of the system.
	// $getCookie - Optional. Default is false. Must be true if this function is called from a context that does not already have a cookie.
	function shouldSendPDFAttachment ($systemID, $getCookie = false) {
		$retvalue = false;
		
		if ($getCookie)	 {
			bp_bypass_cookie(3);
		}
		
		$sendPDF = bp_get_ini_value('Unattended Backup Information', 'NotificationsSendPDF', $systemID);
		$retvalue = ($sendPDF == false || (strtolower($sendPDF) == 'no')) ? false : true;
		
		if ($retvalue !== false) {
			$utilExists = $this->doesPDFUtilityExist();
			if ($utilExists !== false) {
				return true;
			}
		}
		
		if ($getCookie) {
			bp_destroy_cookie();
		}
		
		return false;
	}
	
	// Returns full path to pdf filename if pdf was created, false otherwise.
	// $htmlFilename - Required. Full path of HTML file.
	// $pdfFile - Required. Name of PDF file, can be full path if $pdfFullPathProvided is passed as true.
	// $pdfFullPathProvided - Optional. Default is false. If true, $pdfFile must have the full path provided.
	function convertHtmlToPDF($htmlFilePath, $pdfFile, $pdfFullPathProvided = false) {
		$utilExists = $this->doesPDFUtilityExist();
		if ($utilExists === false) {
			return false;	
		}
		
		$outputDir = $this->getDefaultOutputDir();
		if ($pdfFullPathProvided === true) {
			$outputDir = "";
		}
		
		$pdfFilePath = $outputDir . $pdfFile;
		
		// html file must exist
		if (!file_exists($htmlFilePath)) {
			return false;
		}		
		
		$options = "--encoding utf-8 --minimum-font-size 1 --page-size A4 -B 0 -L 0 -R 0 -T 0 --disable-smart-shrinking --dpi 100";
		
		$cmd = "wkhtmltopdf $options $htmlFilePath $pdfFilePath";
		exec($cmd, $output, $return);
		
		if ($return===false || $return===1) {
			return false;
		}
		else {
			return $pdfFilePath;
		}
	}	
	
	function doesPDFUtilityExist() {
		// check to make sure utility exists
		// name of utility is the same for 32 and 64 bit systems
		$pdfUtilPath = '/usr/bin/wkhtmltopdf';
		if (!file_exists($pdfUtilPath)) {
			return false;
		}
		else {
			return true;
		}
	}
	
	// Returns default output directory.
	function getDefaultOutputDir() {
		return "/var/www/html/recoveryconsole/reports.dir/";
	}
}
		
?>