<?php

require_once 'logger.php';

/**
 * @desc - sets up a new feed
 */
function start_feed(&$xml)
{
	logger(4, 'Starting XML feed');
	header("Content-type: text/xml");
	$conn = pg_connect("host=localhost dbname=bpdb user=postgres") or die ("could not connect to database");
	$xml = '<?xml version="1.0" encoding="iso-8859-1"?>'."\n";
	$xml .= '<data version="0.1">'."\n";
	logger(5, 'XML is now ', false);
	logger(5, $xml);
	return true;
}

/**
 * @desc - closes and echoes xml feed string
 */
function finish_feed(&$xml)
{
	logger(4, 'Finishing XML feed');
	$xml .= '</data>';
	logger(5, $xml);
	echo $xml;
	logger(5, 'Feed echoed');
}

/**
 * @desc - transforms pg resultset into xml format, with each row inside
 *         <row> tags and each field inside tags equal to the field name.
 *         All field values are run through htmlspecialchars() for escaping.
 *
 * @param $pg_result - valid postgres result set pointer
 * @param &$xml - xml return value
 * @return - true on success, false on failure;
 */
function query_to_xml($tag, $pg_result, &$xml)
{
	logger(4, 'Converting query to XML');
	$temp = "\t".'<'.$tag.'>'."\n";
	/* loop through every row */
	while (false !== ($row = @pg_fetch_assoc($pg_result))) {
		$temp .= "\t\t".'<row>'."\n";
		foreach ($row as $field => $value) {
			$insert_value = htmlspecialchars($value);
			$temp .= "\t\t\t".'<'.$field.'>'.$insert_value.'</'.$field.'>'."\n";
		}
		$temp .= "\t\t".'</row>'."\n";
	}
	$temp .= "\t".'</'.$tag.'>'."\n";
	logger(5, 'Added xml: ');
	logger(5, $temp);
	$xml .= $temp;
	return true;
}

/**
 * @desc - takes a query as input, processes it, and adds an xml version
 *         of the results set to the $xml output parameter
 *
 * @param $tag - the xml tag that will contain the query in the xml result
 * @param $query - the query string to execute and store in xml
 * @param &$xml - an existing string on which to append the results
 * @return - true on success, false on failure
 */
function add_query($tag, $query, &$xml)
{
	logger(4, 'Adding query to xml using add_query function');
	logger(5, 'Query is: ', false);
	logger(5, $query);
	/* query, ensure results are valid before attempting to convert results */
	$pg_result = @pg_query($query);
	if (($pg_result === null) or ($pg_result === false)) {
		$error = pg_last_error();
		echo $error;
		logger(2, 'query error, returning false: ', false);
		logger(2, $error);
		return false;
	} else {
		$retval = query_to_xml($tag, $pg_result, $xml);
	}
	return $retval;
};
